<?php
/**
* Copyright since 2007 PrestaShop SA and Contributors
* PrestaShop is an International Registered Trademark & Property of PrestaShop SA
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License 3.0 (AFL-3.0)
* that is bundled with this package in the file LICENSE.md.
* It is also available through the world-wide-web at this URL:
* https://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to https://devdocs.prestashop.com/ for more information.
*
* @author    PrestaShop SA and Contributors <contact@prestashop.com>
* @copyright Since 2007 PrestaShop SA and Contributors
* @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
*/

class DfxbannerClass extends ObjectModel
{
    public $title;
    public $description;
    public $url;
    public $image;
    public $image_w;
    public $image_h;
    public $active;
    public $target_url;
    public $position;
    public $hook;
    public $id_shop;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'dfxbanner',
        'primary' => 'id_dfxbanner',
        'multilang' => true,
        'fields' => array(
            'active' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
            'target_url' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'position' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt', 'required' => true),
            'hook' => array('type' => self::TYPE_STRING, 'size' => 50),
            // Lang fields
            'description' => array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 4000),
            'title' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'url' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isUrl', 'required' => true, 'size' => 255),
            'image' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'image_w' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'image_h' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
        )
    );

    public function __construct($id_banner = null, $id_lang = null, $id_shop = null, Context $context = null)
    {
        parent::__construct($id_banner, $id_lang, $id_shop);
    }

    public function add($autodate = true, $null_values = false)
    {
        $context = Context::getContext();
        $id_shop = $context->shop->id;

        $res = parent::add($autodate, $null_values);
        $res &= Db::getInstance()->execute(
            'INSERT INTO `'._DB_PREFIX_.'dfxbanner_shop` (`id_shop`, `id_dfxbanner`)
             VALUES('.(int)$id_shop.', '.(int)$this->id.')'
        );
        return $res;
    }

    public function delete()
    {
        $res = true;

        $images = $this->image;
        foreach ($images as $image) {
            if (preg_match('/banner-img/', $image) === 0) {
                if ($image && file_exists(dirname(__FILE__).'/views/img/'.$image)) {
                    $res &= @unlink(dirname(__FILE__).'/views/img/'.$image);
                }
            }
        }

        $res &= $this->reOrderPositions();

        $res &= Db::getInstance()->execute(
            'DELETE FROM `'._DB_PREFIX_.'dfxbanner_shop`
             WHERE `id_dfxbanner` = '.(int)$this->id
        );

        $res &= parent::delete();
        return $res;
    }

    public function reOrderPositions()
    {
        $id_banner = $this->id;
        $context = Context::getContext();
        $id_shop = $context->shop->id;

        $max = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT MAX(hss.`position`) as position
            FROM `'._DB_PREFIX_.'dfxbanner` hss, `'._DB_PREFIX_.'dfxbanner_shop` hs
            WHERE hss.`id_dfxbanner` = hs.`id_dfxbanner` AND hs.`id_shop` = '.(int)$id_shop
        );

        if ((int)$max == (int)$id_banner) {
            return true;
        }

        $rows = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT hss.`position` as position, hss.`id_dfxbanner` as id_banner
            FROM `'._DB_PREFIX_.'dfxbanner` hss
            LEFT JOIN `'._DB_PREFIX_.'dfxbanner_shop` hs ON (hss.`id_dfxbanner` = hs.`id_dfxbanner`)
            WHERE hs.`id_shop` = '.(int)$id_shop.' AND hss.`position` > '.(int)$this->position
        );

        foreach ($rows as $row) {
            $current_banner = new DfxbannerClass($row['id_banner']);
            --$current_banner->position;
            $current_banner->update();
            unset($current_banner);
        }

        return true;
    }

    public static function getAssociatedIdsShop($id_banner)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT hs.`id_shop`
            FROM `'._DB_PREFIX_.'dfxbanner_shop` hs
            WHERE hs.`id_dfxbanner` = '.(int)$id_banner
        );

        if (!is_array($result)) {
            return false;
        }

        $return = array();

        foreach ($result as $id_shop) {
            $return[] = (int)$id_shop['id_shop'];
        }

        return $return;
    }
}
