<?php
/**
* Copyright since 2007 PrestaShop SA and Contributors
* PrestaShop is an International Registered Trademark & Property of PrestaShop SA
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License 3.0 (AFL-3.0)
* that is bundled with this package in the file LICENSE.md.
* It is also available through the world-wide-web at this URL:
* https://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to https://devdocs.prestashop.com/ for more information.
*
* @author    PrestaShop SA and Contributors <contact@prestashop.com>
* @copyright Since 2007 PrestaShop SA and Contributors
* @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
*/

/**
 * @since   1.5.0
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

include_once _PS_MODULE_DIR_ . 'dfxbanner/DfxbannerClass.php';

class DfxBanner extends Module implements WidgetInterface
{
    protected $_html = '';
    protected $currentIndex;
    
    public function __construct()
    {
        $this->name = 'dfxbanner';
        $this->tab = 'front_office_features';
        $this->version = '6.1.0';
        $this->author = 'Digitalredfox';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('Dfx Banner', array(), 'Modules.DfxBanner.Admin');
        $this->description = $this->trans('Adds an banner to your site.', array(), 'Modules.DfxBanner.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.4.0', 'max' => _PS_VERSION_);
        
        $this->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
    }

    /**
     * @see Module::install()
     */
    public function install()
    {
        /* Adds Module */
        if (!parent::install() ||
            !$this->installDB() ||
            !$this->installSamples() ||
			!$this->registerHook('displayHeader') ||
            !$this->registerHook('displayBanner') ||
            !$this->registerHook('displayHomeTop') ||
            !$this->registerHook('displayHome') ||
            !$this->registerHook('displayHomeFooter') ||
            !$this->registerHook('displayLeftColumn') ||
            !$this->registerHook('displayRightColumn') ||
            !$this->registerHook('actionShopDataDuplication') ||
            !$this->registerHook('actionObjectLanguageAddAfter')
        ) {
            return false;
        }
        return true;
    }
    
    protected function installDB()
    {
        $res = 1;

        $res = (bool)Db::getInstance()->execute('
              CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxbanner` (
              `id_dfxbanner` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `position` int(10) unsigned NOT NULL DEFAULT \'0\',
              `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              `target_url` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              `hook` varchar(255) NOT NULL,
              PRIMARY KEY (`id_dfxbanner`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
            
        $res &= Db::getInstance()->execute('
               CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxbanner_shop` (
               `id_dfxbanner` int(10) unsigned NOT NULL AUTO_INCREMENT,
               `id_shop` int(10) unsigned NOT NULL,
               PRIMARY KEY (`id_dfxbanner`, `id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        $res &= Db::getInstance()->execute('	
               CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxbanner_lang` (
               `id_dfxbanner` int(10) unsigned NOT NULL AUTO_INCREMENT,
               `id_lang` int(10) unsigned NOT NULL,
               `title` varchar(255) NOT NULL,
               `description` text NOT NULL,
               `url` varchar(255) NOT NULL,
               `image` varchar(255) NOT NULL,
               `image_w` varchar(255) NOT NULL,
               `image_h` varchar(255) NOT NULL,
               PRIMARY KEY (`id_dfxbanner`, `id_lang`)
               ) ENGINE = '._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        return $res;
    }
    protected function installSamples()
    {
        $res = 1;
        include_once dirname(__FILE__).'/sql/sample.php';
        return $res;
    }


    /**
     * @see Module::uninstall()
     */
    public function uninstall()
    {
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();
            return (bool)$res;
        }

        return false;
    }

    /**
     * Creates tables
     */
   
    /**
     * deletes tables
     */
    protected function deleteTables()
    {
        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxbanner`, `'._DB_PREFIX_.'dfxbanner_shop`, `'._DB_PREFIX_.'dfxbanner_lang`;
        ');
    }

    public function getContent()
    {
        $this->context->controller->addJS($this->_path.'views/js/back.js');
        
        $this->_html .= $this->headerHTML();

        /* Validate & process */
        if (Tools::isSubmit('submitBanner') || Tools::isSubmit('delete_id_banner') ||
            Tools::isSubmit('changeStatus')
        ) {
            if ($this->_postValidation()) {
                $this->_postProcess();
                $this->_html .= $this->renderList();
            } else {
                $this->_html .= $this->renderAddForm();
            }

            $this->clearCache();
        } elseif (Tools::isSubmit('id_banner') && $this->bannerExists((int)Tools::getValue('id_banner'))) {
            $mode = 'edit';

            $associated_shop_ids = DfxbannerClass::getAssociatedIdsShop((int)Tools::getValue('id_banner'));
            $context_shop_id = (int)Shop::getContextShopID();

            if ($associated_shop_ids === false) {
                $this->_html .= $this->getShopAssociationError((int)Tools::getValue('id_banner'));
            } elseif (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL && in_array($context_shop_id, $associated_shop_ids)) {
                if (count($associated_shop_ids) > 1) {
                    $this->_html = $this->getSharedSlideWarning();
                }
                $this->_html .= $this->renderAddForm();
            } else {
                $shops_name_list = array();
                foreach ($associated_shop_ids as $shop_id) {
                    $associated_shop = new Shop((int)$shop_id);
                    $shops_name_list[] = $associated_shop->name;
                }
                $this->_html .= $this->getShopContextError($shops_name_list, $mode);
            }
        } else {
            $this->_html .= $this->getWarningMultishopHtml().$this->getCurrentShopInfoMsg();

            if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                $this->_html .= $this->renderList();
            }
        }

        return $this->_html;
    }

    protected function _postValidation()
    {
        $errors = array();

        /* Validation for Slider configuration */
        if (Tools::isSubmit('changeStatus')) {
            if (!Validate::isInt(Tools::getValue('id_banner'))) {
                $errors[] = $this->trans('Invalid slide', array(), 'Modules.DfxBanner.Admin');
            }
        } elseif (Tools::isSubmit('submitBanner')) {
            /* Checks state (active) */
            if (!Validate::isInt(Tools::getValue('active_banner')) || (Tools::getValue('active_banner') != 0 && Tools::getValue('active_banner') != 1)) {
                $errors[] = $this->trans('Invalid banner state.', array(), 'Modules.DfxBanner.Admin');
            }
            /* Checks position */
            if (!Validate::isInt(Tools::getValue('position')) || (Tools::getValue('position') < 0)) {
                $errors[] = $this->trans('Invalid banner position.', array(), 'Modules.DfxBanner.Admin');
            }
            /* If edit : checks id_banner */
            if (Tools::isSubmit('id_banner')) {
                if (!Validate::isInt(Tools::getValue('id_banner')) && !$this->bannerExists(Tools::getValue('id_banner'))) {
                    $errors[] = $this->trans('Invalid slide ID', array(), 'Modules.DfxBanner.Admin');
                }
            }
            /* Checks title/url/legend/description/image */
            $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
                if (Tools::strlen(Tools::getValue('title_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->trans('The title is too long.', array(), 'Modules.DfxBanner.Admin');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->trans('The URL is too long.', array(), 'Modules.DfxBanner.Admin');
                }
                if (Tools::strlen(Tools::getValue('description_' . $language['id_lang'])) > 4000) {
                    $errors[] = $this->trans('The description is too long.', array(), 'Modules.DfxBanner.Admin');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 0 && !Validate::isUrl(Tools::getValue('url_' . $language['id_lang']))) {
                    $errors[] = $this->trans('The URL format is not correct.', array(), 'Modules.DfxBanner.Admin');
                }
                if (Tools::getValue('image_' . $language['id_lang']) != null && !Validate::isFileName(Tools::getValue('image_' . $language['id_lang']))) {
                    $errors[] = $this->trans('Invalid filename.', array(), 'Modules.DfxBanner.Admin');
                }
                if (Tools::getValue('image_old_' . $language['id_lang']) != null && !Validate::isFileName(Tools::getValue('image_old_' . $language['id_lang']))) {
                    $errors[] = $this->trans('Invalid filename.', array(), 'Modules.DfxBanner.Admin');
                }
            }

            /* Checks title/url/legend/description for default lang */
            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            if (Tools::strlen(Tools::getValue('url_' . $id_lang_default)) == 0) {
                $errors[] = $this->trans('The URL is not set.', array(), 'Modules.DfxBanner.Admin');
            }
            if (!Tools::isSubmit('has_picture') && (!isset($_FILES['image_' . $id_lang_default]) || empty($_FILES['image_' . $id_lang_default]['tmp_name']))) {
                $errors[] = $this->trans('The image is not set.', array(), 'Modules.DfxBanner.Admin');
            }
            if (Tools::getValue('image_old_'.$id_lang_default) && !Validate::isFileName(Tools::getValue('image_old_'.$id_lang_default))) {
                $errors[] = $this->trans('The image is not set.', array(), 'Modules.DfxBanner.Admin');
            }
        } elseif (Tools::isSubmit('delete_id_banner') && (!Validate::isInt(Tools::getValue('delete_id_banner')) || !$this->bannerExists((int)Tools::getValue('delete_id_banner')))) {
            $errors[] = $this->trans('Invalid banner ID', array(), 'Modules.DfxBanner.Admin');
        }

        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));

            return false;
        }

        /* Returns if validation is ok */

        return true;
    }
    
    public function hookDisplayHeader($params)
    {
        $this->context->controller->addCSS($this->_path.'views/css/front.css');
    }
    protected function _postProcess()
    {
        $errors = array();
        /* Processes Slider */
        if (Tools::isSubmit('changeStatus') && Tools::isSubmit('id_banner')) {
            $banner = new DfxbannerClass((int)Tools::getValue('id_banner'));
            if ($banner->active == 0) {
                $banner->active = 1;
            } else {
                $banner->active = 0;
            }
            $res = $banner->update();
            $this->clearCache();
            $this->_html .= ($res ? $this->displayConfirmation($this->trans('Configuration updated', array(), 'Admin.Notifications.Success')) : $this->displayError($this->trans('The configuration could not be updated.', array(), 'Modules.DfxBanner.Admin')));
        } elseif (Tools::isSubmit('submitBanner')) {
            /* Sets ID if needed */
            if (Tools::getValue('id_banner')) {
                $banner = new DfxbannerClass((int)Tools::getValue('id_banner'));
                if (!Validate::isLoadedObject($banner)) {
                    $this->_html .= $this->displayError($this->trans('Invalid slide ID', array(), 'Modules.DfxBanner.Admin'));
                    return false;
                }
            } else {
                $banner = new DfxbannerClass();
            }
            
            
            /* Sets position */
            $banner->position = (int)Tools::getValue('position');
            /* Sets active */
            $banner->active = (int)Tools::getValue('active_banner');
            $banner->target_url = (int)Tools::getValue('item_target');
            /* Sets each langue fields */
            $languages = Language::getLanguages(false);

            foreach ($languages as $language) {
                $banner->title[$language['id_lang']] = Tools::getValue('title_'.$language['id_lang']);
                $banner->url[$language['id_lang']] = Tools::getValue('url_'.$language['id_lang']);
                $banner->description[$language['id_lang']] = Tools::getValue('description_'.$language['id_lang']);
                
                
                    
                /* Uploads image and sets slide */
                $type = Tools::strtolower(Tools::substr(strrchr($_FILES['image_'.$language['id_lang']]['name'], '.'), 1));
                $imagesize = @getimagesize($_FILES['image_'.$language['id_lang']]['tmp_name']);
                
            
                $banner->image_w[$language['id_lang']] = is_numeric(Tools::getValue('image_w_'.$language['id_lang'])) ? (int)Tools::getValue('image_w_'.$language['id_lang']) : '';
                $banner->image_h[$language['id_lang']] = is_numeric(Tools::getValue('image_h_'.$language['id_lang'])) ? (int)Tools::getValue('image_h_'.$language['id_lang']) : '';
                
                $salt = sha1(microtime());
                $temp_name = dirname(__FILE__).DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR. $salt . '_' . $_FILES['image_' . $language['id_lang']]['name'];
                if (!move_uploaded_file($_FILES['image_' . $language['id_lang']]['tmp_name'], dirname(__FILE__).DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR . $salt . '_' . $_FILES['image_' . $language['id_lang']]['name'])) {
                } elseif (!ImageManager::resize($temp_name, dirname(__FILE__).DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR.$_FILES['image_'.$language['id_lang']]['name'], null, null, $type)) {
                    $errors[] = $this->displayError($this->getTranslator()->trans('An error occurred during the image upload process.', array(), 'Admin.Notifications.Error'));
                } else {
                    if (isset($_FILES['image_' . $language['id_lang']]) &&
                        isset($_FILES['image_' . $language['id_lang']]['tmp_name']) &&
                        !empty($_FILES['image_' . $language['id_lang']]['tmp_name']) && !empty($imagesize)) {
                        $banner->image[$language['id_lang']] = $salt . '_' . $_FILES['image_' . $language['id_lang']]['name'];
                        $banner->image_w[$language['id_lang']] = $imagesize['0'];
                        $banner->image_h[$language['id_lang']] = $imagesize['1'];
                    } elseif (Tools::getValue('image_old_' . $language['id_lang']) != '') {
                        $banner->image[$language['id_lang']] = Tools::getValue('image_old_' . $language['id_lang']);
                        $banner->image_w[$language['id_lang']] = $imagesize['0'];
                        $banner->image_h[$language['id_lang']] = $imagesize['1'];
                    }
                }
            }

            /* Processes if no errors  */
            if (!$errors) {
                /* Adds */
                if (!Tools::getValue('id_banner')) {
                    if (!$banner->add()) {
                        $errors[] = $this->displayError($this->trans('The slide could not be added.', array(), 'Modules.DfxBanner.Admin'));
                    }
                } elseif (!$banner->update()) {
                    $errors[] = $this->displayError($this->trans('The slide could not be updated.', array(), 'Modules.DfxBanner.Admin'));
                }
                $this->clearCache();
            }
        } elseif (Tools::isSubmit('delete_id_banner')) {
            $banner = new DfxbannerClass((int)Tools::getValue('delete_id_banner'));
            $res = $banner->delete();
            $this->clearCache();
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=1&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        }
    
        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } elseif (Tools::isSubmit('submitBanner') && Tools::getValue('id_banner')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitBanner')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    
    public function getAllbanners($hookName = null)
    {
        $banners = $this->getBanners($hookName, true);
        $totalitems = count($banners);
        if (is_array($banners)) {
            foreach ($banners as &$banner) {
                $banner['sizes'] = @getimagesize((dirname(__FILE__) . DIRECTORY_SEPARATOR . 'views' . DIRECTORY_SEPARATOR . 'img' . DIRECTORY_SEPARATOR . $banner['image']));
                if (isset($banner['sizes'][3]) && $banner['sizes'][3]) {
                    $banner['size'] = $banner['sizes'][3];
                }
            }
        }

       return [
                'htmlitems' => $banners,
                'totalitems' => $totalitems,
                'hook' => $hookName
        ];
       

    }

    public function renderWidget($hookName = null, array $configuration = [])
    {
        $templateFile = 'module:dfxbanner/views/templates/hook/hook.tpl';
		
			if (!$this->isCached($templateFile)) {
				$variables = $this->getWidgetVariables($hookName, $configuration);
	
				if (empty($variables)) {
					return false;
				}
	
				$this->smarty->assign($variables);
			}

        return $this->fetch($templateFile);

    }

    public function getWidgetVariables($hookName = null, array $configuration = [])
    {
        if ($hookName == null && isset($configuration['hook'])) {
            $hookName = $configuration['hook'];
        }
		if (preg_match('/displayBanner\d*$/', $hookName)) {
            $banners = $this->getAllbanners('displayBanner');
        } elseif (preg_match('/displayHome\d*$/', $hookName)) {
            $banners = $this->getAllbanners('displayHome');
        } elseif (preg_match('/displayHomeTop\d*$/', $hookName)) {
            $banners = $this->getAllbanners('displayHomeTop');
        } elseif (preg_match('/displayHomeFooter\d*$/', $hookName)) {
            $banners = $this->getAllbanners('displayHomeFooter');
        } elseif (preg_match('/displayLeftColumn\d*$/', $hookName)) {
            $banners = $this->getAllbanners('displayLeftColumn');
        } elseif (preg_match('/displayRightColumn\d*$/', $hookName)) {
            $banners = $this->getAllbanners('displayRightColumn');
        }
		
		if (!empty($banners)) {
			return $banners;
        }
        return false;
    }
	
    protected function updateUrl($link)
    {
        if (Tools::substr($link, 0, 7) !== "http://" && Tools::substr($link, 0, 8) !== "https://") {
            $link = "http://" . $link;
        }

        return $link;
    }

    public function clearCache()
    {
        $this->_clearCache('*');
    }

    public function hookActionShopDataDuplication($params)
    {
        Db::getInstance()->execute(
            '
            INSERT IGNORE INTO '._DB_PREFIX_.'dfxbanner_shop (id_dfxbanner, id_shop)
            SELECT id_dfxbanner, '.(int)$params['new_id_shop'].'
            FROM '._DB_PREFIX_.'dfxbanner_shop
            WHERE id_shop = '.(int)$params['old_id_shop']
        );
        $this->clearCache();
    }

    public function headerHTML()
    {
        if (Tools::getValue('controller') != 'AdminModules' && Tools::getValue('configure') != $this->name) {
            return;
        }

        $this->context->controller->addJqueryUI('ui.sortable');
        /* Style & js for fieldset 'slides configuration' */
        $html = '<script type="text/javascript">
            $(function() {
                var $mySlides = $("#banners_displayBanner, #banners_displayHomeTop, #banners_displayHome, #banners_displayHomeFooter, #banners_displayLeftColumn, #banners_displayRightColumn"); 
                $mySlides.sortable({
                    opacity: 0.6,
                    cursor: "move",
                    update: function() {
                        var order = $(this).sortable("serialize") + "&action=updateBannersPosition";
                        $.post("'.$this->context->shop->physical_uri.$this->context->shop->virtual_uri.'modules/'.$this->name.'/ajax_'.$this->name.'.php?secure_key='.$this->secure_key.'", order);
                        }
                    });
                $mySlides.hover(function() {
                    $(this).css("cursor","move");
                    },
                    function() {
                    $(this).css("cursor","auto");
                });
            });
        </script>';

        return $html;
    }

    public function getNextPosition()
    {
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow(
            '
            SELECT MAX(hss.`position`) AS `next_position`
            FROM `'._DB_PREFIX_.'dfxbanner` hss, `'._DB_PREFIX_.'dfxbanner_shop` hs
            WHERE hss.`id_dfxbanner` = hs.`id_dfxbanner` AND hs.`id_shop` = '.(int)$this->context->shop->id
        );

        return (++$row['next_position']);
    }

    public function getBanners($hook, $active = null)
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;
        $id_lang = $this->context->language->id;

        $banners = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT hs.`id_dfxbanner` as id_banner, hs.*, hss.*, hssl.*
            FROM '._DB_PREFIX_.'dfxbanner_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxbanner hss ON (hs.id_dfxbanner = hss.id_dfxbanner)
            LEFT JOIN '._DB_PREFIX_.'dfxbanner_lang hssl ON (hss.id_dfxbanner = hssl.id_dfxbanner)
            WHERE id_shop = '.(int)$id_shop.'
            AND hss.hook = \''.pSQL($hook).'\' AND hssl.id_lang = '.(int)$id_lang.
            ($active ? ' AND hss.`active` = 1' : ' ').'
            ORDER BY hss.position'
        );

        foreach ($banners as &$banner) {
            $banner['image_url'] = $this->_path.'views/img/'.$banner['image'];
            $banner['url'] = $banner['url'];
        }

        return $banners;
    }

    public function getAllImagesByBannersId($id_banners, $active = null, $id_shop = null)
    {
        $this->context = Context::getContext();
        $images = array();

        if (!isset($id_shop)) {
            $id_shop = $this->context->shop->id;
        }

        $results = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT hssl.`image`, hssl.`id_lang`
            FROM '._DB_PREFIX_.'dfxbanner_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxbanner hss ON (hs.id_dfxbanner = hss.id_dfxbanner)
            LEFT JOIN '._DB_PREFIX_.'dfxbanner_lang hssl ON (hss.id_dfxbanner = hssl.id_dfxbanner)
            WHERE hs.`id_dfxbanner` = '.(int)$id_banners.' AND hs.`id_shop` = '.(int)$id_shop.
            ($active ? ' AND hss.`active` = 1' : ' ')
        );

        foreach ($results as $result) {
            $images[$result['id_lang']] = $result['image'];
        }

        return $images;
    }

    public function displayStatus($id_banner, $active)
    {
        $title = ((int)$active == 0 ? $this->trans('Disabled', array(), 'Admin.Global') : $this->trans('Enabled', array(), 'Admin.Global'));
        $icon = ((int)$active == 0 ? 'icon-remove' : 'icon-check');
        $class = ((int)$active == 0 ? 'btn-danger' : 'btn-success');
        $html = '<a class="btn '.$class.'" href="'.AdminController::$currentIndex.
            '&configure='.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules').
                '&changeStatus&id_banner='.(int)$id_banner.'" title="'.$title.'"><i class="'.$icon.'"></i> '.$title.'</a>';

        return $html;
    }

    public function bannerExists($id_banner)
    {
        $req = 'SELECT hs.`id_dfxbanner` as id_banner
                FROM `'._DB_PREFIX_.'dfxbanner_shop` hs
                WHERE hs.`id_dfxbanner` = '.(int)$id_banner;
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($req);

        return ($row);
    }

    public function renderList()
    {
        $allbanners = array();
        $htmlitems = array();

        $hooks = array(
                'displayBanner',
                'displayHomeTop',
                'displayHome',
                'displayHomeFooter',
                'displayLeftColumn',
                'displayRightColumn'
            );

        
        foreach ($hooks as $hook) {
            $allbanners =  $this->getBanners($hook, null);
            foreach ($allbanners as $key => $banner) {
                $allbanners[$key]['status'] = $this->displayStatus($banner['id_banner'], $banner['active']);
            }
            $htmlitems[$hook] = $allbanners;
        }
                
        $this->context->smarty->assign(
            array(
                'link' => $this->context->link,
                'htmlitems' => $htmlitems,
                'image_baseurl' => $this->_path.'views/img/',
            )
        );

        return $this->display(__FILE__, 'list.tpl');
    }

    public function renderAddForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Slide information', array(), 'Modules.DfxBanner.Admin'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'file_lang',
                        'label' => $this->trans('Image', array(), 'Admin.Global'),
                        'name' => 'image',
                        'desc' => $this->trans('Maximum image size: %s.', array(ini_get('upload_max_filesize')), 'Admin.Global'),
                        'required' => true,
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Width', array(), 'Modules.DfxBanner.Admin'),
                        'name' => 'image_w',
                        'suffix' => 'pixels',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->trans('Image width', array(), 'Modules.DfxBanner.Admin'),
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Height', array(), 'Modules.DfxBanner.Admin'),
                        'name' => 'image_h',
                        'suffix' => 'pixels',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->trans('Image height', array(), 'Modules.DfxBanner.Admin'),
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Title', array(), 'Admin.Global'),
                        'name' => 'title',
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Target URL', array(), 'Modules.DfxBanner.Admin'),
                        'name' => 'url',
                        'required' => true,
                        'lang' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Open link in a new tab/page', array(), 'Admin.Global'),
                        'name' => 'item_target',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->trans('Description', array(), 'Admin.Global'),
                        'name' => 'description',
                        'autoload_rte' => true,
                        'lang' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Enabled', array(), 'Admin.Global'),
                        'name' => 'active_banner',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                ),
                'buttons' => array(
                    array(
                     'href' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                    'title' => $this->trans('Cancel', array(), 'Modules.DfxBanner.Admin'),
                    'icon' => 'process-icon-back'
                    )
                )
            ),
        );

        if (Tools::isSubmit('id_banner') && $this->bannerExists((int)Tools::getValue('id_banner'))) {
            $banner = new DfxbannerClass((int)Tools::getValue('id_banner'));
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_banner');
            $fields_form['form']['images'] = $banner->image;

            $has_picture = true;

            foreach (Language::getLanguages(false) as $lang) {
                if (!isset($banner->image[$lang['id_lang']])) {
                    $has_picture &= false;
                }
            }

            if ($has_picture) {
                $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'has_picture');
            }
        }

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitBanner';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getAddFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'image_baseurl' => $this->_path.'views/img/'
        );

        $helper->override_folder = '/';

        $languages = Language::getLanguages(false);

        if (count($languages) > 1) {
            return $this->getMultiLanguageInfoMsg() . $helper->generateForm(array($fields_form));
        } else {
            return $helper->generateForm(array($fields_form));
        }
    }

    public function getAddFieldsValues()
    {
        $fields = array();
        $languages = Language::getLanguages(false);
        if (Tools::isSubmit('id_banner') && $this->bannerExists((int)Tools::getValue('id_banner'))) {
            $banner = new DfxbannerClass((int)Tools::getValue('id_banner'));
            $fields['id_banner'] = (int)Tools::getValue('id_banner', $banner->id);
        } else {
            $banner = new DfxbannerClass();
        }

        $fields['active_banner'] = Tools::getValue('active_banner', $banner->active);
        $fields['has_picture'] = true;
        $fields['item_target'] = Tools::getValue('item_target', $banner->target_url);
  
        
                            
        foreach ($languages as $lang) {
            $fields['image'][$lang['id_lang']] = Tools::getValue('image_'.(int)$lang['id_lang'], $banner->image[$lang['id_lang']]);
            
            $fields['image_w'][$lang['id_lang']] = is_numeric(Tools::getValue('image_w_'.(int)$lang['id_lang'], $banner->image_w[$lang['id_lang']])) ? (int)Tools::getValue('image_w_'.(int)$lang['id_lang'], $banner->image_w[$lang['id_lang']]) : '';
                
            $fields['image_h'][$lang['id_lang']] = is_numeric(Tools::getValue('image_h_'.(int)$lang['id_lang'], $banner->image_h[$lang['id_lang']])) ? (int)Tools::getValue('image_h_'.(int)$lang['id_lang'], $banner->image_h[$lang['id_lang']]) : '';
            
            $fields['title'][$lang['id_lang']] = Tools::getValue('title_'.(int)$lang['id_lang'], $banner->title[$lang['id_lang']]);
            $fields['url'][$lang['id_lang']] = Tools::getValue('url_'.(int)$lang['id_lang'], $banner->url[$lang['id_lang']]);
            $fields['description'][$lang['id_lang']] = Tools::getValue('description_'.(int)$lang['id_lang'], $banner->description[$lang['id_lang']]);
        }

        return $fields;
    }

    protected function getMultiLanguageInfoMsg()
    {
        return '<p class="alert alert-warning">'.
                    $this->trans('Since multiple languages are activated on your shop, please mind to upload your image for each one of them', array(), 'Modules.DfxBanner.Admin').
                '</p>';
    }

    protected function getWarningMultishopHtml()
    {
        if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
            return '<p class="alert alert-warning">' .
            $this->trans('You cannot manage slides items from a "All Shops" or a "Group Shop" context, select directly the shop you want to edit', array(), 'Modules.DfxBanner.Admin') .
            '</p>';
        } else {
            return '';
        }
    }
    
    public function hookActionObjectLanguageAddAfter($params)
    {
        return $this->installSamples(array(array('id_lang' => (int)$params['object']->id)));
    }

    protected function getShopContextError($shop_contextualized_name, $mode)
    {
        if (is_array($shop_contextualized_name)) {
            $shop_contextualized_name = implode('<br/>', $shop_contextualized_name);
        }

        if ($mode == 'edit') {
            return '<p class="alert alert-danger">' .
            $this->trans('You can only edit this slide from the shop(s) context: %s', array($shop_contextualized_name), 'Modules.DfxBanner.Admin') .
            '</p>';
        } else {
            return '<p class="alert alert-danger">' .
            $this->trans('You cannot add slides from a "All Shops" or a "Group Shop" context', array(), 'Modules.DfxBanner.Admin') .
            '</p>';
        }
    }

    protected function getShopAssociationError($id_banner)
    {
        return '<p class="alert alert-danger">'.
                        $this->trans('Unable to get slide shop association information (id_banner: %d)', array((int)$id_banner), 'Modules.DfxBanner.Admin') .
                '</p>';
    }


    protected function getCurrentShopInfoMsg()
    {
        $shop_info = null;

        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_SHOP) {
                $shop_info = $this->trans('The modifications will be applied to shop: %s', array($this->context->shop->name), 'Modules.DfxBanner.Admin');
            } elseif (Shop::getContext() == Shop::CONTEXT_GROUP) {
                $shop_info = $this->trans('The modifications will be applied to this group: %s', array(Shop::getContextShopGroup()->name), 'Modules.DfxBanner.Admin');
            } else {
                $shop_info = $this->trans('The modifications will be applied to all shops and shop groups', array(), 'Modules.DfxBanner.Admin');
            }

            return '<div class="alert alert-info">'.
                        $shop_info.
                    '</div>';
        } else {
            return '';
        }
    }

    protected function getSharedSlideWarning()
    {
        return '<p class="alert alert-warning">'.
                    $this->trans('This slide is shared with other shops! All shops associated to this slide will apply modifications made here', array(), 'Modules.DfxBanner.Admin').
                '</p>';
    }
}
