<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    // module validation
    exit;
}

class DfxCategory extends ObjectModel
{
    public $id;
    public $id_dfxblog_category;
    public $name;
    public $image = '';
    public $active = 1;
    public $position;
    public $description;
    public $id_parent;
    public $level_depth;
    public $link_rewrite;
    public $meta_title;
    public $meta_keywords;
    public $meta_description;
    public $date_add;
    public $date_upd;
    public $id_image = false;

    public static $definition = array(
        'table' => 'dfxblog_category',
        'primary' => 'id_dfxblog_category',
        'multilang' => true,
        'multilang_shop' => false,
        'fields' => array(
            'image' => array('type' => self::TYPE_STRING, 'validate' => 'isCatalogName'),
            'level_depth' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'),
            'active' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
            'id_parent' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt'),
            'position' => array('type' => self::TYPE_INT),
            'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'name' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCatalogName',
            'required' => true, 'size' => 128),
            'link_rewrite' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isLinkRewrite',
            'required' => true, 'size' => 128),
            'description' => array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml'),
            'meta_title' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName',
            'size' => 128),
            'meta_description' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName',
            'size' => 255),
            'meta_keywords' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName',
            'size' => 255),
        ),
    );

    /* admin function */
    public function __construct($id_dfxblog_category = null, $id_lang = null)
    {
        parent::__construct($id_dfxblog_category, $id_lang);
        
        
        $id_lang = (int) Context::getContext()->language->id;

        if (is_array($this->link_rewrite)) {
            $this->url = self::getLink($this->id_dfxblog_category, $this->link_rewrite[$id_lang], $id_lang);
        } else {
            $this->url = self::getLink($this->id_dfxblog_category, $this->link_rewrite, $id_lang);
        }
        
        //$this->url = self::getLink($this->id_dfxblog_category, $this->link_rewrite, $id_lang);
        
        $this->id_image = ($this->id &&
            file_exists(_DFXCAT_IMG_DIR_.$this->id.'.'.$this->image_format)) ?
            (int) $this->id : false;
    }

    
    public function getObjectLink($id_lang)
    {
        if (is_array($this->link_rewrite)) {
            $objectlink = self::getLink($this->id_dfxblog_category, $this->link_rewrite[$id_lang], $id_lang);
        } else {
            $objectlink = self::getLink($this->id_dfxblog_category, $this->link_rewrite, $id_lang);
        }
        return $objectlink;
    }
    
    public static function getLink($id_dfxblog_category, $rewrite, $id_lang = null, $id_shop = null)
    {
        $url = dfxblog::myRealUrl();
        $id_lang = Context::getContext()->language->id;
        
        $params = array();
        $params['id'] = $id_dfxblog_category;
        $params['rewrite'] = $rewrite;

        $dispatcher = Dispatcher::getInstance();

        return $url.$dispatcher->createUrl('dfx_category_rule', $id_lang, $params);
    }
    
    public static function getCategories(
        $id_lang = false,
        $active = true,
        $order = true,
        $sql_filter = '',
        $sql_sort = '',
        $without_parent = true,
        $sql_limit = ''
    ) {
        if (!Validate::isBool($active)) {
            die(Tools::displayError());
        }
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            'SELECT *
			FROM `'._DB_PREFIX_.'dfxblog_category` c
			LEFT JOIN `'._DB_PREFIX_.'dfxblog_category_lang` cl ON c.`id_dfxblog_category` = cl.`id_dfxblog_category`
			WHERE 1=1 '.($id_lang ? 'AND `id_lang` = '.(int) $id_lang : '').'
			'.($active ? 'AND `active` = 1' : '').'
			'.(!$id_lang ? 'GROUP BY c.id_dfxblog_category' : '').'
			'.($without_parent ? 'AND c.`id_parent` = 0' : '').'
			'.($sql_sort != '' ? $sql_sort : 'ORDER BY c.`position` ASC').'
			'.($sql_limit != '' ? $sql_limit : '')
        );

        if (!$order) {
            return $result;
        }

        $categories = array();
    
        foreach ($result as $row) {
            $categories[$row['id_dfxblog_category']]['name'] = $row['name'];
            $categories[$row['id_dfxblog_category']]['link_rewrite'] = $row['link_rewrite'];
            $categories[$row['id_dfxblog_category']]['id'] = $row['id_dfxblog_category'];
            $categories[$row['id_dfxblog_category']]['is_child'] = $row['id_parent'] > 0 ? 1 : 0;
            if (sizeof(self::getChildren($row['id_dfxblog_category']))) {
                $categories[$row['id_dfxblog_category']]['childrens'] = self::getChildren($row['id_dfxblog_category']);
            }
        }

        return $categories;
    }

    public static function getDescriptionClean($description)
    {
        return Tools::getDescriptionClean($description);
    }

    public function add($autodate = true, $null_values = false)
    {
        $this->calcLevelDepth();
        $this->setLastPosition();
        $ret = parent::add($autodate, $null_values);

        Hook::exec('actionDfxBlogUpdate', array('category' => $this));

        return $ret;
    }

    public function calcLevelDepth()
    {
        if (!$this->id_parent || $this->id_parent == 0) {
            $this->level_depth = 0;
        } else {
            $parent_category = new self((int) $this->id_parent);
            if (!Validate::isLoadedObject($parent_category)) {
                throw new PrestaShopException('Parent category does not exist');
            }

            $this->level_depth = $parent_category->level_depth + 1;
        }
    }

    public function setLastPosition()
    {
        $max_position = (int) Db::getInstance()->getValue(
            'SELECT MAX(dfxc.`position`)
			FROM `'._DB_PREFIX_.'dfxblog_category` dfxc
			WHERE dfxc.`id_parent` = '.(int) $this->id_parent
        );

        $this->position = 1 + $max_position;
    }

    public function delete()
    {
        $id_lang = Configuration::get('PS_LANG_DEFAULT');

        if ($this->getChildren($this->id_dfxblog_category, $id_lang, false)) {
            throw new PrestaShopException(
                'Category "'.$this->name[$id_lang].'" does not empty. Please delete all sub categories.'
            );
        }

        $this->deleteImg();

        $ret = parent::delete();

        Hook::exec('actionDfxBlogUpdate', array('category' => $this));

        return $ret;
    }

    public function deleteImg()
    {
        if (!$this->id) {
            return false;
        }

        if ($this->id_image) {
            $image_path = _DFXCAT_IMG_DIR_.$this->id.'.'.$this->image_format;
            
            if (file_exists($image_path)) {
                unlink($image_path);
            }
        }
        return true;
    }
    
    
    public static function getChildren($id_parent)
    {
        $id_lang = Context::getContext()->language->id;

        $child_categories = DB::getInstance()->executeS('
            SELECT *
            FROM `'._DB_PREFIX_.'dfxblog_category` rbc
            LEFT JOIN `'._DB_PREFIX_.'dfxblog_category_lang` rbcl
                ON (rbc.`id_dfxblog_category` = rbcl.`id_dfxblog_category` AND rbcl.`id_lang` = '.(int)$id_lang.')
            WHERE rbc.`id_parent` = '.(int)$id_parent.' AND rbc.active = 1
            ORDER BY rbc.`position` ASC
        ');

        return $child_categories;
    }

    public function update($null_values = false)
    {
        if ($this->id_parent == $this->id) {
            throw new PrestaShopException('A category cannot be it\'s own parent');
        }

        $this->calcLevelDepth();
        $ret = parent::update($null_values);

        Hook::exec('actionDfxBlogUpdate', array('category' => $this));

        return $ret;
    }

    /* front-end function */
    public static function getRewriteByCategory($id_dfxblog_category, $id_lang)
    {
        $sql = new DbQuery();
        $sql->select('l.link_rewrite');
        $sql->from('dfxblog_category_lang', 'l');
        $sql->where('l.id_dfxblog_category = '.$id_dfxblog_category.' AND l.id_lang = '.(int)$id_lang);

        return Db::getInstance()->getValue($sql);
    }
    public static function getImgFolderStatic($id = null)
    {
        if (!is_numeric($id)) {
            return false;
        }

        $folders = str_split((string) $id);

        return implode('/', $folders).'/';
    }
    
    /* front-end function */
    public function getAllPosts($id_lang, $p, $n, $get_total = false, $order_by = null, $order_way = null, $active = true)
    {
        if ($p < 1) {
            $p = 1;
        }

        if (empty($order_by)) {
            $order_by = 'date_add';
        } else {
            $order_by = Tools::strtolower($order_by);
        }

        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_dfxblog_post' ||
            $order_by == 'date_add' ||
            $order_by == 'date_upd' ||
            $order_by == 'position'
        ) {
            $order_by_prefix = 'p';
        } elseif ($order_by == 'name') {
            $order_by_prefix = 'pl';
        }

        if (!Validate::isBool($active) || !Validate::isOrderBy($order_by) || !Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }

        /* Return only the number of post */
        if ($get_total) {
            $sql = 'SELECT COUNT(`id_dfxblog_post`) AS total
					FROM `'._DB_PREFIX_.'dfxblog_post` p
					WHERE p.`id_dfxblog_category` = '.(int) $this->id.
                ($active ? ' AND p.`active` = 1' : '');

            return (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
        }

        $sql = 'SELECT p.*, pl.`name`, pl.`description_short`,
					pl.`link_rewrite`, pl.`tags`
				FROM `'._DB_PREFIX_.'dfxblog_post` p
				LEFT JOIN `'._DB_PREFIX_.'dfxblog_post_lang` pl
					ON (p.`id_dfxblog_post` = pl.`id_dfxblog_post`
					AND pl.`id_lang` = '.(int) $id_lang.')
				WHERE p.`id_dfxblog_category` = '.(int) $this->id
            .($active ? ' AND p.`active` = 1' : '')
            .' GROUP BY p.id_dfxblog_post';

        $sql .= ' ORDER BY '.(isset($order_by_prefix) ? $order_by_prefix.'.' : '').'`'.pSQL($order_by).
        '` '.pSQL($order_way).'
			LIMIT '.(((int) $p - 1) * (int) $n).','.(int) $n;

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if ($result) {
            foreach ($result as &$row) {
                $row['related_products'] = Tools::unSerialize($row['related_products']);
                $category_rewrite = self::getRewriteByCategory($row['id_dfxblog_category'], $id_lang);
                $category_obj = new DfxCategory($row['id_dfxblog_category'], $id_lang);
                $category_url = self::getLink($category_obj->id, $category_obj->link_rewrite, $id_lang);
                $row['url'] = DfxPost::getLink($row['id_dfxblog_post'], $row['link_rewrite'], $id_lang);
                $row['category'] = $category_obj->name;
                $row['category_rewrite'] = $category_obj->link_rewrite;
                $row['category_url'] = $category_url;
            }
        }

        return DfxPost::getPostsInfo($result, $id_lang);
    }
}
