<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class AdminDfxCommentController extends ModuleAdminController
{
    protected $position_identifier = 'id_dfxblog_post';
    
    public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'dfxblog_comment';
        $this->className = 'DfxComment';
        $this->identifier = 'id_dfxblog_comment';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->lang = false;
        $this->explicitSelect = true;
        $this->allow_export = true;
        
        $this->context = Context::getContext();
        parent::__construct();
        $this->settings = unserialize(Configuration::get('DFX_BLOG_SETTINGS'));
        $comment_validate = isset($this->settings['comment_validate']) ? $this->settings['comment_validate'] : 1;
        if ($comment_validate == 0) {
            $this->fields_list = array(
            'id_dfxblog_comment' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',
            ),
            'comment' => array(
                'title' => $this->l('Comment'),
                'maxlength' => 90,
                'callback' => 'getDescriptionClean',
                'orderby' => false,
            ),
            'customer_name' => array(
                'title' => $this->l('Customer'),
                'class' => 'fixed-width-xs',
                'orderby' => true,
            ),
             'customer_email' => array(
                'title' => $this->l('Customer Email'),
                'class' => 'fixed-width-xs',
                'orderby' => true,
            ),
            'id_dfxblog_post' => array(
                'title' => $this->l('Post'),
                'callback' => 'getPostName',
                'orderby' => true,
            ),
            'date_add' => array(
                'title' => $this->l('Date Add'),
                'orderby' => true,
                'align' => 'text-right',
                'class' => 'fixed-width-sm',
                'type' => 'datetime',
            ),
               );
        } else {
            $this->fields_list = array(
            'id_dfxblog_comment' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',
            ),
            'comment' => array(
                'title' => $this->l('Comment'),
                'maxlength' => 90,
                'callback' => 'getDescriptionClean',
                'orderby' => false,
            ),
            'customer_name' => array(
                'title' => $this->l('Customer'),
                'class' => 'fixed-width-xs',
                'orderby' => true,
            ),
             'customer_email' => array(
                'title' => $this->l('Customer Email'),
                'class' => 'fixed-width-xs',
                'orderby' => true,
            ),
            'id_dfxblog_post' => array(
                'title' => $this->l('Post'),
                'callback' => 'getPostName',
                'orderby' => true,
            ),
            'date_add' => array(
                'title' => $this->l('Date Add'),
                'orderby' => true,
                'align' => 'text-right',
                'class' => 'fixed-width-sm',
                'type' => 'datetime',
            ),
            'active' => array(
                'title' => $this->l('Approved'),
                'active' => 'status',
                'align' => 'center',
                'type' => 'bool',
                'class' => 'fixed-width-sm',
                'orderby' => false,
            ),
            );
        }
        

        $this->bulk_actions = array('delete' => array('text' => $this->l('Delete selected')));
    }

    public static function getDescriptionClean($description)
    {
        return Tools::getDescriptionClean($description);
    }

    public static function getPostName($id_dfxblog_post = 0)
    {
        $dfxpost = new DfxPost($id_dfxblog_post);
        $id_lang = (int) Configuration::get('PS_LANG_DEFAULT');

        if (isset($dfxpost->name[$id_lang])) {
            return '#'.$dfxpost->id_dfxblog_post.' - '.$dfxpost->name[$id_lang];
        } else {
            return 'Not found Post';
        }
    }

    public function renderList()
    {
        if (isset($this->_filter) && trim($this->_filter) == '') {
            $this->_filter = $this->original_filter;
        }
        
        
        $this->addRowAction('view');
        $this->addRowAction('delete');
        
        return parent::renderList();
    }

    public function initToolbar()
    {
        switch ($this->display) {
            default: // list
                if ($this->allow_export) {
                    $this->toolbar_btn['export'] = array(
                        'href' => self::$currentIndex.'&amp;export'.$this->table.'&amp;token='.$this->token,
                        'desc' => $this->l('Export'),
                    );
                }
        }
        $this->addToolBarModulesListButton();
    }

    public function renderView()
    {
        if (!($comment = $this->loadObject())) {
            return;
        }
        $comment_validate = isset($this->settings['comment_validate']) ? $this->settings['comment_validate'] : 1;
        $approved = array();
        if ($comment_validate == 0) {
            $approved = '<div class="list-action-enable action-enabled"><i class="icon-check"></i></div>';
        } else {
            $approved = $this->displayApproveLink($comment->id_dfxblog_comment, $comment->active);
        }
        $this->tpl_view_vars = array(
            'legend' => array(
                'title' => $this->l('Blog Comment'),
                'icon' => 'icon-tag',
            ),
            'fieldset' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Post:'),
                    'value' => $this->getPostName($comment->id_dfxblog_post),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Customer Name:'),
                    'value' => $comment->customer_name,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Comment:'),
                    'value' => $comment->comment,
                ),
                array(
                    'type' => 'active',
                    'label' => $this->l('Approved:'),
                    'value' => $approved,
                ),
                array(
                    'type' => 'date',
                    'label' => $this->l('Date Add:'),
                    'value' => $comment->date_add,
                ),
            ),
        );

        return parent::renderView();
    }

    public function displayApproveLink($id_dfxblog_comment, $active)
    {
        $url = Tools::safeOutput(
            self::$currentIndex.'&id_dfxblog_comment='.(int) $id_dfxblog_comment.
            '&statusdfxblog_comment&token='.$this->token
        );

        if ($active) {
            return '<a class="list-action-enable action-enabled" href="'.$url.'" title="Enabled"><i 
            class="icon-check"></i></a>';
        } else {
            return '<a class="list-action-enable action-disabled" href="'.$url.'" title="Disabled"><i 
            class="icon-remove"></i></a>';
        }
    }
}
