<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class DfxBlogCategoryModuleFrontController extends ModuleFrontController
{
    public $php_self;
    protected $category;
    protected $dfxlink;
    public $id_category;
    
    public function __construct()
    {
        parent::__construct();
        $this->context = Context::getContext();
        $this->template_path = _PS_MODULE_DIR_.'dfxblog/views/templates/front/';
        // $this->rewrite = 'aaaa';
    }
    
    public function init()
    {
        parent::init();

        // Get category
        $id_category = (int) Tools::getValue('id_dfxblog_category');
        $this->category = new DfxCategory($id_category, $this->context->language->id);
        $this->dfxlink = new DfxLink();
        
        if (!$this->category->id || !$this->category->active) {
            $this->errors[] = Tools::displayError('Missing Category ID');
            $headers = array('HTTP/1.1 404 Not Found', 'Status: 404 Not Found');
            Tools::redirect('404', __PS_BASE_URI__, null, $headers);
        }

        $meta_title = (isset($this->category->meta_title) && $this->category->meta_title != '') ?
        $this->category->meta_title : $this->category->name;
        $meta_description = isset($this->category->meta_description) ? $this->category->meta_description : '';
        $meta_keywords = isset($this->category->meta_keywords) ? $this->category->meta_keywords : '';

        $this->context->smarty->assign(array(
            'meta_title' => $meta_title,
            'meta_description' => $meta_description,
            'meta_keywords' => $meta_keywords,
        ));
    }

    public function initContent()
    {
        $this->php_self = 'category';
        
        parent::initContent();
        $id_lang = $this->context->language->id;

        // breadcrumb
        $path = $this->category->name;

        // get post by Category
        $num_posts = $this->category->getAllPosts(null, null, null, true);
        $this->pagination((int) $num_posts);
        $dfxposts = $this->category->getAllPosts($id_lang, (int) $this->p, (int) $this->n);

        $this->context->smarty->assign(array(
            'path' => $path,
            'dfxposts' => $dfxposts,
            'dfxcategory' => $this->category,
            'dfxlink' => $this->dfxlink,
        ));

        $this->setTemplate('module:dfxblog/views/templates/front/dfxcategory.tpl');
    }
    
    public function getLayout()
    {
        $entity = 'module-dfxblog-'.$this->php_self;
        
        $layout = $this->context->shop->theme->getLayoutRelativePathForPage($entity);
        
        if ($overridden_layout = Hook::exec(
            'overrideLayoutTemplate',
            array(
                'default_layout' => $layout,
                'entity' => $entity,
                'locale' => $this->context->language->locale,
                'controller' => $this,
            )
        )
        ) {
            return $overridden_layout;
        }

        if ((int) Tools::getValue('content_only')) {
            $layout = 'layouts/layout-content-only.tpl';
        }

        return $layout;
    }
    
    public function getBreadcrumbLinks()
    {
        $breadcrumb = parent::getBreadcrumbLinks();

        $id_lang = Context::getContext()->language->id;

        $breadcrumb['links'][] = [
            'title' => $this->l('Blog'),
            'url' => DfxLink::getBlogLink($id_lang)
        ];
        

        $breadcrumb['links'][] = [
                'title' =>  $this->category->name,
                'url' => $this->category->url
            ];


        return $breadcrumb;
    }
    
    public function pagination($nb_posts = 10)
    {
        if (!self::$initialized) {
            $this->init();
        } elseif (!$this->context) {
            $this->context = Context::getContext();
        }

        $post_per_page = isset($this->settings['post_per_page']) ? $this->settings['post_per_page'] : 10;

        $n_array = (int) $post_per_page != 10 ? array((int) $post_per_page, 10, 20, 30) : array(10, 20, 30);
        // Clean duplicate values
        $n_array = array_unique($n_array);
        asort($n_array);
        $this->n = abs((int) Tools::getValue('n', ((isset($this->context->cookie->nb_post_per_page) &&
        $this->context->cookie->nb_post_per_page >= 10) ?
        $this->context->cookie->nb_post_per_page : (int) $post_per_page)));
        $this->p = abs((int) Tools::getValue('p', 1));

        if (!is_numeric(Tools::getValue('p', 1)) || Tools::getValue('p', 1) < 0) {
            Tools::redirect(self::$link->getPaginationLink(false, false, $this->n, false, 1, false));
        }

        $current_url = Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']);
        //delete parameter page
        $current_url = preg_replace('/(\?)?(&amp;)?p=\d+/', '$1', $current_url);

        $range = 2;

        if ($this->p < 1) {
            $this->p = 1;
        }

        if (isset($this->context->cookie->nb_post_per_page) &&
            $this->n != $this->context->cookie->nb_post_per_page &&
            in_array($this->n, $n_array)
        ) {
            $this->context->cookie->nb_post_per_page = $this->n;
        }

        $pages_nb = ceil($nb_posts / (int) $this->n);
        if ($this->p > $pages_nb && $nb_posts != 0) {
            Tools::redirect(self::$link->getPaginationLink(false, false, $this->n, false, $pages_nb, false));
        }

        $start = (int) $this->p - $range;
        if ($start < 1) {
            $start = 1;
        }
        $stop = (int) $this->p + $range;
        if ($stop > $pages_nb) {
            $stop = (int) $pages_nb;
        }
        $this->context->smarty->assign('nb_posts', $nb_posts);
        $pagination_infos = array(
            'post_per_page' => (int) Configuration::get('PS_PRODUCTS_PER_PAGE'),
            'pages_nb' => $pages_nb,
            'p' => $this->p,
            'n' => $this->n,
            'nArray' => $n_array,
            'range' => $range,
            'start' => $start,
            'stop' => $stop,
            'current_url' => $current_url,
        );
        $this->context->smarty->assign($pagination_infos);
    }
}
