<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class DfxBlogRssModuleFrontController extends ModuleFrontController
{
    public $php_self;
    
    public function __construct()
    {
        parent::__construct();
        $this->context = Context::getContext();
        $this->template_path = _PS_MODULE_DIR_.'dfxblog/views/templates/front/';
        // $this->rewrite = 'aaaa';
    }
    
    public function initContent()
    {
        $this->php_self = 'rss';
        parent::initContent();

        // get all post
        $dfxposts = DfxPost::getAllPosts($this->context->language->id, 1, 50);

        $this->generateXMLFeed($dfxposts);
    }

    public function generateXMLFeed($posts = array())
    {
        $items = '';

        foreach ($posts as $post) {
            $category = array();
            foreach ($post['categories'] as $c) {
                $category[] = $c['name'];
            }

            $items .= '
		<item> 
			<title>'.htmlspecialchars($post['name']).'</title>
			<link>'.htmlspecialchars($post['link']).'</link>
			<description>'.htmlspecialchars($post['description']).'</description>
			<author>'.htmlspecialchars($post['posted_by']).'</author>
			<category>'.htmlspecialchars(implode(', ', $category)).'</category>
			<pubDate>'.date('F j, Y', strtotime($post['date_add'])).'</pubDate>			
		</item>';
        }

        $metas = Meta::getHomeMetas($this->context->language->id, 'index');

        $xml = '<rss version="2.0">
	<channel>
		<title>'.$this->l('Our Blogs').' - '.htmlspecialchars($metas['meta_title']).'</title>
		<description>'.htmlspecialchars($metas['meta_description']).'</description>
		<link>'.htmlspecialchars(_PS_BASE_URL_.__PS_BASE_URI__).'</link>
		<language>'.$this->context->language->language_code.'</language>
		<copyright>'.$this->l('Copyright 2022 Digitalredfox').'</copyright>'.$items.'	
	</channel>
</rss>';

        //header('Content-type: text/xml');
        die($xml);
    }
    
    public function getLayout()
    {
        $entity = 'module-dfxblog-'.$this->php_self;
        
        $layout = $this->context->shop->theme->getLayoutRelativePathForPage($entity);
        
        if ($overridden_layout = Hook::exec(
            'overrideLayoutTemplate',
            array(
                'default_layout' => $layout,
                'entity' => $entity,
                'locale' => $this->context->language->locale,
                'controller' => $this,
            )
        )
        ) {
            return $overridden_layout;
        }

        if ((int) Tools::getValue('content_only')) {
            $layout = 'layouts/layout-content-only.tpl';
        }

        return $layout;
    }
}
