<?php
/**
 * 2007-2021 PrestaShop.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

require_once _PS_MODULE_DIR_.'dfxblog/loader.php';

class DfxBlog extends Module implements WidgetInterface
{
    public $blog_config = array(
    'post_img_small' => array('width' => 285, 'height' => 204),
        'post_img_list' => array('width' => 320, 'height' => 230),
        'post_img_large' => array('width' => 812, 'height' => 580),
        'category_img' => array('width' => 870, 'height' => 220),
        'post_per_page' => 10,
        'allow_comment' => 1,
        'allow_views' => 1,
        'allow_likes' => 1,
        'comment_validate' => 1,
        'time_comment' => 60,
        'module_category' => 1,
        'module_archive' => 1,
        'module_latestpost' => 1,
        'module_currentcomment' => 1,
        'enable_relatedproduct' => 1,
    );
	
	public $templateFile;

    public function __construct()
    {
        $currentIndex = '';
        $this->name = 'dfxblog';
        $this->tab = 'front_office_features';
        $this->version = '4.5.0';
        $this->author = 'DigitalRedfox';
        $this->controllers = array('blog', 'category', 'archive', 'post', 'rss');
        $this->need_instance = 0;
        $this->module_key = '';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->trans('Dfx Blog', array(), 'Modules.DfxBlog.Admin');
        $this->description = $this->trans('Professional Blog Management.', array(), 'Modules.DfxBlog.Admin');

        $this->confirmUninstall = $this->trans('All blog data will be deleted. Are you sure you want to uninstall 
        this module?', array(), 'Modules.DfxBlog.Admin');
        
        if ($this->id && !$this->isRegisteredInHook('moduleRoutes')) {
            $this->registerHook('moduleRoutes');
        }
        if ($this->id && !$this->isRegisteredInHook('displayDfxblogAfterPostContent')) {
            $this->registerHook('displayDfxblogAfterPostContent');
        }
    }

    public function install()
    {
        $parent_id = $this->installTab('AdminDfxBlog', 'Dfx Blog', 0, true);

        if (!$parent_id) {
            return false;
        }

        if (!parent::install() ||
            !$this->createTables() ||
            !$this->installSample() ||
            !$this->installTab('AdminDfxPost', 'Blog Posts', $parent_id) ||
            !$this->installTab('AdminDfxCategory', 'Blog Categories', $parent_id) ||
            !$this->installTab('AdminDfxComment', 'Blog Comments', $parent_id) ||
            !$this->installTab('AdminDfxSetting', 'Blog Settings', $parent_id) ||
            !$this->registerHook('displayDfxBlogSidebar') ||
            !$this->registerHook('moduleRoutes') ||
            !$this->registerHook('displayBackOfficeHeader') ||
            !$this->registerHook('displayHeader') ||
            !$this->registerHook('actionDfxBlogUpdate') ||
            !$this->registerHook('displayBlogColumn') ||
			!$this->registerHook('displayLeftColumn') ||
			!$this->registerHook('displayRightColumn') ||
            !$this->registerHook('displayDfxblogAfterPostContent') ||
            !Configuration::updateValue('DFX_LATEST_POSTS_NBR', 10) ||
            !Configuration::updateValue('DFX_BLOG_SETTINGS', serialize($this->blog_config)) ||
            !Configuration::updateValue('DFX_LATEST_POSTS_TITLE', 0)) {
            return false;
        }
        $this->catFolder();
        return true;
    }

    public function uninstall()
    {
        $this->_clearCache('*');

        $sql = 'DROP TABLE
            `'._DB_PREFIX_.'dfxblog_category`,
            `'._DB_PREFIX_.'dfxblog_category_lang`,
			`'._DB_PREFIX_.'dfxblog_category_shop`,
            `'._DB_PREFIX_.'dfxblog_post`,
            `'._DB_PREFIX_.'dfxblog_post_lang`,
			`'._DB_PREFIX_.'dfxblog_post_shop`,
            `'._DB_PREFIX_.'dfxblog_comment`';

        if (!Db::getInstance()->execute($sql)) {
            return false;
        }

        if (!parent::uninstall() ||
            !$this->uninstallTab('AdminDfxSetting') ||
            !$this->uninstallTab('AdminDfxComment') ||
            !$this->uninstallTab('AdminDfxCategory') ||
            !$this->uninstallTab('AdminDfxPost') ||
            !$this->uninstallTab('AdminDfxBlog') ||
            !Configuration::deleteByName('DFX_BLOG_SETTINGS')) {
            return false;
        }

        return true;
    }
    protected function createTables()
    {
        $res = 1;
        include_once dirname(__FILE__).'/sql/install.php';
        return $res;
    }

    protected function installSample()
    {
        $res = 1;
        include_once dirname(__FILE__).'/sql/sample.php';
        return $res;
    }
    
    private function installTab($tabClass, $tabName, $idTabParent, $getID = false)
    {
        $tab = new Tab();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $tabName;
        }

        $tab->class_name = $tabClass;
        $tab->module = $this->name;
        $tab->id_parent = $idTabParent;

        $result = $tab->add();

        if ($getID && $result) {
            return $tab->id;
        }

        return $result;
    }

    private function uninstallTab($tabClass)
    {
        $result = true;

        $idTab = Tab::getIdFromClassName($tabClass);

        if ($idTab != 0) {
            $tab = new Tab($idTab);
            $result = $tab->delete();
        }

        return $result;
    }
    public static function catFolder()
    {
        if (is_dir(_DFXBLOG_MOD_DIR_._DFXCAT_) && is_dir(_DFXCAT_IMG_DIR_)) {
            $objects_b = scandir(_DFXBLOG_MOD_DIR_._DFXCAT_);

            foreach ($objects_b as $objects_b_val) {
                if ($objects_b_val != '.' && $objects_b_val != '..') {
                    if (filetype(_DFXBLOG_MOD_DIR_._DFXCAT_.$objects_b_val) == 'file') {
                        @copy(_DFXBLOG_MOD_DIR_._DFXCAT_.$objects_b_val, _DFXCAT_IMG_DIR_.$objects_b_val);
                    }
                }
            }
        }
            
        if (is_dir(_DFXBLOG_MOD_DIR_._DFXPRO_) && is_dir(_DFXPRO_IMG_DIR_)) {
            $objects_b = scandir(_DFXBLOG_MOD_DIR_._DFXPRO_);
            foreach ($objects_b as $objects_b_val) {
                if ($objects_b_val != '.' && $objects_b_val != '..') {
                    self::recurseCopy(_DFXBLOG_MOD_DIR_._DFXPRO_, _DFXPRO_IMG_DIR_);
                }
            }
        }
    }
    public static function recurseCopy($src, $dst, $del = false)
    {
        if (!file_exists($src)) {
            return false;
        }
        $dir = opendir($src);
        if (!Tools::file_exists_cache($dst)) {
            mkdir($dst);
        }
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                if (is_dir($src.'/'.$file)) {
                    self::recurseCopy($src.'/'.$file, $dst.'/'.$file, $del);
                } else {
                    copy($src.'/'.$file, $dst.'/'.$file);
                    if ($del && is_writable($src.'/'.$file)) {
                        unlink($src.'/'.$file);
                    }
                }
            }
        }
        closedir($dir);
        if ($del && is_writable($src)) {
            rmdir($src);
        }
    }
    
    public function getPath()
    {
        return $this->_path;
    }

    public function getContent()
    {
        $output = '';
        if (Tools::isSubmit('submitDfxLatestPosts')) {
            $postNbr = Tools::getValue('DFX_LATEST_POSTS_NBR');
            $postTitle = Tools::getValue('DFX_LATEST_POSTS_TITLE');

            if (!$postNbr || empty($postNbr) || (int) $postNbr == 0) {
                $output .= $this->displayError($this->getTranslator()->trans('Invalid number.'));
            } else {
                Configuration::updateValue('DFX_LATEST_POSTS_NBR', (int) $postNbr);
                Configuration::updateValue('DFX_LATEST_POSTS_TITLE', $postTitle);
                $this->_clearCache('*');
                $output .= $this->displayConfirmation($this->getTranslator()->trans('Settings updated'));
            }
			$this->_clearCache('*');
        }

        return $output.$this->renderForm();
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->getTranslator()->trans('Latest Posts Block Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Posts to display'),
                        'name' => 'DFX_LATEST_POSTS_NBR',
                        'class' => 'fixed-width-xs',
                        'desc' => $this->getTranslator()->trans('Define the number of posts to be displayed in Latest Posts block.'),
                    ),
                   
                ),
                'submit' => array(
                    'title' => $this->getTranslator()->trans('Save'),
                ),
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ?
        Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitDfxLatestPosts';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).
        '&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues()
    {
        return array(
            'DFX_LATEST_POSTS_NBR' => Tools::getValue(
                'DFX_LATEST_POSTS_NBR',
                Configuration::get('DFX_LATEST_POSTS_NBR')
            ),
            'DFX_LATEST_POSTS_TITLE' => Tools::getValue(
                'DFX_LATEST_POSTS_TITLE',
                Configuration::get('DFX_LATEST_POSTS_TITLE')
            ),
        );
    }
    public static function getLink()
    {
        return Context::getContext()->link->getModuleLink('dfxblog', 'blog');
    }
    
    public function hookActionDfxBlogUpdate()
    {
        $this->_clearCache('*');
    }

    public function hookDisplayBackOfficeHeader()
    {
        $this->context->controller->addCss($this->_path.'views/css/dfxblogadmin.css');
        $this->context->controller->addJS($this->_path.'views/js/dfxblog.js');
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->addCSS($this->_path.'views/css/dfxblog.css');
        $this->context->controller->addJS($this->_path.'views/js/dfxblog.js');
        
        if (Tools::getValue('module') && Tools::getValue('module') == 'dfxblog' && Tools::getValue('controller') == 'post') {
            $controller = Context::getContext()->controller;

            Media::addJsDef(
                array(
                    'dfx_sharing_name' => addcslashes($controller->dfxblog_post_name, "'"),
                    'dfx_sharing_url' => addcslashes($controller->dfxblog_post_url, "'"),
                    'dfx_sharing_img' => addcslashes($controller->dfxblog_post_thumbnail, "'")
                )
            );
        }
    }
	
	public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache($this->templateFile);
    }
	
    public static function myRealURL()
    {
        return Context::getContext()->shop->getBaseUrl(true);
    }

    private function preProcess()
    {
        $settings = unserialize(Configuration::get('DFX_BLOG_SETTINGS'));
        $id_lang = $this->context->language->id;
        $dfxlink = new DfxLink();

        // blog category
        $category_active = isset($settings['module_category']) ? $settings['module_category'] : true;
        $category_tree = false;
        if ($category_active) {
            $category_tree = DfxCategory::getCategories($id_lang);
            
            foreach ($category_tree as &$category) {
                $childcat = DfxCategory::getChildren($category['id']);
                $category['link'] = DfxCategory::getLink($category['id'], $category['link_rewrite'], $id_lang);
                    
                foreach ($childcat as &$child) {
                    $category['childlink'] = DfxCategory::getLink($child['id_dfxblog_category'], $child['link_rewrite'], $id_lang);
                }
            }
        }

        // blog archives
        $archive_active = isset($settings['module_archive']) ? $settings['module_archive'] : true;
        $archives = false;
        if ($archive_active) {
            $archives_translated = array(
                1 => $this->getTranslator()->trans('January'),
                2 => $this->getTranslator()->trans('February'),
                3 => $this->getTranslator()->trans('March'),
                4 => $this->getTranslator()->trans('April'),
                5 => $this->getTranslator()->trans('May'),
                6 => $this->getTranslator()->trans('June'),
                7 => $this->getTranslator()->trans('July'),
                8 => $this->getTranslator()->trans('August'),
                9 => $this->getTranslator()->trans('September'),
                10 => $this->getTranslator()->trans('October'),
                11 => $this->getTranslator()->trans('November'),
                12 => $this->getTranslator()->trans('December'),
            );
            $archives = DfxPost::getArchives();

            foreach ($archives as &$archive) {
                $archive['link'] = $dfxlink->getArchiveLink($archive['year'], $archive['month']);
                $archive['month'] = $archives_translated[(int) $archive['month']];
            }
        }

        // latest post
        $post_active = isset($settings['module_latestpost']) ? $settings['module_latestpost'] : true;
        $latest_post = false;
        if ($post_active) {
            $latest_post = DfxPost::getLatestPosts($id_lang);
        }

        // current comment
        $comment_active = isset($settings['module_currentcomment']) ? $settings['module_currentcomment'] : true;
        $current_comment = false;
        if ($comment_active) {
            $current_comment = DfxComment::getCurrentComments();

            foreach ($current_comment as &$comment) {
                $dfxpost = new DfxPost($comment['id_dfxblog_post']);
                $comment['link'] = DfxPost::getLink($comment['id_dfxblog_post'], $dfxpost->link_rewrite, $id_lang);
            }
        }

        return array(
            'category_active' => $category_active,
            'post_active' => $post_active,
            'comment_active' => $comment_active,
            'archive_active' => $archive_active,
            'category_tree' => $category_tree,
            'latest_post' => $latest_post,
            'current_comment' => $current_comment,
            'archives' => $archives,
            'self_tpl_dir' => _PS_MODULE_DIR_.$this->name.'/views/templates/hook/',
        );
    }
    
	public function renderWidget($hookName = null, array $configuration = array())
    {
		$this->templateFile = 'module:'.$this->name.'/views/templates/hook/dfxsidebar.tpl';
		if (preg_match('/^displayBlogColumn\d*$/', $hookName)) {
            $this->templateFile = 'module:'.$this->name.'/views/templates/hook/dfxposts_home.tpl';
        } elseif (preg_match('/^displayLeftColumn\d*$/', $hookName) || preg_match('/^displayRightColumn\d*$/', $hookName)) {
			 $this->templateFile = 'module:'.$this->name.'/views/templates/hook/dfxposts_column.tpl';
		} elseif (preg_match('/^displayDfxBlogSidebar\d*$/', $hookName)) {
			 $this->templateFile = 'module:'.$this->name.'/views/templates/hook/dfxsidebar.tpl';
		} elseif (preg_match('/^displayDfxblogAfterPostContent\d*$/', $hookName)) {
			 $this->templateFile = 'module:'.$this->name.'/views/templates/hook/after-post-content.tpl';
		}
		

		  if (!$this->isCached($this->templateFile, $this->getCacheId($this->name))) {
			 $assign = $this->getWidgetVariables($hookName, $configuration);
			 $this->smarty->assign($assign);
		  }

		return $this->fetch($this->templateFile, $this->getCacheId($this->name));
    }
	
	public function getWidgetVariables($hookName = null, array $configuration = array())
    {
		$posts = $sideblog = array();
		if (preg_match('/^displayBlogColumn\d*$/', $hookName) || preg_match('/^displayLeftColumn\d*$/', $hookName) || preg_match('/^displayRightColumn\d*$/', $hookName)) {
			$posts = $this->preProcessPosts();
			if (!empty($posts)) {
			 return $posts;
            }
        } else {
			$sideblog = $this->preProcess();
			if (!empty($sideblog)) {
			 return $sideblog;
            }
		}
		return false;
	}

    private function preProcessPosts()
    {
        $nbr = Configuration::get('DFX_LATEST_POSTS_NBR');
        $dfxlink = new DfxLink();

        if (!$nbr) {
            $nbr = 4;
        }
         
		$listimg_width = array();
		$listimg_height = array();
		
        $settings = unserialize(Configuration::get('DFX_BLOG_SETTINGS'));
		
        $allow_views = isset($settings['allow_views']) ? $settings['allow_views'] : 1;
        $allow_likes = isset($settings['allow_likes']) ? $settings['allow_likes'] : 1;
        $dfxposts = DfxPost::getLatestPosts($this->context->language->id, (int) $nbr);
		
		

		$listimg_width = isset($settings['post_img_list']['width']) ? $settings['post_img_list']['width'] : 1;
        $listimg_height = isset($settings['post_img_list']['height']) ? $settings['post_img_list']['height'] : 1;
        
		
        return array(
            'dfxposts' => $dfxposts,
            'listimg_width' => $listimg_width,
			'listimg_height' => $listimg_height,
            'dfxlink' => $dfxlink,
            'allow_views' => $allow_views,
            'allow_likes' => $allow_likes,
            'self_tpl_dir' => _PS_MODULE_DIR_.$this->name.'/views/templates/hook/',
			'title_position' => Configuration::get('DFX_LATEST_POSTS_TITLE')
        );
    }
	
	public function hookModuleRoutes($params)
    {
        $dfx_routes = array(
            'dfxblog_rule' => array(
                'controller' => 'blog',
                'rule' => 'blog',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'dfxblog',
                ),
            ),
            'dfx_category_rule' => array(
                'controller' => 'category',
                'rule' => 'blog/category/{id}-{rewrite}',
                'keywords' => array(
                    'id' => array('regexp' => '[0-9]+', 'param' => 'id_dfxblog_category'),
                    'rewrite' => array('regexp' => '[_a-zA-Z0-9-\pL]*', 'param' => 'rewrite'),
                ),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'dfxblog',
                ),
            ),
            'dfx_post_rule' => array(
                'controller' => 'post',
                'rule' => 'blog/{id}-{rewrite}',
                'keywords' => array(
                    'id' => array('regexp' => '[0-9]+', 'param' => 'id_dfxblog_post'),
                    'rewrite' => array('regexp' => '[_a-zA-Z0-9-\pL]*', 'param' => 'rewrite'),
                ),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'dfxblog',
                ),
            ),
            'dfx_archive_rule' => array(
                'controller' => 'archive',
                'rule' => 'blog/archive/{year}-{month}',
                'keywords' => array(
                    'year' => array('regexp' => '[0-9]+', 'param' => 'year'),
                    'month' => array('regexp' => '[0-9]+', 'param' => 'month'),
                ),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'dfxblog',
                ),
            ),
            'dfx_rss_rule' => array(
                'controller' => 'rss',
                'rule' => 'blog/rss',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'dfxblog',
                ),
            ),
        );

        return $dfx_routes;
    }
}
