<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class DfxCategorytabGroup extends ObjectModel
{
    public $id;
	public $offer;
	public $button;
    public $active;
    public $position;
    public $id_shop;
    public $banner_image;
    public $banner_image_w = '120';
    public $banner_image_h = '120';
    public $scrollcategories = 3;
    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'dfxcategory',
        'primary' => 'id_dfxcategory',
		'multilang' => true,
        'fields' => array(
            'active' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
            'position' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt', 'required' => true),
            // Lang fields
			'offer' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
			'button' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'banner_image_w' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'banner_image_h' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'banner_image' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
            'scrollcategories' => array('type' => self::TYPE_STRING, 'size' => 50),
        )
    );

    public function __construct($id_dfxcategory = null, $id_lang = null, $id_shop = null, Context $context = null)
    {
        parent::__construct($id_dfxcategory, $id_lang, $id_shop);
        
        if (!$this->position) {
            $this->position = 1 + $this->reOrderPositions();
        }
    }

    public function add($autodate = true, $null_values = false)
    {
        $context = Context::getContext();
        $id_shop = $context->shop->id;

        $res = parent::add($autodate, $null_values);
        $res &= Db::getInstance()->execute(
            '
			INSERT INTO `'._DB_PREFIX_.'dfxcategory_shop` (`id_shop`, `id_dfxcategory`)
			VALUES('.(int)$id_shop.', '.(int)$this->id.')'
        );
        return $res;
    }

    public function delete()
    {
        $res = true;

        $res &= $this->reOrderPositions();

        $res &= Db::getInstance()->execute(
            '
			DELETE FROM `'._DB_PREFIX_.'dfxcategory_shop`
			WHERE `id_dfxcategory` = '.(int)$this->id
        );

        $res &= parent::delete();
        return $res;
    }
    
    public function reOrderPositions()
    {
        $id_dfxcategory = $this->id;
        $context = Context::getContext();
        $id_shop = $context->shop->id;

        $max = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT MAX(hss.`position`) as position
			FROM `'._DB_PREFIX_.'dfxcategory` hss, `'._DB_PREFIX_.'dfxcategory_shop` hs
			WHERE hss.`id_dfxcategory` = hs.`id_dfxcategory` AND hs.`id_shop` = '.(int)$id_shop
        );

        if ((int)$max == (int)$id_dfxcategory) {
            return true;
        }

        $datarows = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT hss.`position` as position, hss.`id_dfxcategory` as id_dfxcategory
			FROM `'._DB_PREFIX_.'dfxcategory` hss
			LEFT JOIN `'._DB_PREFIX_.'dfxcategory_shop` hs ON (hss.`id_dfxcategory` = hs.`id_dfxcategory`)
			WHERE hs.`id_shop` = '.(int)$id_shop.' AND hss.`position` > '.(int)$this->position
        );

        foreach ($datarows as $datarow) {
            $current_category = new DfxCategorytabGroup($datarow['id_dfxcategory']);
            --$current_category->position;
            $current_category->update();
            unset($current_category);
        }

        return true;
    }

    public static function getAssociatedIdsShop($id_dfxcategory)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT hs.`id_shop`
			FROM `'._DB_PREFIX_.'dfxcategory_shop` hs
			WHERE hs.`id_dfxcategory` = '.(int)$id_dfxcategory
        );

        if (!is_array($result)) {
            return false;
        }

        $return = array();

        foreach ($result as $id_shop) {
            $return[] = (int)$id_shop['id_shop'];
        }

        return $return;
    }
}
