<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class DfxCompareActionsModuleFrontController extends ModuleFrontController
{
    public $id_product;

    public function init()
    {
        parent::init();

        $this->id_product = (int)Tools::getValue('id_product');
    }
    
    public function postProcess()
    {
        if (Tools::getValue('process') == 'remove') {
            $this->processRemove();
        } elseif (Tools::getValue('process') == 'add') {
            $this->processAdd();
        } elseif (Tools::getValue('process') == 'removeAll') {
            $this->processRemoveAll();
        }
    }

    /**
     * Remove a compare product.
     */
    public function processRemove()
    {
        $idProduct = (int)Tools::getValue('idProduct');
        $productsIds = $this->context->cookie->dfxCompare;
        $productsIds = Tools::jsonDecode($productsIds, true);
        unset($productsIds[$idProduct]);
        $productsIds = Tools::jsonEncode($productsIds, true);
        $this->context->cookie->__set('dfxCompare', $productsIds);
        $this->context->cookie->__set('dfxCompareNb', (int) $this->context->cookie->dfxCompareNb - 1);

        $this->ajaxDie(Tools::jsonEncode(array(
            'success' => true,
            'data' => array(
                'message' => $this->l('Product removed'),
                'type' => 'removed'
            )
        )));
    }

    /**
     * Remove all compare products.
     */
    public function processRemoveAll()
    {
        $productsIds = array();
        $productsIds = Tools::jsonEncode($productsIds, true);
        $this->context->cookie->__set('dfxCompare', $productsIds);
        $this->context->cookie->__set('dfxCompareNb', 0);

       $this->ajaxDie(Tools::jsonEncode(array(
            'success' => true,
            'data' => array(
                'message' => $this->l('All products removed'),
                'type' => 'removedAll'
            )
        )));
    }

    /**
     * Add a compare product.
     */
    public function processAdd()
    {
        $idProduct = (int)Tools::getValue('idProduct');

        $productsIds = $this->context->cookie->dfxCompare;
        $productsIds = Tools::jsonDecode($productsIds, true);

        if (!isset($productsIds[$idProduct])) {
            $productsIds[$idProduct] = $idProduct;
            $productsIds = Tools::jsonEncode($productsIds, true);

            $this->context->cookie->__set('dfxCompare', $productsIds);
            $this->context->cookie->__set('dfxCompareNb', (int) $this->context->cookie->dfxCompareNb + 1);

            $this->ajaxDie(Tools::jsonEncode(array(
                'success' => true,
                'data' => array(
                    'message' => $this->l('Product added to comparator'),
                    'type' => 'added'
                )
            )));
        }
		
    }

}
