<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;

class DfxCompareComparatorModuleFrontController extends ModuleFrontController
{
    public function init()
    {
        parent::init();
		$this->id_product = (int)Tools::getValue('id_product');
    }

    public function initContent()
    {
        parent::initContent();

        $presentedCompareProducts = array();
        $compareProducts = array();
        $orderedFeatures = array();
        $listFeatures = array();

        $idLang = (int)$this->context->language->id;
        $idShop = (int)$this->context->shop->id;
        $productsIds = $this->context->cookie->dfxCompare;

        if ($productsIds) {
            $productsIds = Tools::jsonDecode($productsIds, true);
            foreach ($productsIds as $idProduct) {
                $product =  new Product($idProduct, false, $idLang, $idShop, $this->context);

                if (Validate::isLoadedObject($product)) {
                    $product->id_product = $product->id;
                    $compareProducts[] = (array) $product;
                }
            }

            $presenterFactory = new ProductPresenterFactory($this->context);
            $presentationSettings = $presenterFactory->getPresentationSettings();

            $presenter = new ProductListingPresenter(
                new ImageRetriever(
                    $this->context->link
                ),
                $this->context->link,
                new PriceFormatter(),
                new ProductColorsRetriever(),
                $this->getTranslator()
            );

            if (is_array($compareProducts)) {
                foreach ($compareProducts as $item) {
                    $presentedProduct = $presenter->present(
                        $presentationSettings,
                        Product::getProductProperties($this->context->language->id, $item, $this->context),
                        $this->context->language
                    );
                
                    $presentedCompareProducts[] = $presentedProduct;
                    foreach ($presentedProduct['features'] as $feature) {
                        $listFeatures[$presentedProduct['id_product']][$feature['id_feature']] = $feature['value'];
                    }
                }
            }
            $orderedFeatures = $this->module->getFeaturesForComparison($productsIds, $idLang);
        }


        $this->context->smarty->assign(array(
            'compareProducts' => $presentedCompareProducts,
            'orderedFeatures' => $orderedFeatures,
            'listFeatures' => $listFeatures
        ));

        $this->setTemplate('module:dfxcompare/views/templates/front/comparator.tpl');
    }
}
