<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'dfxcookie/classes/DfxCookieclass.php';

class DfxCookie extends Module
{
    private $templateFile;

    public function __construct()
    {
        $this->name = 'dfxcookie';
        $this->tab = 'front_office_features';
        $this->author = 'DigitalRedfox';
        $this->version = '4.1.1';
        $this->need_instance = 0;

        $this->bootstrap = true;
        parent::__construct();

        Shop::addTableAssociation('dfxcookie', ['type' => 'shop']);

        $this->displayName = $this->trans('Dfx Cookielaw', [], 'Modules.DfxCookie.Admin');
        $this->description = $this->trans('Cookie confirmation block', [], 'Modules.DfxCookie.Admin');

        $this->ps_versions_compliancy = ['min' => '1.7.4.0', 'max' => _PS_VERSION_];

        $this->templateFile = 'module:dfxcookie/views/templates/hook/dfxcookie.tpl';
    }

    public function install()
    {
        return $this->runInstallSteps()
            && $this->installFixtures();
    }

    public function runInstallSteps()
    {
        return parent::install()
            && $this->installDB()
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayBeforeBodyClosingTag')
            && $this->registerHook('actionShopDataDuplication');
    }

    public function uninstall()
    {
        return parent::uninstall() && $this->uninstallDB();
    }

    public function installDB()
    {
        $return = true;
        $return &= Db::getInstance()->execute(
            '
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'dfxcookie` (
                `id_dfxcookie` INT UNSIGNED NOT NULL AUTO_INCREMENT,
                PRIMARY KEY (`id_dfxcookie`)
            ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;'
        );

        $return &= Db::getInstance()->execute(
            '
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'dfxcookie_shop` (
                `id_dfxcookie` INT(10) UNSIGNED NOT NULL,
                `id_shop` INT(10) UNSIGNED NOT NULL,
                PRIMARY KEY (`id_dfxcookie`, `id_shop`)
            ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;'
        );

        $return &= Db::getInstance()->execute(
            '
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'dfxcookie_lang` (
                `id_dfxcookie` INT UNSIGNED NOT NULL,
                `id_shop` INT(10) UNSIGNED NOT NULL,
                `id_lang` INT(10) UNSIGNED NOT NULL ,
                `content` varchar(255) NOT NULL,
				`link` text NOT NULL,
                PRIMARY KEY (`id_dfxcookie`, `id_lang`, `id_shop`)
            ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;'
        );

        return $return;
    }

    public function uninstallDB($drop_table = true)
    {
        $ret = true;
        if ($drop_table) {
            $ret &= Db::getInstance()->execute('DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'dfxcookie`')
                && Db::getInstance()->execute('DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'dfxcookie_shop`')
                && Db::getInstance()->execute('DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'dfxcookie_lang`');
        }

        return $ret;
    }

    public function getContent()
    {
        $output = '';

        if (Tools::isSubmit('savedfxcookie')) {
            if (!Tools::getValue('content_' . (int) Configuration::get('PS_LANG_DEFAULT'), false)) {
                $output = $this->displayError($this->trans('Please fill out all fields.', [], 'Admin.Notifications.Error'));
            }
            if (!Tools::getValue('link_' . (int) Configuration::get('PS_LANG_DEFAULT'), false)) {
                $output = $this->displayError($this->trans('Please fill out all fields.', [], 'Admin.Notifications.Error'));
            } else {
                $update = $this->processSaveDfxcookie();

                if (!$update) {
                    $output = '<div class="alert alert-danger conf error">'
                        . $this->trans('An error occurred on saving.', [], 'Admin.Notifications.Error')
                        . '</div>';
                }

                $this->_clearCache($this->templateFile);
            }
        }

        return $output . $this->renderForm();
    }

    public function processSaveDfxcookie()
    {
        $shops = Tools::getValue('checkBoxShopAsso_configuration', [$this->context->shop->id]);
        $content = [];
        $link = [];
        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
            $content[$lang['id_lang']] = (string) Tools::getValue('content_' . $lang['id_lang']);
            $link[$lang['id_lang']] = (string) Tools::getValue('link_' . $lang['id_lang']);
        }

        $saved = true;
        foreach ($shops as $shop) {
            Shop::setContext(Shop::CONTEXT_SHOP, $shop);
            $dfxcookie = new DfxCookieclass(Tools::getValue('id_dfxcookie', 1));
            $dfxcookie->content = $content;
            $dfxcookie->link = $link;
            $saved &= $dfxcookie->save();
        }

        return $saved;
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->addJS(($this->_path).'views/js/cookie.js');
        $this->context->controller->addCSS(($this->_path).'views/css/cookieclass.css');
    }

    protected function renderForm()
    {
        $default_lang = (int) Configuration::get('PS_LANG_DEFAULT');

        $fields_form = [
            'tinymce' => true,
            'legend' => [
                'title' => $this->trans('Cookie Law', [], 'Modules.Xipixel_cookie.Admin'),
            ],
            'input' => [
                'id_dfxcookie' => [
                    'type' => 'hidden',
                    'name' => 'id_dfxcookie',
                ],
                'content' => [
                    'type' => 'textarea',
                    'label' => $this->trans('Text block', [], 'Modules.Xipixel_cookie.Admin'),
                    'lang' => true,
                    'name' => 'content',
                    'cols' => 40,
                    'rows' => 10,
                    'class' => 'rte',
                    'autoload_rte' => true,
                ],
                 'link' => [
                    'type' => 'text',
                    'label' => $this->trans('Text block', [], 'Modules.Xipixel_cookie.Admin'),
                    'lang' => true,
                    'name' => 'link',
                ],
            ],
            'submit' => [
                'title' => $this->trans('Save', [], 'Admin.Actions'),
            ],
            'buttons' => [
                [
                    'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                    'title' => $this->trans('Back to list', [], 'Admin.Actions'),
                    'icon' => 'process-icon-back',
                ],
            ],
        ];

        if (Shop::isFeatureActive() && Tools::getValue('id_dfxcookie') == false) {
            $fields_form['input'][] = [
                'type' => 'shop',
                'label' => $this->trans('Shop association', [], 'Admin.Global'),
                'name' => 'checkBoxShopAsso_theme',
            ];
        }

        $helper = new HelperForm();
        $helper->module = $this;
        $helper->name_controller = 'xipixel_cookie';
        $helper->identifier = $this->identifier;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        foreach (Language::getLanguages(false) as $lang) {
            $helper->languages[] = [
                'id_lang' => $lang['id_lang'],
                'iso_code' => $lang['iso_code'],
                'name' => $lang['name'],
                'is_default' => ($default_lang == $lang['id_lang'] ? 1 : 0),
            ];
        }

        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        $helper->toolbar_scroll = true;
        $helper->title = $this->displayName;
        $helper->submit_action = 'savedfxcookie';

        $helper->fields_value = $this->getFormValues();

        return $helper->generateForm([['form' => $fields_form]]);
    }

    public function getFormValues()
    {
        $fields_value = [];
        $idShop = $this->context->shop->id;
        $idDfxcookie = DfxCookieclass::getDfxcookieIdByShop($idShop);

        Shop::setContext(Shop::CONTEXT_SHOP, $idShop);
        $dfxcookie = new DfxCookieclass((int) $idDfxcookie);
		
		$languages = Language::getLanguages(false);
        foreach ($languages as $lang) {
		 $fields_value['content'][$lang['id_lang']] = Tools::getValue('content_'.(int)$lang['id_lang'], $dfxcookie->content[$lang['id_lang']]);
		 $fields_value['link'][$lang['id_lang']] = Tools::getValue('link_'.(int)$lang['id_lang'], $dfxcookie->link[$lang['id_lang']]);
        }
        $fields_value['id_dfxcookie'] = $idDfxcookie;

        return $fields_value;
    }

    public function getCookietext()
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;
        $id_lang = $this->context->language->id;

        $res_prod = Db::getInstance()->getRow('
            SELECT hssl.*
            FROM '._DB_PREFIX_.'dfxcookie_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxcookie hss ON (hss.id_dfxcookie = hs.id_dfxcookie)
            LEFT JOIN '._DB_PREFIX_.'dfxcookie_lang hssl ON (hss.id_dfxcookie = hssl.id_dfxcookie)
            WHERE hs.id_shop = '.(int)$id_shop.'
            AND hssl.id_lang = '.(int)$id_lang
        );

        $this->smarty->assign(array(
			'cookies' => $res_prod,
		));
    }

    public function hookDisplayBeforeBodyClosingTag()
    {
		 if (!$this->isCached($this->templateFile, $this->getCacheId('dfxcookie'))) {
            $this->getCookietext();
        }
        return $this->fetch($this->templateFile, $this->getCacheId('dfxcookie'));
	}
    public function installFixtures()
    {
        $return = true;
        $tabTexts = [
            [
                'content' => '<p>This website stores data such as cookies to enable site functionality including analytics and personalization. By using this website, you automatically accept that we use cookies. </p>',
                'link' => 'content/3-terms-and-conditions-of-use',
            ],
        ];

        $shopsIds = Shop::getShops(true, null, true);
        $languages = Language::getLanguages(false);
        $content = [];
        $link = [];

        foreach ($tabTexts as $tab) {
            $dfxcookie = new DfxCookieclass();
            foreach ($languages as $lang) {
                $content[$lang['id_lang']] = $tab['content'];
                $link[$lang['id_lang']] = $tab['link'];
            }
            $dfxcookie->content = $content;
            $dfxcookie->link = $link;
            $return &= $dfxcookie->add();
        }

        if ($return && count($shopsIds) > 1) {
            foreach ($shopsIds as $idShop) {
                Shop::setContext(Shop::CONTEXT_SHOP, $idShop);
                $dfxcookie->content = $content;
                $dfxcookie->link = $link;
                $return &= $dfxcookie->save();
            }
        }

        return $return;
    }

    /**
     * Add CustomText when adding a new Shop
     *
     * @param array $params
     */
    public function hookActionShopDataDuplication($params)
    {
        if ($infoId = DfxCookieclass::getDfxcookieIdByShop($params['old_id_shop'])) {
            Shop::setContext(Shop::CONTEXT_SHOP, $params['old_id_shop']);
            $oldInfo = new DfxCookieclass($infoId);

            Shop::setContext(Shop::CONTEXT_SHOP, $params['new_id_shop']);
            $newInfo = new DfxCookieclass($infoId, null, $params['new_id_shop']);
            $newInfo->content = $oldInfo->content;
            $newInfo->link = $oldInfo->link;

            $newInfo->save();
        }
    }
}
