<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 * International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Adapter\NewProducts\NewProductsProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;
use PrestaShop\PrestaShop\Adapter\BestSales\BestSalesProductSearchProvider;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

include_once(_PS_MODULE_DIR_.'dfxisotopecategory/classes/DfxIsotopecategoryClass.php');

class DfxIsotopecategory extends Module implements WidgetInterface
{
	protected $_html = '';
	protected $default_options = 0;
	protected $order_by = 'name';
	protected $order_way = 'ASC';
    protected $link;
	public $confirmUninstall;
    private $templateFile;
    
    public function __construct()
    {
        $this->name = 'dfxisotopecategory';
        $this->tab = 'front_office_features';
        $this->author = 'DigitalRedfox';
        $this->version = '4.1.2';
        $this->need_instance = 0;

        $this->ps_versions_compliancy = array('min' => '1.7.2.0', 'max' => _PS_VERSION_);

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->trans('Dfx Isotope Category', array(), 'Modules.DfxIsotopecategory.Admin');
        $this->description = $this->trans('Displays Isotope products in Homepage.', array(), 'Modules.DfxIsotopecategory.Admin');
		
		$this->confirmUninstall = $this->trans('Are you sure you want to uninstall?', array(), 'Modules.DfxIsotopecategory.Admin');
		
        $this->templateFile = 'module:dfxisotopecategory/views/templates/hook/dfxisotopecategory.tpl';
    }

    public function install()
    {
		if (parent::install() &&
            $this->registerHook('actionProductAdd') &&
            $this->registerHook('actionProductUpdate') &&
			$this->registerHook('actionProductDelete') &&
			$this->registerHook('actionCategoryAdd') &&
			$this->registerHook('actionCategoryUpdate') &&
			$this->registerHook('actionCategoryDelete') &&
			$this->registerHook('updateOrderStatus') &&
			$this->registerHook('updateOrderStatus') &&
			$this->registerHook('displayHome') &&
			$this->registerHook('displayHeader') &&
            $this->registerHook('actionShopDataDuplication') &&
			$this->registerHook('actionObjectLanguageAddAfter') 
        ) {
            $shops = Shop::getContextListShopID();
            $shop_groups_list = array();

            /* Setup each shop */
            foreach ($shops as $shop_id) {
                $shop_group_id = (int)Shop::getGroupFromShop($shop_id, true);

                if (!in_array($shop_group_id, $shop_groups_list)) {
                    $shop_groups_list[] = $shop_group_id;
                }

                /* Sets up configuration */
                $res = Configuration::updateValue('BP_CATOGORIES_ALL', $this->default_options, false, $shop_group_id, $shop_id);
				$res &= Configuration::updateValue('ORDER_BY', $this->order_by, false, $shop_group_id, $shop_id);
				$res &= Configuration::updateValue('ORDER_WAY', $this->order_way, false, $shop_group_id, $shop_id);
            }

            /* Sets up Shop Group configuration */
            if (count($shop_groups_list)) {
                foreach ($shop_groups_list as $shop_group_id) {
					$res &= Configuration::updateValue('BP_CATOGORIES_ALL', $this->default_options, false, $shop_group_id);
					$res &= Configuration::updateValue('ORDER_BY', $this->order_by, false, $shop_group_id);
					$res &= Configuration::updateValue('ORDER_WAY', $this->order_way, false, $shop_group_id);
                }
            }

            /* Sets up Global configuration */
			$res &= Configuration::updateValue('BP_CATOGORIES_ALL', $this->default_options);
			$res &= Configuration::updateValue('ORDER_BY', $this->order_by);
			$res &= Configuration::updateValue('ORDER_WAY', $this->order_way);

            /* Creates tables */
            $res &= $this->createTables();

            /* Adds samples */
            if ($res) {
                $this->installSamples();
            }

            return (bool)$res;
        }

        return false;
    }
	
	protected function installSamples()
    {
        $res = 1;
        include_once dirname(__FILE__).'/sql/sample.php';
        return $res;
    }

    public function createTables()
    {
		 /* Isotopecategory Shop */
        $res = (bool)Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxisotopecategory_shop` (
                `id_dfxisotopecategory` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `id_shop` int(10) unsigned NOT NULL,
                PRIMARY KEY (`id_dfxisotopecategory`, `id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Isotopecategory configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxisotopecategory` (
              `id_dfxisotopecategory` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `position` int(10) unsigned NOT NULL DEFAULT \'0\',
			  `status` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `tab_type` varchar(50) NOT NULL,
			  `prod_count` varchar(255) DEFAULT \'10\',
			  `products` text DEFAULT NULL,
			  `cat_id` varchar(150) DEFAULT NULL,
              PRIMARY KEY (`id_dfxisotopecategory`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Isotopecategory lang configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxisotopecategory_lang` (
              `id_dfxisotopecategory` int(10) unsigned NOT NULL,
              `id_lang` int(10) unsigned NOT NULL,
              `name` varchar(255) NOT NULL,
			  `alias_name` varchar(255) NOT NULL,
              PRIMARY KEY (`id_dfxisotopecategory`,`id_lang`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        return $res;
    }


	 public function uninstall()
    {
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();
			$res &= Configuration::deleteByName('BP_CATOGORIES_ALL');
			$res &= Configuration::deleteByName('ORDER_BY');
			$res &= Configuration::deleteByName('ORDER_WAY');
            return (bool)$res;
        }

        return false;
    }
	
	 public function deleteTables()
    {
        $tabs = $this->getIsotopetabs();
        foreach ($tabs as $tab) {
            $to_del = new DfxIsotopecategoryClass($tab['id_dfxisotopecategory']);
            $to_del->delete();
        }

        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxisotopecategory_shop`, `'._DB_PREFIX_.'dfxisotopecategory`, `'._DB_PREFIX_.'dfxisotopecategory_lang`;
        ');
    }

    public function hookDisplayHeader($params)
    {
        if ('index' === $this->context->controller->php_self) {
            $jsFile = 'modules/'.$this->name.'/views/js/front.js';
            $attributes = array(
            'position' => 'bottom',
            'priority' => 10
            );

            $this->context->controller->registerJavascript($this->name.'-isotope', 'modules/'.$this->name.'/views/js/jquery.isotope.min.js', array('position' => 'bottom', 'priority' => 8));
            $this->context->controller->registerJavascript($this->name, $jsFile, $attributes);
            $display_rtl = $this->context->language->is_rtl;
			if($display_rtl) {
			$this->context->controller->addCSS($this->_path.'views/css/front_rtl.css');
			} else {
            $this->context->controller->addCSS($this->_path.'views/css/front.css');
			}
        }
    }
	
    public function getContent()
    {
		$tabid = (int)Tools::getValue('tabid');
		
		if (isset($tabid) && $tabid != '') {
		  if (Tools::isSubmit('submitTab_'.$tabid)) 
		  {
			  if ($this->_postValidation($tabid)) {
				  $this->_postProcess();
			  }
		  } elseif (Tools::isSubmit('id_tab') && $this->tabIdExists((int)Tools::getValue('id_tab'))) {
		    
			  $mode = 'edit';
			  $associated_shop_ids = DfxIsotopecategoryClass::getAssociatedIdsShop((int)Tools::getValue('id_tab'));
              $context_shop_id = (int)Shop::getContextShopID();

                if ($associated_shop_ids === false) {
                    $this->_html .= $this->getShopAssociationError((int)Tools::getValue('id_tab'));
                } elseif (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL && in_array($context_shop_id, $associated_shop_ids)) {
                    if (count($associated_shop_ids) > 1) {
                        $this->_html = $this->getSharedTabWarning();
                    }
                } else {
                    $shops_name_list = array();
                    foreach ($associated_shop_ids as $shop_id) {
                        $associated_shop = new Shop((int)$shop_id);
                        $shops_name_list[] = $associated_shop->name;
                    }
                    $this->_html .= $this->getShopContextError($shops_name_list, $mode);
                }
		 }
		} elseif (Tools::isSubmit('submitAddCategory')) {
			$mode = 'add';
			if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
               $this->_postProcess();
            } else {
                $this->_html .= $this->getShopContextError(null, $mode);
            }
		} elseif (Tools::isSubmit('submitEnableSettings')) {
            $this->_postProcess();
        } else {
            $this->_html .= $this->getWarningMultishopHtml().$this->getCurrentShopInfoMsg();
        }

		return $this->_html.$this->renderNewTabForm().$this->renderSettingsForm().$this->renderList();
	}
	
	protected function _postValidation($tabid)
    {
        $errors = array();
	
		if (Tools::isSubmit('submitTab_'.$tabid)) {

            /* If edit : checks id_tab */
            if (Tools::isSubmit('id_tab')) {
                if (!Validate::isInt(Tools::getValue('id_tab')) && !$this->tabIdExists(Tools::getValue('id_tab'))) {
                    $errors[] = $this->getTranslator()->trans('Invalid tab ID', array(), 'Modules.DfxIsotopecategory.Admin');
                }
            }
			
			 $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
				if (Tools::strlen(Tools::getValue('aliasname_'.$tabid.'_'.$language['id_lang'])) > 255) {
                    $errors[] = $this->getTranslator()->trans('The title is too long.', array(), 'Modules.DfxIsotopecategory.Admin');
                }
            }
			if (Tools::strlen(Tools::getValue('prod_count_'.$tabid)) == '') {
				$errors[] = $this->getTranslator()->trans('The Product count is not set.', array(), 'Modules.DfxIsotopecategory.Admin');
			}
			$id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            if (Tools::strlen(Tools::getValue('aliasname_'.$tabid.'_'.$id_lang_default)) == '') {
                $errors[] = $this->getTranslator()->trans('The Tab name is not set.', array(), 'Modules.DfxIsotopecategory.Admin');
            }
			
		}
		
		/* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));

            return false;
        }

        /* Returns if validation is ok */

        return true;
		
	}
	
	public function _postProcess()
    {
		$languages = Language::getLanguages(false);
        $default_language = (int)Configuration::get('PS_LANG_DEFAULT');
        $id_shop = (int)Shop::getContextShopID();
		
		$errors = array();
        $shop_context = Shop::getContext();
		
		$tabid = (int)Tools::getValue('tabid');
		$sel_product = Tools::getValue('selProduct');
		
		if (Tools::isSubmit('submitEnableSettings')) {
            $shop_groups_list = array();
            $shops = Shop::getContextListShopID();

            foreach ($shops as $shop_id) {
                $shop_group_id = (int)Shop::getGroupFromShop($shop_id, true);

                if (!in_array($shop_group_id, $shop_groups_list)) {
                    $shop_groups_list[] = $shop_group_id;
                }
                $res = Configuration::updateValue('BP_CATOGORIES_ALL', (int)Tools::getValue('BP_CATOGORIES_ALL'), false, $shop_group_id, $shop_id);
				$res &= Configuration::updateValue('ORDER_BY', Tools::getValue('ORDER_BY'), false, $shop_group_id, $shop_id);
				$res &= Configuration::updateValue('ORDER_WAY', Tools::getValue('ORDER_WAY'), false, $shop_group_id, $shop_id);
            }

            /* Update global shop context if needed*/
            switch ($shop_context) {
                case Shop::CONTEXT_ALL:
                    $res &= Configuration::updateValue('BP_CATOGORIES_ALL', (int)Tools::getValue('BP_CATOGORIES_ALL'));
					$res &= Configuration::updateValue('ORDER_BY', Tools::getValue('ORDER_BY'));
                    if (count($shop_groups_list)) {
                        foreach ($shop_groups_list as $shop_group_id) {
                            $res &= Configuration::updateValue('BP_CATOGORIES_ALL', (int)Tools::getValue('BP_CATOGORIES_ALL'), false, $shop_group_id);
							$res &= Configuration::updateValue('ORDER_BY', Tools::getValue('ORDER_BY'), false, $shop_group_id);
							$res &= Configuration::updateValue('ORDER_WAY', Tools::getValue('ORDER_WAY'), false, $shop_group_id);
                        }
                    }
                    break;
                case Shop::CONTEXT_GROUP:
                    if (count($shop_groups_list)) {
                        foreach ($shop_groups_list as $shop_group_id) {
                            $res &= Configuration::updateValue('BP_CATOGORIES_ALL', (int)Tools::getValue('BP_CATOGORIES_ALL'), false, $shop_group_id);
							$res &= Configuration::updateValue('ORDER_BY', Tools::getValue('ORDER_BY'), false, $shop_group_id);
							$res &= Configuration::updateValue('ORDER_WAY', Tools::getValue('ORDER_WAY'), false, $shop_group_id);
                        }
                    }
                    break;
            }

            if (!$res) {
                $errors[] = $this->displayError($this->getTranslator()->trans('The configuration could not be updated.', array(), 'Modules.DfxIsotopecategory.Admin'));
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=6&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        } elseif (Tools::isSubmit('submitTab_'.$tabid)) {
				$Isotopetabs = new DfxIsotopecategoryClass();
				if($tabid) {
				$Isotopetabs = new DfxIsotopecategoryClass($tabid);
				}
				$Isotopetabs->tab_type = Tools::getValue('type_'.$tabid);
				$Isotopetabs->status = (int)Tools::getValue('status_'.$tabid);
				
	            $cat_id = $Isotopetabs->cat_id;
				$Isotopetabs->cat_id = ($cat_id)?$cat_id:0;
				$prod_count = Tools::getValue('prod_count_'.$tabid);
				$Isotopetabs->prod_count = $prod_count;

				foreach ($languages as $language) {
				$Isotopetabs->alias_name[$language['id_lang']] = Tools::getValue('aliasname_'.$tabid.'_'.$language['id_lang']);
   			   }
	
				if ($Isotopetabs->tab_type == 'manual') {
					$Isotopetabs->products = $sel_product;
				} else {
					$Isotopetabs->products = '';
				}
				
				 if (!$errors) {
                /* Adds */
					if (!$Isotopetabs->update()) {
						$errors[] = $this->displayError($this->trans('The tab could not be updated.', array(), 'Modules.DfxIsotopecategory.Admin'));
					} else {
                        $this->_html .= $this->displayConfirmation($this->trans('Settings Updated.'));
					}
				}
		} 
		
		if (Tools::isSubmit('submitAddCategory')) {
            $selected_tab = Tools::getValue('rootCat');
            $val_arr = array();
            $val_arr = explode('_', $selected_tab);
			
			if($tabid) {
		     $Isotopetabs = new DfxIsotopecategoryClass($tabid);
			 if (!Validate::isLoadedObject($Isotopetabs)) {
                $this->_html .= $this->displayError($this->trans('Invalid tab ID', array(), 'Modules.DfxIsotopecategory.Admin'));
                return false;
              }
		    } else {
                $Isotopetabs = new DfxIsotopecategoryClass();
				$Isotopetabs->position = (int)$this->getNextPosition();
            }

            $Isotopetabs->status = 1;


			$Isotopetabs->tab_type = $val_arr[1];
		   			
			$flag_exists = 0;
            if ($val_arr[1] !== 'manual') {
                $flag_exists = $this->tabExist($val_arr[0]);
			}
            if ($flag_exists) {
                $errors[] = $val_arr[0].' '.$this->trans('tab is already added.');
            } else {
				foreach ($languages as $l) {
					if ($val_arr[1] == 'category') {
						$category = new Category($val_arr[2], (int)$l['id_lang']);
						$Isotopetabs->name[$l['id_lang']] = $val_arr[0];
						$Isotopetabs->alias_name[$l['id_lang']] = trim($category->name);
						$Isotopetabs->cat_id = $val_arr[2] ? $val_arr[2] : 0;
					} elseif ($val_arr[1] == 'brands') {
						$manufacturer = new Manufacturer((int)$val_arr[2], (int)$l['id_lang']);
						$Isotopetabs->name[$l['id_lang']] = trim($val_arr[0]);
						$Isotopetabs->alias_name[$l['id_lang']] = trim($manufacturer->name);
						$Isotopetabs->cat_id = $val_arr[2] ? $val_arr[2] : 0;
					} else {
						$Isotopetabs->name[$l['id_lang']] = trim($val_arr[0]);
						$Isotopetabs->alias_name[$l['id_lang']] = trim($val_arr[0]);
						$Isotopetabs->cat_id = 0;
					}
				}
			}
           if (!$errors) {
					if (!Tools::getValue('tabid')) {
						if (!$Isotopetabs->add()) {
							$errors[] = $this->displayError($this->trans('The tab could not be added.', array(), 'Modules.DfxIsotopecategory.Admin'));
						}
					} elseif (!$Isotopetabs->update()) {
						$errors[] = $this->displayError($this->trans('The tab could not be updated.', array(), 'Modules.DfxIsotopecategory.Admin'));
					} else {
						$this->_html .= $this->displayConfirmation($this->trans('New section is added successfully.'));
					}
         	}

        }
       
         if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        }
	}
	
    public function renderNewTabForm()
    { 
	
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Category Configuration', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs',
                ),


                'description' => $this->trans('To add products to your homepage, simply add them to the corresponding product category (default: "Home").'),
                'input' => array(
                    array(
                        'type' => 'selectsection',
                        'label' => $this->trans('Add Section'),
                        'name' => 'rootCat',
                        'class' => 'fixed-width-xs',
                        'desc' => $this->trans('Add item to the list.'),
                    ),
                ),
                
                'submit' => array(
                     'title' => $this->trans('Add Tab', array(), 'Admin.Actions'),
                     'name' => 'submitAddCategory',
                ),
            ),
        );

        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
		
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAddCategory';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'cat_options' => $this->getAllCatLink(),
        );
        return $helper->generateForm(array($fields_form));
    }
		public function getConfigFieldsValues()
    {
        return array(
            'rootCat' => 'Manual Products_manual'
        );
    }
    
    public function getAllCatLink($id_lang = null)
    {
        if (is_null($id_lang)) {
            $id_lang = (int)$this->context->language->id;
        }
        $html = '<optgroup label="'.$this->trans('Categories').'">';
        $html .= $this->getCategoryOption(1, $id_lang, false, true);
        $html .= '</optgroup>';
        $html .= '<optgroup label="' . $this->trans('Brands', [], 'Admin.Global') . '">';
        // Option to show all Manufacturers
        $manufacturers = Manufacturer::getManufacturers(false, $this->context->language->id);
        foreach ($manufacturers as $manufacturer) {
            if ($manufacturer['id_manufacturer']) {
                $html .= '<option value="'.$manufacturer['name'].'_brands_' . $manufacturer['id_manufacturer'] . '">' . $manufacturer['name'] . '</option>';
            }
        }
        $html .= '</optgroup>';
        return $html;
    }
    
    public function getCategoryOption($id_category = 1, $id_lang = false, $id_shop = false, $recursive = true)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $id_shop = $id_shop ? (int)$id_shop : (int)Context::getContext()->shop->id;
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
        if (is_null($category->id)) {
            return;
        }
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
            $spacer = str_repeat('&nbsp;', 3 * (int)$category->level_depth);
        }

        if (!in_array($category->id, array(Configuration::get('PS_HOME_CATEGORY'), Configuration::get('PS_ROOT_CATEGORY')))) {

                $html .= '<option value="'.$category->name.'_category_'.(int)$category->id.'">'.str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'</option>';

        }
        
        if (isset($children) && count($children)) {
            foreach ($children as $child) {
                $html .= $this->getCategoryOption(
                    (int)$child['id_category'],
                    (int)$id_lang,
                    (int)$child['id_shop'],
                    $recursive
                );
            }
        }
        return $html;
    }
    public function renderSettingsForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings'),
                    'icon' => 'icon-cogs'
                ),
                'description' => $this->trans('Configure Settings.'),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('"All" Tab Enable / Disable'),
                        'name' => 'BP_CATOGORIES_ALL',
                        'desc' => $this->trans('You can enable or disable "ALL" tab on Home page'),
                        'values' => array(
                                    array(
                                        'id' => 'active_on',
                                        'value' => 1,
                                        'label' => $this->trans('Enabled')
                                    ),
                                    array(
                                        'id' => 'active_off',
                                        'value' => 0,
                                        'label' => $this->trans('Disabled')
                                    )
                                ),
                    ),
					array(
                        'type' => 'selectby',
                        'label' => $this->trans('Products to display by default'),
                        'name' => 'ORDER_BY',
						'class' => 'fixed-width-xs',
                        'desc' => $this->trans('Products to display by'),
                    ),
					array(
                        'type' => 'selectway',
                        'label' => $this->trans('Products order way'),
                        'name' => 'ORDER_WAY',
						'class' => 'fixed-width-xs',
                        'desc' => $this->trans('Products order way'),
                    ),

                ),
                'submit' => array(
                    'title' => $this->trans('Save'),
                )
            ),
        );


	
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
		$helper->module = $this;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitEnableSettings';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValuesSettings(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
			'order_by' => Tools::getValue('ORDER_BY', Configuration::get('ORDER_BY')),
			'order_way' => Tools::getValue('ORDER_WAY', Configuration::get('ORDER_WAY')),
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValuesSettings()
    {
		$id_shop_group = Shop::getContextShopGroupID();
        $id_shop = Shop::getContextShopID();
		
        return array(
            'BP_CATOGORIES_ALL' => Tools::getValue('BP_CATOGORIES_ALL', Configuration::get('BP_CATOGORIES_ALL', null, $id_shop_group, $id_shop)),
			'ORDER_BY' => Tools::getValue('ORDER_BY', Configuration::get('ORDER_BY', null, $id_shop_group, $id_shop)),
			'ORDER_WAY' => Tools::getValue('ORDER_WAY', Configuration::get('ORDER_WAY', null, $id_shop_group, $id_shop)),
        );
    }
     public function renderList()
    {
        $this->context->controller->addCSS(($this->_path).'views/css/back.css', 'all');
        
        $this->context->controller->addJqueryUI('ui.sortable');

        $this->context->controller->addJS($this->_path.'views/js/back.js');
        
        $id_lang = (int)Context::getContext()->language->id;
        $id_shop = (int)Shop::getContextShopID();
        $languages = Language::getLanguages(false);
		$tabAlias = array();
		$res_prod_arr = array();
        $customPrds = array();
		
        $tabs = $this->getIsotopetabs();
		foreach ($tabs as $tab) {
			$Isotopetabs = new DfxIsotopecategoryClass($tab['id_tab']);
			foreach ($languages as $lang) {
            	$tabAlias[$tab['id_tab']][$lang['id_lang']] = $Isotopetabs->alias_name[$lang['id_lang']];
				if ($tab['id_lang'] == $lang['id_lang']) {
					
					if ($tab['tab_type'] == "manual") {

                    $res_prod = $tab['products'];
						if ($res_prod) {
							$res_prod_arr = explode(',', $res_prod);
							$customPrds[$tab['id_tab']] = $res_prod_arr;
						}
                	}
				}
				
        	}
		}
		
        $manualPrds = array();
        $productObj = new Product();
        $res_all_prod = $productObj->getProducts($id_lang, 0, 0, 'id_product', 'ASC');
        $prodList = array();
        
		
        foreach ($res_all_prod as $prod) {
            $prodList[$prod['id_product']] = $prod['name'];
        }


        $this->context->smarty->assign(
            array(
                'tabs' => $tabs,
				'tabAlias' => $tabAlias,
                'languages' => $languages,
                'defaultFormLanguage' => $id_lang,
                'res_all_prod' => $res_all_prod,
                'manualPrds' => $manualPrds,
                'prodList' => $prodList,
                'customPrds' => $customPrds,
				'link' => $this->context->link,
                'ajaxPath' => _MODULE_DIR_.$this->name
            )
        );

        return $this->display(__FILE__, 'views/templates/admin/list.tpl');
    }
	public function getIsotopetabs($status = null)
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;
        $id_lang = $this->context->language->id;

        $tabs = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            'SELECT hs.id_dfxisotopecategory as id_tab, hss.*, hssl.*
            FROM '._DB_PREFIX_.'dfxisotopecategory_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxisotopecategory hss ON (hs.id_dfxisotopecategory = hss.id_dfxisotopecategory)
            LEFT JOIN '._DB_PREFIX_.'dfxisotopecategory_lang hssl ON (hss.id_dfxisotopecategory = hssl.id_dfxisotopecategory)
            WHERE id_shop = '.(int)$id_shop.'
            AND hssl.id_lang = '.(int)$id_lang.
            ($status ? ' AND hss.`status` = 1' : ' ').'
            ORDER BY hss.position'
        );
        return $tabs;
    }

	public function getNextPosition()
    {
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
            SELECT MAX(hss.`position`) AS `next_position`
            FROM `'._DB_PREFIX_.'dfxisotopecategory` hss, `'._DB_PREFIX_.'dfxisotopecategory_shop` hs
            WHERE hss.`id_dfxisotopecategory` = hs.`id_dfxisotopecategory` AND hs.`id_shop` = '.(int)$this->context->shop->id
        );

        return (++$row['next_position']);
    }
	
	public function tabExist($name) {
		$id_shop = (int)Shop::getContextShopID();
		$sql_sel = 'SELECT COUNT(*) FROM `'._DB_PREFIX_.'dfxisotopecategory_lang` hsl
				LEFT JOIN '._DB_PREFIX_.'dfxisotopecategory_shop hss ON (hsl.id_dfxisotopecategory = hss.id_dfxisotopecategory) 
				WHERE hss.`id_shop` = '.$id_shop.' AND hsl.`name` = "'.$name.'"';
		$flag_exists = Db::getInstance()->getValue($sql_sel);		
		return $flag_exists;
	}
	
	public function tabIdExist($id_tab) {
		$req = 'SELECT hs.`id_dfxisotopecategory` as id_tab
                FROM `'._DB_PREFIX_.'dfxisotopecategory_shop` hs
                WHERE hs.`id_dfxisotopecategory` = '.(int)$id_tab;
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($req);

        return ($row);
	}

      public function renderWidget($hookName = null, array $configuration = [])
    {
		if (!preg_match('/^displayHeader\d*$/', $hookName)) {

				$variables = $this->getWidgetVariables($hookName, $configuration);
	
				if (empty($variables)) {
					return false;
				}
	
				$this->smarty->assign($variables);


        return $this->fetch($this->templateFile);
		}
    }

    public function getWidgetVariables($hookName = null, array $configuration = [])
    {
        $products = $this->getIsotopeProducts();

		if (!empty($products)) {
			return $products;
        }
        return false;
    }
  
    protected function getIsotopeProducts()
    {
        $cat_id = $prod_count = $type = $cat_display = $id_tab = $alias_name = $cat_display_name = array();
        $allproductid = $allproductids = $brandproductids = $category_ids = $brand_ids = $customtab = array();
        $mproducts_ids = $arr_merged = [];
        $bestsale_id = array();
        $tabclass = [];

		$orderby = Tools::getValue('ORDER_BY', Configuration::get('ORDER_BY'));
		$orderway = Tools::getValue('ORDER_WAY', Configuration::get('ORDER_WAY'));
		
       $resultant = $this->getIsotopetabs(true);

            foreach ($resultant as $results) {
				$cat_id = $results['cat_id'];
				$id_tab = $results['id_tab'];
				$prod_count = $results['prod_count'];
				$tab_type = explode(',', trim($results['tab_type'], ','));
				$alias_name = $results['alias_name'];
				
                if (in_array('category', $tab_type)) {
                     $category_ids[] = $id_tab;

                     $allproductids = $this->getCatproductIds($cat_id, $orderby, $orderway);
							
					$sel_prod_cnt = count($allproductids);
					
						if ($sel_prod_cnt > $prod_count)
							$allproductids = array_slice($allproductids, 0, $prod_count);
									
                     if (isset($allproductids) && $allproductids) {
                        $cat_display_name[$id_tab] = $alias_name;
                        $cat_display[$id_tab] = $this->assignName($alias_name);
                        $tabclass[$id_tab] = $allproductids;
                     }
                } elseif (in_array('brands', $tab_type)) {
                    $brand_ids[] = $id_tab;
                    $brandproductids = $this->getManufactureIds($cat_id, $orderby, $orderway);

					$sel_prod_cnt = count($brandproductids);

						if ($sel_prod_cnt > $prod_count)
							$brandproductids = array_slice($brandproductids, 0, $prod_count);
							
                    if (isset($brandproductids) && $brandproductids) {
                        $cat_display_name[$id_tab] = $alias_name;
                        $cat_display[$id_tab] = $this->assignName($alias_name);
                        $tabclass[$id_tab] = $brandproductids;
                    }
                } elseif (in_array('Featured', $tab_type)) {
                    $featuredIds = $this->getCatproductIds((int)Configuration::get('PS_HOME_CATEGORY'), $orderby, $orderway);
					
					$sel_prod_cnt = count($featuredIds);
					
						if ($sel_prod_cnt > $prod_count) {
							$featuredId = array_slice($featuredIds, 0, $prod_count);
						}
									
                    if (isset($featuredId) && $featuredId) {
                        $feature_id = $id_tab;
                        $cat_display_name[$id_tab] = $alias_name;
                        $cat_display[$id_tab] = $this->assignName($alias_name);
                        $tabclass[$id_tab] = $featuredId;
                    }
                } elseif (in_array('New', $tab_type)) {
					if($orderby == 'RAND()') {
					$newproductIds = $this->getNewproductIds($orderway);

					$sel_prod_cnt = count($newproductIds);

						if ($sel_prod_cnt > $prod_count) {
							$newproductIds = array_slice($newproductIds, 0, $prod_count);
						}
					} else {
                    	$newproductIds = Product::getNewProducts(Context::getContext()->language->id, 0, ($prod_count ? $prod_count : 10), false, $orderby, $orderway);
					}
				
                    if (isset($newproductIds) && $newproductIds) {
                        $new_id = $id_tab;
                        $cat_display_name[$id_tab] = $alias_name;
                        $cat_display[$id_tab] = $this->assignName($alias_name);

						$tabclass[$id_tab] = $this->assignIDs($newproductIds);

                    }
                } elseif (in_array('Topsellers', $tab_type)) {
                    $bestsellers_Ids = $this->getBestSellers($prod_count);

                    if (isset($bestsellers_Ids) && $bestsellers_Ids) {
                        $bestsale_id = $id_tab;
                        $cat_display_name[$id_tab] = $alias_name;
                        $cat_display[$id_tab] = $this->assignName($alias_name);
                        $tabclass[$id_tab] = $this->assignIDs($bestsellers_Ids);
                    }
                } elseif (in_array('Specials', $tab_type)) {

                  $specialIds = Product::getPricesDrop((int)Context::getContext()->language->id, 0, ($prod_count ? $prod_count : 10));
				 
                    if (isset($specialIds) && $specialIds) {
                        $special_id = $id_tab;
                        $cat_display_name[$id_tab] = $alias_name;
                        $cat_display[$id_tab] = $this->assignName($alias_name);
                        $tabclass[$id_tab] = $this->assignIDs($specialIds);
                    }
                } elseif (in_array('manual', $tab_type)) {
                    $mproducts_ids = explode(',', $results['products']);
                    if ($mproducts_ids) {
                        $customtab[] = $id_tab;
                        $cat_display[$id_tab] = 'custom_'.str_replace(' ', '_', $id_tab);
                        $cat_display_name[$id_tab] = $alias_name;

						
				$mproductsIDs = $this->getProductselection($mproducts_ids, true, $orderby, $orderway);

                        $sel_prod_cnt = count($mproductsIDs);
                        
                        if ($sel_prod_cnt > $prod_count) {
                            $mproductsIDs = array_slice($mproductsIDs, 0, $prod_count);
                        }
                        
                        $tabclass[$id_tab] = $mproductsIDs;
                    }
                }
            }


        foreach ($tabclass as $ids) {
            $arr_merged = array_merge($arr_merged, $ids);
        }
        $productids_unique = array_unique($arr_merged);


		$all_prods = $this->getProductByIds($productids_unique, $orderby, $orderway);

        $allprod = $dispAll = array();
        if (!empty($all_prods)) {
            foreach ($all_prods as $product) {
                $product['tab_class'] = '';
                if (isset($category_ids) && !empty($category_ids)) {
                    foreach ($category_ids as $catid) {
                        if (in_array($product['id_product'], $tabclass[$catid])) {
                            $product['tab_class'] .= ' '.$cat_display[$catid];
                        }
                    }
                }
                if (isset($brand_ids) && !empty($brand_ids)) {
                    foreach ($brand_ids as $brandid) {
                        if (in_array($product['id_product'], $tabclass[$brandid])) {
                            $product['tab_class'] .= ' '.$cat_display[$brandid];
                        }
                    }
                }
                if (isset($feature_id) && !empty($feature_id)) {
                    if (in_array($product['id_product'], $tabclass[$feature_id])) {
                        $product['tab_class'] .= ' '.$cat_display[$feature_id];
                    }
                }
                if (isset($new_id) && !empty($new_id)) {
                    if (in_array($product['id_product'], $tabclass[$new_id])) {
                        $product['tab_class'] .= ' '.$cat_display[$new_id];
                    }
                }
                if (isset($bestsale_id) && !empty($bestsale_id)) {
                    if (in_array($product['id_product'], $tabclass[$bestsale_id])) {
                        $product['tab_class'] .= ' '.$cat_display[$bestsale_id];
                    }
                }
                      
                if (isset($special_id) && !empty($special_id)) {
                    if (in_array($product['id_product'], $tabclass[$special_id])) {
                        $product['tab_class'] .= ' '.$cat_display[$special_id];
                    }
                }
                if (isset($customtab) && !empty($customtab)) {
                    foreach ($customtab as $idtab) {
                        if (in_array($product['id_product'], $tabclass[$idtab])) {
                            $product['tab_class'] .= ' '.$cat_display[$idtab];
                        }
                    }
                }
                                    
                $allprod[] = $product;
            }

        }
		$dispAll = Configuration::get('BP_CATOGORIES_ALL');
		return [
           'products' => $allprod,
           'dispAll' => $dispAll,
           'cat_display'=> $cat_display,
           'cat_display_name'=> $cat_display_name
        ];
    }
    
    private function assignIDs($productIds)
    {
        $assignIds = [];
        foreach ($productIds as $product) {
            if (isset($product['id_product'])) {
                $assignIds[] = $product['id_product'];
            }
        }
        return $assignIds;
    }
    private function assignName($alias)
    {
        $assignName = [];
        $string = [];
        
        $string = str_replace(' ', '-', Tools::strtolower($alias));
        $assignName = preg_replace('/[^A-Za-z0-9\-]/', '-', $string);
        /*$replaced_alias = preg_replace('/-+/', '-', $string);*/
                    
        return $assignName;
    }
        
    public function getCatproductIds($cat_id, $order_by, $order_way)
    {
        if (empty($cat_id)) {
            return;
        }

		if (!Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }
		if (empty($order_by) || $order_by == 'position') {
            $order_by = 'price';
        }
        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_product' || $order_by == 'price' || $order_by == 'date_add' || $order_by == 'date_upd') {
            $order_by_prefix = 'p';
        } elseif ($order_by == 'name') {
            $order_by_prefix = 'pl';
        } elseif ($order_by == 'position') {
            $order_by_prefix = 'cp';
        } elseif ($order_by == 'quantity') {
			$order_by_prefix = 'stock';
		} 
		
        if (strpos($order_by, '.') > 0) {
            $order_by = explode('.', $order_by);
            $order_by_prefix = $order_by[0];
            $order_by = $order_by[1];
        }
		
		if ($order_by == 'RAND()') {
			 $sql = '
		SELECT cp.`id_product`
        FROM `' . _DB_PREFIX_ . 'product` p
		LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (p.`id_product` = cp.`id_product`)
        WHERE cp.`id_category` = ' . (int) $cat_id. ' AND p.`active` = 1 AND p.`visibility` != \'none\' ORDER BY RAND() '.$order_way;
        } elseif ($order_by == 'quantity') {
			$sql = '
		SELECT cp.`id_product` 
        FROM `' . _DB_PREFIX_ . 'product` p
		LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (p.`id_product` = cp.`id_product`)
		LEFT JOIN `'._DB_PREFIX_.'stock_available` stock ON (stock.`id_product` = cp.`id_product`) AND stock.id_product_attribute = 0
        WHERE stock.`id_product` = cp.`id_product` AND cp.`id_category` = ' . (int) $cat_id. ' AND p.`active` = 1 AND p.`visibility` != \'none\' ORDER BY ' . (isset($order_by_prefix) ? pSQL($order_by_prefix) . '.' : '') . '`' . $order_by . '` ' . pSQL($order_way);
		} else {
			$sql = '
		SELECT cp.`id_product` 
        FROM `' . _DB_PREFIX_ . 'product` p
		LEFT JOIN `'._DB_PREFIX_.'product_sale` ps ON (p.`id_product` = ps.`id_product`)
		LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (ps.`id_product` = cp.`id_product`)
		LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (cp.`id_product` = pl.`id_product` AND pl.`id_lang` = '.(int)$this->context->language->id.Shop::addSqlRestrictionOnLang('pl').')
        WHERE cp.`id_category` = ' . (int) $cat_id. ' AND p.`active` = 1 AND p.`visibility` != \'none\' ORDER BY ' . (isset($order_by_prefix) ? pSQL($order_by_prefix) . '.' : '') . '`' . $order_by . '` ' . pSQL($order_way);
        }
		
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (!$result) {
            return [];
        }

        return $this->assignIDs($result);
    }
	
	public function getNewproductIds($order_way)
    {
		if (!Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }

        if (empty($order_way)) {
            $order_way = 'DESC';
        }
		$nb_days_new_product = (int) Configuration::get('PS_NB_DAYS_NEW_PRODUCT');
		
    			 $sql = '
		SELECT p.`id_product` as id_product
        FROM `' . _DB_PREFIX_ . 'product` p
		' . Shop::addSqlAssociation('product', 'p') . '
		WHERE product_shop.`active` = 1
                    AND product_shop.`date_add` > "' . date('Y-m-d', strtotime('-' . $nb_days_new_product . ' DAY')) . '" AND p.`visibility` != \'none\' ORDER BY RAND() '.$order_way;
		
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (!$result) {
            return [];
        }

        return $result;
    }
	
    public function getManufactureIds($brand_id, $order_by, $order_way)
    {
        if (empty($brand_id)) {
            return;
        }
		
		$id_lang = (int)$this->context->language->id;
		if (!Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }
		if (empty($order_by) || $order_by == 'position') {
            $order_by = 'price';
        }
        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_product' || $order_by == 'price' || $order_by == 'date_add' || $order_by == 'date_upd') {
            $order_by_prefix = 'p';
        } elseif ($order_by == 'name') {
            $order_by_prefix = 'pl';
        } elseif ($order_by == 'position') {
            $order_by_prefix = 'cp';
        } elseif ($order_by == 'quantity') {
			$order_by_prefix = 'stock';
		} 
		
        if (strpos($order_by, '.') > 0) {
            $order_by = explode('.', $order_by);
            $order_by_prefix = $order_by[0];
            $order_by = $order_by[1];
        }
		
		if ($order_by == 'RAND()') {
			 $sql = '
		SELECT p.`id_product` as id_product
        FROM `' . _DB_PREFIX_ . 'product` p
		LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (p.`id_product` = cp.`id_product`)
        WHERE p.`id_manufacturer` = ' . (int) $brand_id. ' AND p.`active` = 1 AND p.`visibility` != \'none\' ORDER BY RAND() '.$order_way.';';
        } elseif ($order_by == 'quantity') {
			$sql = '
		SELECT p.`id_product` as id_product
        FROM `' . _DB_PREFIX_ . 'product` p
		LEFT JOIN `'._DB_PREFIX_.'stock_available` stock ON (stock.`id_product` = p.`id_product`) AND stock.id_product_attribute = 0
        WHERE p.`id_manufacturer` = ' . (int) $brand_id. ' AND p.`active` = 1 AND p.`visibility` != \'none\' ORDER BY ' . (isset($order_by_prefix) ? pSQL($order_by_prefix) . '.' : '') . '`' . $order_by . '` ' . pSQL($order_way.';');
		} else {
			$sql = '
		SELECT p.`id_product` as id_product 
        FROM `' . _DB_PREFIX_ . 'product` p
		LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (p.`id_product` = pl.`id_product` AND pl.`id_lang` = '.(int)$this->context->language->id.Shop::addSqlRestrictionOnLang('pl').')
        WHERE p.`id_manufacturer` = ' . (int) $brand_id. ' AND p.`active` = 1 AND p.`visibility` != \'none\' ORDER BY ' . (isset($order_by_prefix) ? pSQL($order_by_prefix) . '.' : '') . '`' . $order_by . '` ' . pSQL($order_way);
        }
				
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (!$result) {
            return [];
        }
		foreach ($result as &$row) {
            $row['id_product_attribute'] = Product::getDefaultAttribute((int)$row['id_product']);
        }
        return $this->assignIDs($result);
    }
    
	 public function getProductselection($productId, $active = true, $order_by, $order_way)
    {
        $product_ids = join(',', $productId);
		$id_shop = $this->context->shop->id;
		$id_lang = $this->context->language->id;
		if (!Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }
		if (empty($order_by) || $order_by == 'position') {
            $order_by = 'price';
        }
        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_product' || $order_by == 'price' || $order_by == 'date_add' || $order_by == 'date_upd') {
            $order_by_prefix = 'p';
        } elseif ($order_by == 'name') {
            $order_by_prefix = 'pl';
        } elseif ($order_by == 'position') {
            $order_by_prefix = 'cp';
        } elseif ($order_by == 'quantity') {
			$order_by_prefix = 'stock';
		} 
		
        if (strpos($order_by, '.') > 0) {
            $order_by = explode('.', $order_by);
            $order_by_prefix = $order_by[0];
            $order_by = $order_by[1];
        }
		
		 $sql = 'SELECT p.id_product
				FROM  `'._DB_PREFIX_.'product` p 
				'.Shop::addSqlAssociation('product', 'p').'
				LEFT JOIN `'._DB_PREFIX_.'product_attribute_shop` product_attribute_shop
					ON (p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop='.(int)$id_shop.')
				LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (
					p.`id_product` = pl.`id_product`
					AND pl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('pl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'product_sale` ps ON (
					pl.`id_product` = ps.`id_product`
				)
				LEFT JOIN `' . _DB_PREFIX_ . 'category_product` c ON (c.`id_product` = p.`id_product`)
				LEFT JOIN `'._DB_PREFIX_.'category_lang` cl ON (
					product_shop.`id_category_default` = cl.`id_category`
					AND cl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('cl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'manufacturer` m ON (p.`id_manufacturer`= m.`id_manufacturer`)
				'.Product::sqlStock('p', 0).'
				WHERE p.id_product IN ('.$product_ids.') AND c.`id_category` = cl.`id_category`'.
            ($active ? ' AND product_shop.`active` = 1 AND product_shop.`visibility` != \'none\'' : '');
			
				
				
		if ($order_by == 'RAND()') {
            $sql .= ' ORDER BY RAND() '.pSQL($order_way);
        } else {
            $sql .= ' ORDER BY ' . (isset($order_by_prefix) ? pSQL($order_by_prefix) . '.' : '') . '`' . pSQL($order_by) . '` ' . pSQL($order_way);
        }
		
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql, true, false);

        if (!$result) {
            return [];
        }

        return $this->assignIDs($result);
    }
    
    public function getBestSellers($prod_count = null)
    {
        if (Configuration::get('PS_CATALOG_MODE')) {
            return false;
        }

        $searchProvider = new BestSalesProductSearchProvider(
            $this->context->getTranslator()
        );

        $context = new ProductSearchContext($this->context);

        $query = new ProductSearchQuery();

        $query
            ->setResultsPerPage($prod_count)
            ->setPage(1)
        ;

        $query->setSortOrder(SortOrder::random());

        $result = $searchProvider->runQuery(
            $context,
            $query
        );

        $assembler = new ProductAssembler($this->context);

        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );

        $products_for_template = array();

        foreach ($result->getProducts() as $rawProduct) {
            $products_for_template[] = $presenter->present(
                $presentationSettings,
                $assembler->assembleProduct($rawProduct),
                $this->context->language
            );
        }

        return $products_for_template;
    }
    
    public function getProductByIds($productId, $orderby, $orderway)
    {
        if (!is_array($productId)) {
            return;
        }
        if (empty($productId)) {
            return;
        }

        $products = $this->getProductsInfoByIds($productId, $this->context->language->id, true, $orderby, $orderway);
        
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();

        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );


        if (is_array($products)) {
            foreach ($products as &$product) {
                $product = $presenter->present(
                    $presentationSettings,
                    Product::getProductProperties($this->context->language->id, $product, $this->context),
                    $this->context->language
                );
            }
            unset($product);
        }

        return $products;
    }

    public function getProductsInfoByIds($productId, $id_lang, $active = true, $order_by, $order_way)
    {
        $product_ids = join(',', $productId);

        $id_shop = (int) $this->context->shop->id;
		if (!Validate::isOrderWay($order_way)) {
            die(Tools::displayError());
        }
		if (empty($order_by) || $order_by == 'position') {
            $order_by = 'price';
        }
        if (empty($order_way)) {
            $order_way = 'DESC';
        }
        if ($order_by == 'id_product' || $order_by == 'price' || $order_by == 'date_add' || $order_by == 'date_upd') {
            $order_by_prefix = 'p';
        } elseif ($order_by == 'name') {
            $order_by_prefix = 'pl';
        } elseif ($order_by == 'position') {
            $order_by_prefix = 'c';
        } elseif ($order_by == 'quantity') {
			$order_by_prefix = 'stock';
		} 
		
        if (strpos($order_by, '.') > 0) {
            $order_by = explode('.', $order_by);
            $order_by_prefix = $order_by[0];
            $order_by = $order_by[1];
        }
       $sql = 'SELECT p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) as quantity, pl.`description`, pl.`description_short`, pl.`link_rewrite`,
					pl.`meta_description`, pl.`meta_keywords`, pl.`meta_title`, pl.`name`, pl.`available_now`, pl.`available_later`, pl.`delivery_in_stock`, pl.`delivery_out_stock`, ps.`quantity` AS sales, p.`additional_delivery_times`, image_shop.`id_image` id_image, il.`legend`, m.`name` as manufacturer_name, cl.`name` AS category_default, IFNULL(product_attribute_shop.id_product_attribute, 0) id_product_attribute,
					DATEDIFF(
						p.`date_add`,
						DATE_SUB(
							"'.date('Y-m-d').' 00:00:00",
							INTERVAL '.(Validate::isUnsignedInt(Configuration::get('PS_NB_DAYS_NEW_PRODUCT')) ? Configuration::get('PS_NB_DAYS_NEW_PRODUCT') : 20).' DAY
						)
					) > 0 AS new
				FROM  `'._DB_PREFIX_.'product` p 
				'.Shop::addSqlAssociation('product', 'p').'
				LEFT JOIN `'._DB_PREFIX_.'product_attribute_shop` product_attribute_shop
					ON (p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop='.(int)$id_shop.')
				LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (
					p.`id_product` = pl.`id_product`
					AND pl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('pl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'product_sale` ps ON (
					pl.`id_product` = ps.`id_product`
				)
				LEFT JOIN `' . _DB_PREFIX_ . 'category_product` c ON (c.`id_product` = p.`id_product`)
				LEFT JOIN `'._DB_PREFIX_.'category_lang` cl ON (
					product_shop.`id_category_default` = cl.`id_category`
					AND cl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('cl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'image_shop` image_shop
					ON (image_shop.`id_product` = p.`id_product` AND image_shop.cover=1 AND image_shop.id_shop='.(int)$id_shop.')
				LEFT JOIN `'._DB_PREFIX_.'image_lang` il ON (image_shop.`id_image` = il.`id_image` AND il.`id_lang` = '.(int)$id_lang.')
				LEFT JOIN `'._DB_PREFIX_.'manufacturer` m ON (p.`id_manufacturer`= m.`id_manufacturer`)
				'.Product::sqlStock('p', 0).'
				WHERE p.id_product IN ('.$product_ids.') AND c.`id_category` = cl.`id_category`'.
            ($active ? ' AND product_shop.`active` = 1 AND product_shop.`visibility` != \'none\'' : '');
			
				
				
		if ($order_by == 'RAND()') {
            $sql .= ' ORDER BY RAND() '.pSQL($order_way);
        } else {
            $sql .= ' ORDER BY ' . (isset($order_by_prefix) ? pSQL($order_by_prefix) . '.' : '') . '`' . pSQL($order_by) . '` ' . pSQL($order_way);
        }
		
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql, true, false);

        if (!$result) {
            return [];
        }
		
		if ($order_by == 'price') {
            Tools::orderbyPrice($result, $order_way);
        }
		
        foreach ($result as &$row) {
            $row['id_product_attribute'] = Product::getDefaultAttribute((int)$row['id_product']);
        }
        return Product::getProductsProperties($id_lang, $result);
    }
	
	  public function hookActionShopDataDuplication($params)
    {
        Db::getInstance()->execute('
            INSERT IGNORE INTO '._DB_PREFIX_.'dfxisotopecategory_shop (id_dfxisotopecategory, id_shop)
            SELECT id_dfxisotopecategory, '.(int)$params['new_id_shop'].'
            FROM '._DB_PREFIX_.'dfxisotopecategory_shop
            WHERE id_shop = '.(int)$params['old_id_shop']
        );
    }
	
	protected function getShopContextError($shop_contextualized_name, $mode)
    {
        if (is_array($shop_contextualized_name)) {
            $shop_contextualized_name = implode('<br/>', $shop_contextualized_name);
        }

        if ($mode == 'edit') {
            return '<p class="alert alert-danger">' .
            $this->trans('You can only edit this tab from the shop(s) context: %s', array($shop_contextualized_name), 'Modules.Dfxisotopecategory.Admin') .
            '</p>';
        } else {
            return '<p class="alert alert-danger">' .
            $this->trans('You cannot add tabs from a "All Shops" or a "Group Shop" context', array(), 'Modules.Dfxisotopecategory.Admin') .
            '</p>';
        }
    }
	
	protected function getShopAssociationError($id_tab)
    {
        return '<p class="alert alert-danger">'.
                        $this->trans('Unable to get isotopetab shop association information (id_tab: %d)', array((int)$id_tab), 'Modules.Dfxisotopecategory.Admin') .
                '</p>';
    }
	
	protected function getSharedTabWarning()
    {
        return '<p class="alert alert-warning">'.
                    $this->trans('This tab is shared with other shops! All shops associated to this tab will apply modifications made here', array(), 'Modules.Dfxisotopecategory.Admin').
                '</p>';
    }
	
	protected function getWarningMultishopHtml()
    {
        if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
            return '<p class="alert alert-warning">' .
            $this->getTranslator()->trans('You cannot manage tabs items from a "All Shops" or a "Group Shop" context, select directly the shop you want to edit', array(), 'Modules.Dfxisotopecategory.Admin') .
            '</p>';
        } else {
            return '';
        }
    }
	
	protected function getCurrentShopInfoMsg()
    {
        $shop_info = null;

        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_SHOP) {
                $shop_info = $this->trans('The modifications will be applied to shop: %s', array($this->context->shop->name),'Modules.Dfxisotopecategory.Admin');
            } else if (Shop::getContext() == Shop::CONTEXT_GROUP) {
                $shop_info = $this->trans('The modifications will be applied to this group: %s', array(Shop::getContextShopGroup()->name), 'Modules.Dfxisotopecategory.Admin');
            } else {
                $shop_info = $this->trans('The modifications will be applied to all shops and shop groups', array(), 'Modules.Dfxisotopecategory.Admin');
            }

            return '<div class="alert alert-info">'.
                        $shop_info.
                    '</div>';
        } else {
            return '';
        }
    }
	
	public function hookActionObjectLanguageAddAfter($params)
    {
        return $this->installSamples(array(array('id_lang' => (int)$params['object']->id)));
    }

}
