<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class DfxManufacture extends Module
{
    public function __construct()
    {
        $this->name = 'dfxmanufacture';
        $this->tab = 'front_office_features';
        $this->version = '1.7.6';
        $this->author = 'Digitalredfox';
        $this->module_key = '';
        parent::__construct();
        $this->displayName = $this->getTranslator()->trans('DFX Manufacturer Logo Slider', array(), 'Modules.Dfx_Manufacture.Admin');
        $this->description = $this->getTranslator()->trans('Adds Slider of Manufacturer logo on your site.', array(), 'Modules.Dfx_Manufacture.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.0.0', 'max' => _PS_VERSION_);
    }

    public function install()
    {
        $this->_clearCache('*');
    
        return parent::install()
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayFooterBefore')
        ;
    }
	
	public function uninstall()
    {
        $this->_clearCache('*');

        return parent::uninstall();
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->registerJavascript('modules-manufacture', 'modules/'.$this->name.'/views/js/front.js', ['position' => 'bottom', 'priority' => 160]);
    }
    public function hookDisplayFooterBefore()
    {
        $list_manufacture = Manufacturer::getManufacturers();
        $manufactures = array();

        if ($list_manufacture) {
            foreach ($list_manufacture as $item) {
                $item['image'] = $this->context->link->getManufacturerImageLink($item['id_manufacturer'], 'manufacturer_default');
                $item['url'] = $this->context->link->getManufacturerLink($item['id_manufacturer'], $item['link_rewrite']);

                if (file_exists(_PS_MANU_IMG_DIR_.$item['id_manufacturer'].'.jpg')) {
                    $manufactures[] = $item;
                }
            }
        }

        $image_size = ImageType::getByNameNType('manufacturer_default', 'manufacturers');

        $this->smarty->assign(array(
            'manufactures' => $manufactures,
            'imageSize' => $image_size,
        ));
		
		return $this->fetch('module:'.$this->name.'/views/templates/hook/'.$this->name.'.tpl');
    }

}
