<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class DfxmenuColumn extends ObjectModel
{
    public $id;
    public $title;
    public $url;
    public $width;
    public $position;
    public $active;
    public $col_title;
    public $content_type;
    public $category_list;
    public $cat_thumb;
    public $products;
    public $manufacturers;
    public $cmslinks;
    public $id_dfxmenucolumn;
    public $id_dfxmenurow;
    public $banner_image;
    public $banner_image_w;
    public $banner_image_h;
    public $banner_url;
     
    public $static_content;
    public $prod_count;
    public $short_product;

    public static $definition = array(
        'table' => 'dfxmenucolumn',
        'primary' => 'id_dfxmenucolumn',
        'multilang' => true,
        'fields' => array(
            'id_dfxmenurow' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt', 'required' => true),
            'title' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'required' => true, 'size' => 255),
            'url' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isUrl', 'size' => 255),
            'width' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'position' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt', 'required' => true),
            'active' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt'),
            'content_type' => array('type' => self::TYPE_STRING, 'size' => 50),
            'category_list' => array('type' => self::TYPE_STRING, 'size' => 50),
            'cat_thumb' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt'),
            'col_title' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt'),
            'products' => array('type' => self::TYPE_STRING),
            'static_content' => array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml'),
            'manufacturers' => array('type' => self::TYPE_STRING),
            'banner_image' => array('type' => self::TYPE_STRING, 'size' => 128),
            'banner_image_w' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'banner_image_h' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'banner_url' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isUrl', 'size' => 255),
            'cmslinks' => array('type' => self::TYPE_STRING),
            'prod_count' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'short_product' => array('type' => self::TYPE_STRING, 'size' => 50),
        )
    );

    public function __construct($id_dfxmenurow = null, $id_dfxmenucolumn = null, $id_lang = null, $id_shop = null, Context $context = null)
    {
        parent::__construct($id_dfxmenucolumn, $id_lang, $id_shop);
        if ($id_dfxmenurow) {
            $this->id_dfxmenurow = $id_dfxmenurow;
        }
        
        if (!$this->position) {
            $this->position = 1 + $this->reOrderPositions();
        }
    }

    public function add($autodate = true, $null_values = false)
    {
        $context = Context::getContext();
        $id_shop = $context->shop->id;
        
        $res = parent::add($autodate, $null_values);
        $res &= Db::getInstance()->execute(
            '
			INSERT INTO `'._DB_PREFIX_.'dfxmenucolumn_shop` (`id_shop`, `id_dfxmenurow`, `id_dfxmenucolumn`)
			VALUES('.(int)$id_shop.', '.(int)$this->id_dfxmenurow.', '.(int)$this->id.')'
        );
        return $res;
    }

    public function delete()
    {
        $res = true;
        
        $res &= Db::getInstance()->execute(
            '
			DELETE FROM `'._DB_PREFIX_.'dfxmenucolumn_shop`
			WHERE `id_dfxmenucolumn` = '.(int)$this->id
        );
        
        $res &= parent::delete();
        return $res;
    }

    public function reOrderPositions()
    {
        $context = Context::getContext();
    
        $query = 'SELECT MAX(hss.`position`)
            FROM `'._DB_PREFIX_.'dfxmenucolumn` hss
            WHERE hss.`id_dfxmenucolumn` = '.(int) $this->id;

        return (int) Db::getInstance()->getValue($query);
    }
}
