<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once _PS_MODULE_DIR_ . 'dfxpopup/classes/Popletter.php';

class DfxPopup extends Module
{
    /**
     * @var string Name of the module running on PS 1.6.x. Used for data migration.
     */
    protected $html = '';
    private $templateFile;

    public function __construct()
    {
        $this->name = 'dfxpopup';
        $this->tab = 'front_office_features';
        $this->version = '2.1.1';
        $this->author = 'DigitalRedfox';
        $this->need_instance = 0;
        $this->controllers = ['subscription'];

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->trans('Dfx Popup Newsletter', array(), 'Modules.DfxPopup.Admin');
        $this->description = $this->trans('Popup Newsletter to promot your offer to your customer.', array(), 'Modules.DfxPopup.Admin');

        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);

        $this->templateFile = 'module:dfxpopup/views/templates/hook/dfxpopup.tpl';
    }

    public function install()
    {
        Configuration::updateValue('logo_enable', 1);
        /* Adds Module */
        if (parent::install() &&
            $this->registerHook('actionFrontControllerSetMedia') &&
            $this->registerHook('displayBeforeBodyClosingTag')) {

            /* Creates tables */
            $res = $this->installDB();

            /* Adds samples */
            if ($res) {
                $this->installFixtures();
            }

            return (bool)$res;
        }

        return false;
    }

    /**
     * Migrate data from 1.6 equivalent module (if applicable), then uninstall
     */

    protected function installFixtures()
    {
        $return = true;
        
        $image = array();
        $description = array();
        $languages = Language::getLanguages(false);
		$id_lang = $this->context->language->id;
        $popletter = Popletter::getdfxpopupIdByShop();
        foreach ($languages as $lang) {
            $image[$lang['id_lang']] = 'dfxpopupimage.jpg';
            $description[$lang['id_lang']] = '<h1>Newsletter</h1><p>Join over 100 people who get free and fresh content delivered automatically each time we publish</p>';
        }
        $popletter->image = $image;
        $popletter->description = $description;
        $return &= $popletter->add();

        return $return;
    }
  
    public function installDB()
    {
        $return = true;
        $return &= Db::getInstance()->execute(
            '
                CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxpopup` (
                `id_dfxpopup` INT UNSIGNED NOT NULL AUTO_INCREMENT,
				`id_shop` int(10) unsigned NOT NULL DEFAULT 1,
                PRIMARY KEY (`id_dfxpopup`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;'
        );

        $return &= Db::getInstance()->execute(
            '
                CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxpopup_lang` (
                `id_dfxpopup` INT UNSIGNED NOT NULL,
                `id_lang` INT(10) UNSIGNED NOT NULL ,
				`image` varchar(255) NOT NULL,
				`description` text DEFAULT NULL,
                PRIMARY KEY (`id_dfxpopup`, `id_lang`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;'
        );

        return $return;
    }
    
    public function uninstall()
    {
        return parent::uninstall() && $this->uninstallDB();
    }
    
    public function uninstallDB($drop_table = true)
    {
        $ret = true;
        if ($drop_table) {
            $ret &= Db::getInstance()->execute('DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxpopup`')
                && Db::getInstance()->execute('DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxpopup_lang`');
        }

        return $ret;
    }

    public function postProcess()
    {
        $languages = Language::getLanguages(false);
        $id_lang_default = (int) Configuration::get('PS_LANG_DEFAULT');
		$id_lang = $this->context->language->id;
            
        $values = array();
        $update_images_values = false;
        Configuration::updateValue('logo_enable', Tools::getValue('logo_enable'));
		
        $popletter = Popletter::getdfxpopupIdByShop($id_lang);
            
        foreach ($languages as $lang) {
            if (isset($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']])
                    && isset($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']]['tmp_name'])
                    && !empty($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']]['tmp_name'])) {
                if ($error = ImageManager::validateUpload($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']], 4000000)) {
                    $this->html .= $this->displayError($error);
                } else {
                    $ext = Tools::substr($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']]['name'], strrpos($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']]['name'], '.') + 1);
                    $file_name = md5($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']]['name']).'.'.$ext;

                    if (!move_uploaded_file($_FILES['DFXPOPUP_IMG_'.$lang['id_lang']]['tmp_name'], dirname(__FILE__).DIRECTORY_SEPARATOR. 'views'.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR.$file_name)) {
                        $this->html .= $this->displayError($this->trans('An error occurred while attempting to upload the file.', array(), 'Admin.Notifications.Error'));
                    } else {
                        if (Configuration::hasContext('DFXPOPUP_IMG', $lang['id_lang'], Shop::getContext())
                                && Configuration::get('DFXPOPUP_IMG', $lang['id_lang']) != $file_name) {
                            @unlink(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'views' .  DIRECTORY_SEPARATOR . 'img' . DIRECTORY_SEPARATOR . Configuration::get('DFXPOPUP_IMG', $lang['id_lang']));
                        }

                        $values['DFXPOPUP_IMG'][$lang['id_lang']] = $file_name;
                    }
                }

                $update_images_values = true;
            }

            $values['DFXPOPUP_DESC'][$lang['id_lang']] = Tools::getValue('DFXPOPUP_DESC_'.$lang['id_lang']);
            if (!$values['DFXPOPUP_DESC'][$lang['id_lang']]) {
                $values['DFXPOPUP_DESC'][$lang['id_lang']] = Tools::getValue('DFXPOPUP_DESC_'.$id_lang_default);
            }
        }

        if ($update_images_values) {
            $popletter->image = $values['DFXPOPUP_IMG'];
        }

        $popletter->description = $values['DFXPOPUP_DESC'];

        $this->_clearCache($this->templateFile);
            
        $result = $popletter->validateFields(false) && $popletter->validateFieldsLang(false);
            
        if ($result) {
            $popletter->save();
            $this->html .= $this->displayConfirmation($this->trans('The settings have been updated.', array(), 'Admin.Notifications.Success'));
        } else {
            $this->html .= $this->displayError($this->getTranslator()->trans(
                'An error occurred while attempting to save Settings.',
                array(),
                'Modules.DfxPopup.Admin'
            ));
        }

        return $result;
    }

    public function getContent()
    {
        if (Tools::isSubmit('submitPopup')) {
            $this->postProcess();
            return $this->renderForm();
        } else {
            return $this->renderForm();
        }
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'file_lang',
                        'label' => $this->trans('Newsletter image', array(), 'Modules.DfxPopup.Admin'),
                        'name' => 'DFXPOPUP_IMG',
                        'desc' => $this->trans('Upload an image for your popup newsletter', array(), 'Modules.DfxPopup.Admin'),
                        'lang' => true,
                    ),
                    array(
                        'type' => 'textarea',
                        'lang' => true,
                        'label' => $this->trans('Popup description', array(), 'Modules.DfxPopup.Admin'),
                        'name' => 'DFXPOPUP_DESC',
                        'autoload_rte' => true,
                        'desc' => $this->trans('Please enter a short but meaningful description for popup newsletter.', array(), 'Modules.DfxPopup.Admin')
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Shop Logo'),
                        'name' => 'logo_enable',
                        'desc' => $this->trans('You can enable Shop Logo on popup Newsletter'),
                        'values' => array(
                                    array(
                                        'id' => 'active_on',
                                        'value' => 1,
                                        'label' => $this->trans('Enabled')
                                    ),
                                    array(
                                        'id' => 'active_off',
                                        'value' => 0,
                                        'label' => $this->trans('Disabled')
                                    )
                                ),
                    ),
                     
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions')
                )
            ),
        );
        
        $popletter = Popletter::getdfxpopupIdByShop();
        
        $fields_form['form']['images'] = $popletter->image;

        $has_picture = true;

        foreach (Language::getLanguages(false) as $lang) {
            if (!isset($popletter->image[$lang['id_lang']])) {
                $has_picture &= false;
            }
        }

        if ($has_picture) {
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'has_picture');
        }


        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->module = $this;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPopup';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
             'base_url' => $this->context->shop->getBaseURL(),
             'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'image_baseurl' => $this->_path.'views/img/',
        );
        
        $languages = Language::getLanguages(false);

        
        if (count($languages) > 1) {
            return $this->getMultiLanguageInfoMsg() . $helper->generateForm(array($fields_form));
        } else {
            return $helper->generateForm(array($fields_form));
        }
    }

    public function getConfigFieldsValues()
    {
        $languages = Language::getLanguages(false);
        $fields = array();
        
        $popletter = Popletter::getdfxpopupIdByShop();
        $fields['has_picture'] = true;
        
        $fields['logo_enable'] = Tools::getValue('logo_enable', Configuration::get('logo_enable'));
        
        foreach ($languages as $lang) {
            $fields['DFXPOPUP_IMG'][$lang['id_lang']] = Tools::getValue('DFXPOPUP_IMG_'.(int)$lang['id_lang']);
            $default_content = isset($popletter->description[$lang['id_lang']]) ? $popletter->description[$lang['id_lang']] : '';
            $fields['DFXPOPUP_DESC'][$lang['id_lang']] = Tools::getValue('DFXPOPUP_DESC_'.(int) $lang['id_lang'], $default_content);
        }

        return $fields;
    }
    
    public function hookDisplayBeforeBodyClosingTag($params)
    {
        $id_lang = (int) $this->context->language->id;
        $popletter = Popletter::getdfxpopupIdByShop($id_lang);
        
        $name = 'ps_emailsubscription';
        $module_instance = Module::getInstanceByName($name);
        $subscribe_form = false;
    
        if (Module::isInstalled($name) && Module::isEnabled($name)) {
            $subscribe_form = $module_instance->renderWidget();
        }
            
        $imgname = $popletter->image;
             
        $dfxpopup_img = false;
        if ($imgname && file_exists(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'img'.DIRECTORY_SEPARATOR.$imgname)) {
            $dfxpopup_img = $this->context->link->protocol_content . Tools::getMediaServer($imgname) . $this->_path . 'views/img/' . $imgname;
        }
            
        $this->smarty->assign(array(
                'subscribe_form' => $subscribe_form,
                'dfxpopup_img' => $dfxpopup_img,
                'dfxpopup_desc' => $popletter->description,
                'logo_enable' => Configuration::get('logo_enable')
            ));

        return $this->fetch($this->templateFile);
    }
    protected function getMultiLanguageInfoMsg()
    {
        return '<p class="alert alert-warning">'.
                    $this->getTranslator()->trans('Since multiple languages are activated on your shop, please mind to upload your newsletter image for each one of them', array(), 'Modules.DfxPopup.Admin').
                '</p>';
    }

    public function hookActionFrontControllerSetMedia()
    {
        if ((isset(Context::getContext()->cookie->dfxpopup) && Context::getContext()->cookie->dfxpopup == true)) {
            return;
        }

        $this->context->controller->addCSS(_PS_MODULE_DIR_.$this->name."/views/css/dfxpopup.css");
        $this->context->controller->registerJavascript('modules-popletter', 'modules/'.$this->name.'/views/js/scripts.js', ['position' => 'bottom', 'priority' => 150]);
        $this->context->controller->registerJavascript('modules-popletter-2', 'modules/'.$this->name.'/views/js/jquery_cookie.js', ['position' => 'bottom', 'priority' => 152]);
        $this->context->controller->addJqueryPlugin('fancybox');
         
        $this->config['newsletterControllerURL'] = $this->context->link->getModuleLink($this->name, 'subscription');
        $this->config['msg']['nogdpr'] = addslashes($this->trans('You have to accept our GDPR policy', [], 'Modules.DfxPopup.Shop'));

        Media::addJsDef(['dfxpopup' => $this->config]);
    }
}
