<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

/**
 * @since   1.5.0
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;
use PrestaShop\PrestaShop\Adapter\BestSales\BestSalesProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\NewProducts\NewProductsProductSearchProvider;

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

include_once(_PS_MODULE_DIR_.'dfxshopbycategory/classes/DfxShopbycategorygroup.php');

class DfxShopbycategory extends Module implements WidgetInterface
{
    public $banner_img_folder = 'views/img/';
    public $_html = '';
    public $templateFile;
    public $currentIndex;

    public function __construct()
    {
        $this->name = 'dfxshopbycategory';
        $this->tab = 'front_office_features';
        $this->version = '2.1.2';
        $this->author = 'DigitalRedfox';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('Dfx Shop by Category', array(), 'Modules.DfxShopbycategory.Admin');
        $this->description = $this->trans('Shop by category', array(), 'Modules.DfxShopbycategory.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        
        $this->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        
        
        $this->templateFile = 'module:dfxshopbycategory/views/templates/hook/dfxshopbycategory.tpl';
    }

    /**
     * @see Module::install()
     */
    public function install()
    {
        /* Adds Module */
        if (parent::install() &&
            $this->registerHook('displayHeader') &&
            $this->registerHook('actionShopDataDuplication') &&
            $this->registerHook('displayHome')
        ) {
            /* Creates tables */
            $res = $this->createTables();

            /* Adds samples */
            if ($res) {
                $this->installSamples();
            }

            return (bool)$res;
        }

        return false;
    }

    /**
     * Adds samples
     */
    public function installSamples()
    {
        $languages = Language::getLanguages(false);
        $values = array();
        $categorytab = new DfxShopbycategorygroup();
            
        for ($i = 1; $i < 2; $i++) {
            $categorytab->position = $i;
            $categorytab->active = 1;
            $categorytab->banner_image = 'sample.jpg';
            $categorytab->banner_image_w = '211';
            $categorytab->banner_image_h = '215';
            $categorytab->scrollcategories = '3';
            $categorytab->add();
        }
        for ($i = 2; $i < 3; $i++) {
            $categorytab->position = $i;
            $categorytab->active = 1;
            $categorytab->banner_image = 'sample1.jpg';
            $categorytab->banner_image_w = '211';
            $categorytab->banner_image_h = '215';
            $categorytab->scrollcategories = '6';
            $categorytab->add();
        }
        for ($i = 3; $i < 4; $i++) {
            $categorytab->position = $i;
            $categorytab->active = 1;
            $categorytab->banner_image = 'sample2.jpg';
            $categorytab->banner_image_w = '211';
            $categorytab->banner_image_h = '215';
            $categorytab->scrollcategories = '9';
            $categorytab->add();
        }
        for ($i = 4; $i < 5; $i++) {
            $categorytab->position = $i;
            $categorytab->active = 1;
            $categorytab->banner_image = 'sample3.jpg';
            $categorytab->banner_image_w = '211';
            $categorytab->banner_image_h = '215';
            $categorytab->scrollcategories = '4';
            $categorytab->add();
        }
        for ($i = 5; $i < 6; $i++) {
            $categorytab->position = $i;
            $categorytab->active = 1;
            $categorytab->banner_image = 'sample4.jpg';
            $categorytab->banner_image_w = '211';
            $categorytab->banner_image_h = '215';
            $categorytab->scrollcategories = '5';
            $categorytab->add();
        }
        for ($i = 6; $i < 7; $i++) {
            $categorytab->position = $i;
            $categorytab->active = 1;
            $categorytab->banner_image = 'sample5.jpg';
            $categorytab->banner_image_w = '211';
            $categorytab->banner_image_h = '215';
            $categorytab->scrollcategories = '5';
            $categorytab->add();
        }
        
        foreach ($languages as $lang) {
            $values['shopcategory_title'][$lang['id_lang']] = 'Shop by Categories';
        }
        Configuration::updateValue('shopcategory_title', $values['shopcategory_title']);
        
        return true;
    }

    /**
     * @see Module::uninstall()
     */
    public function uninstall()
    {
        Configuration::deleteByName('shopcategory_title');
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();
            return (bool)$res;
        }

        return false;
    }

    /**
     * Creates tables
     */
    public function createTables()
    {
        /* Tab category Shop */
        $res = (bool)Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxshopbycategory_shop` (
                `id_dfxshopbycategory` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `id_shop` int(10) unsigned NOT NULL,
                PRIMARY KEY (`id_dfxshopbycategory`, `id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Tab category configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxshopbycategory` (
              `id_dfxshopbycategory` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `position` int(10) unsigned NOT NULL DEFAULT \'0\',
              `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `banner_image` varchar(128) DEFAULT NULL,
			  `banner_image_w` varchar(255) NOT NULL,
	  		  `banner_image_h` varchar(255) NOT NULL,
			  `scrollcategories` varchar(50) NOT NULL,
              PRIMARY KEY (`id_dfxshopbycategory`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        return $res;
    }

    /**
     * deletes tables
     */
    public function deleteTables()
    {
        $dfxcategory = $this->getDfxshopbycategory();
        foreach ($dfxcategory as $ttcat) {
            $to_del = new DfxShopbycategorygroup($ttcat['id_dfxshopbycategory']);
            $to_del->delete();
        }

        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxshopbycategory_shop`, `'._DB_PREFIX_.'dfxshopbycategory`;
        ');
    }
    public function hookDisplayHeader()
    {
        $this->context->controller->addCSS($this->_path.'views/css/front.css');
    }

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache($this->templateFile);
    }

    public function getContent()
    {
        $this->context->controller->addJS($this->_path.'views/js/adminback.js');
        $this->context->controller->addCSS($this->_path.'views/css/adminback.css');
        
        $this->_html .= $this->headerHTML();
        
        /* Validate & process */
        if (Tools::isSubmit('submitTitle')) {
            $this->_postProcess();
            $this->_html .= $this->renderTitleform().$this->renderList();
            $this->_clearCache('*');
        } elseif (Tools::isSubmit('submitDfxshopbycategory') || Tools::isSubmit('delete_id_dfxshopbycategory') ||
            Tools::isSubmit('changeStatus')
        ) {
            $this->_postProcess();
            $this->_html .= $this->renderTitleform().$this->renderList();
            
            $this->_clearCache('*');
        } elseif ((Tools::isSubmit('delete_tab')) && Tools::isSubmit('id_dfxshopbycategory') ||
        Tools::isSubmit('deleteBannerImage')) {
            $this->_postProcess();
            $this->_html .= $this->renderDfxshopbycategoryForm();

            $this->_clearCache('*');
        } elseif (Tools::isSubmit('addDfxshopbycategory') || (Tools::isSubmit('id_dfxshopbycategory') && $this->dfxshopbycategoryExists((int)Tools::getValue('id_dfxshopbycategory')))) {
            if (Tools::isSubmit('addDfxshopbycategory')) {
                $mode = 'add';
            } else {
                $mode = 'edit';
            }

            if ($mode == 'add') {
                if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                    $this->_html .= $this->renderDfxshopbycategoryForm();
                } else {
                    $this->_html .= $this->getShopContextError(null, $mode);
                }
            } else {
                $associated_shop_ids = DfxShopbycategorygroup::getAssociatedIdsShop((int)Tools::getValue('id_dfxshopbycategory'));
                $context_shop_id = (int)Shop::getContextShopID();

                if ($associated_shop_ids === false) {
                    $this->_html .= $this->getShopAssociationError((int)Tools::getValue('id_dfxshopbycategory'));
                } elseif (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL && in_array($context_shop_id, $associated_shop_ids)) {
                    if (count($associated_shop_ids) > 1) {
                        $this->_html = $this->getSharedTabcategoryWarning();
                    }
                    $this->_html .= $this->renderDfxshopbycategoryForm();
                } else {
                    $shops_name_list = array();
                    foreach ($associated_shop_ids as $shop_id) {
                        $associated_shop = new Shop((int)$shop_id);
                        $shops_name_list[] = $associated_shop->name;
                    }
                    $this->_html .= $this->getShopContextError($shops_name_list, $mode);
                }
            }
        } else {
            if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                $this->_html .= $this->renderTitleform().$this->renderList();
            }
            $this->_html .= $this->getWarningMultishopHtml().$this->getCurrentShopInfoMsg();
        }

        return $this->_html;
    }

    
    public function _postProcess()
    {
        $errors = array();
        $languages = Language::getLanguages(false);
        $result = array();
        /* Processes Slider */
        if (Tools::isSubmit('submitTitle')) {
            foreach ($languages as $language) {
                $result['shopcategory_title'][$language['id_lang']] = Tools::getValue('shopcategory_title_'.$language['id_lang']);
            }
            
            Configuration::updateValue('shopcategory_title', $result['shopcategory_title']);
        } elseif (Tools::isSubmit('submitDfxshopbycategory')) {
            /* Sets ID if needed */
            if (Tools::getValue('id_dfxshopbycategory')) {
                $ttcategory = new DfxShopbycategorygroup((int)Tools::getValue('id_dfxshopbycategory'));
                if (!Validate::isLoadedObject($ttcategory)) {
                    $this->_html .= $this->displayError($this->trans('Invalid Tab category ID', array(), 'Modules.DfxShopbycategory.Admin'));
                    return false;
                }
            } else {
                $ttcategory = new DfxShopbycategorygroup();
            }
            /* Sets position */
            
            $ttcategory->position = (int)Tools::getValue('position');
            /* Sets active */
            $ttcategory->active = (int)Tools::getValue('active_dfxshopbycategory');
            $ttcategory->scrollcategories = Tools::getValue('scrollcategories');
            /* Uploads image and sets slide */
            $type = Tools::strtolower(Tools::substr(strrchr($_FILES['banner_image']['name'], '.'), 1));
            $imagesize = @getimagesize($_FILES['banner_image']['tmp_name']);
                
            if (Tools::getValue('banner_image_w')) {
                $ttcategory->banner_image_w = Tools::getValue('banner_image_w');
            } else {
                $ttcategory->banner_image_w = $imagesize['0'];
            }
            if (Tools::getValue('banner_image_w')) {
                $ttcategory->banner_image_h = Tools::getValue('banner_image_h');
            } else {
                $ttcategory->banner_image_h = $imagesize['1'];
            }
                    
            if (isset($_FILES['banner_image']) &&
                    isset($_FILES['banner_image']['tmp_name']) &&
                    !empty($_FILES['banner_image']['tmp_name']) &&
                    !empty($imagesize) &&
                    in_array(Tools::strtolower(Tools::substr(strrchr($imagesize['mime'], '/'), 1)), array( 'jpg', 'gif', 'jpeg', 'png')) &&
                    in_array($type, array('jpg', 'gif', 'jpeg', 'png'))
                ) {
                $temp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
                $salt = sha1(microtime());
                if ($error = ImageManager::validateUpload($_FILES['banner_image'])) {
                    $errors[] = $error;
                } elseif (!$temp_name || !move_uploaded_file($_FILES['banner_image']['tmp_name'], $temp_name)) {
                    return false;
                } elseif (!ImageManager::resize($temp_name, dirname(__FILE__).'/views/img/'.$salt.'_'.$_FILES['banner_image']['name'], null, null, $type)) {
                    $errors[] = $this->displayError($this->trans('An error occurred during the image upload process.', array(), 'Admin.Notifications.Error'));
                }
                if (isset($temp_name)) {
                    @unlink($temp_name);
                }
                
                $ttcategory->banner_image = $salt.'_'.$_FILES['banner_image']['name'];
                $ttcategory->banner_image_w = $imagesize['0'];
                $ttcategory->banner_image_h = $imagesize['1'];
            } elseif (Tools::getValue('banner_image_old') != '') {
                $ttcategory->banner_image = Tools::getValue('banner_image_old');
                $ttcategory->banner_image_w = $imagesize['0'];
                $ttcategory->banner_image_h = $imagesize['1'];
            }
                
                    

            /* Processes if no errors  */
            if (!$errors) {
                /* Adds */
                if (!Tools::getValue('id_dfxshopbycategory')) {
                    if (!$ttcategory->add()) {
                        $errors[] = $this->displayError($this->trans('The Tab category could not be added.', array(), 'Modules.DfxShopbycategory.Admin'));
                    }
                } elseif (!$ttcategory->update()) {
                    $errors[] = $this->displayError($this->trans('The Tab category could not be updated.', array(), 'Modules.DfxShopbycategory.Admin'));
                }
                $this->_clearCache('*');
            }

            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=6&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('changeStatus') && Tools::isSubmit('id_dfxshopbycategory')) {
            $ttcategory = new DfxShopbycategorygroup((int)Tools::getValue('id_dfxshopbycategory'));
            if ($ttcategory->active == 0) {
                $ttcategory->active = 1;
            } else {
                $ttcategory->active = 0;
            }
            
            
            $res = $ttcategory->update();

            $this->_clearCache('*');
            $this->_html .= ($res ? $this->displayConfirmation($this->trans('Configuration updated', array(), 'Admin.Notifications.Success')) : $this->displayError($this->trans('The configuration could not be updated.', array(), 'Modules.DfxShopbycategory.Admin')));
            
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=4&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxshopbycategory='.$ttcategory->id);
        } elseif (Tools::isSubmit('delete_id_dfxshopbycategory')) {
            $ttcategory = new DfxShopbycategorygroup((int)Tools::getValue('delete_id_dfxshopbycategory'));
            $res = $ttcategory->delete();
            $this->_clearCache('*');
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=1&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        } elseif (Tools::isSubmit('deleteBannerImage')) {
            $id_dfxshopbycategory = (int)Tools::getValue('id_dfxshopbycategory');

            $ttcategory = new DfxShopbycategorygroup($id_dfxshopbycategory);
            if ($ttcategory->banner_image) {
                $image_path = $this->local_path.$this->banner_img_folder.$ttcategory->banner_image;

                if (file_exists($image_path)) {
                    unlink($image_path);
                }

                $ttcategory->banner_image = null;
                $ttcategory->banner_image_w = null;
                $ttcategory->banner_image_h = null;
                $ttcategory->update(false);
                $this->_clearCache('*');
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&conf=7&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&id_dfxshopbycategory='.$id_dfxshopbycategory);
            }
        }

        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } elseif (Tools::isSubmit('submitDfxshopbycategory') && Tools::getValue('id_dfxshopbycategory')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitDfxshopbycategory')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }
    
    public function renderTitleform()
    {
        $fields_form = array(
        'form' => array(
            'legend' => array(
                'title' => $this->trans('Category Title'),
            ),
            'input' => array(
               array(
                        'col' => 7,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Title', array(), 'Admin.Global'),
                        'name' => 'shopcategory_title',
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Add your Title', array(), 'Admin.Global')
                    )
            ),
            'submit' => array(
                     'title' => $this->getTranslator()->trans('Submit', array(), 'Admin.Actions'),
                     'name' => 'submitTitle',
                )
            ),
        );
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitTitle';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsTitleValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }
    
    public function getConfigFieldsTitleValues()
    {
        $fields = array();

        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
            $fields['shopcategory_title'][$lang['id_lang']] = Tools::getValue('shopcategory_title_'.$lang['id_lang'], Configuration::get('shopcategory_title', $lang['id_lang']));
        }

        return $fields;
    }
    
    public function clearCache()
    {
        $this->_clearCache($this->templateFile);
    }

    public function hookActionShopDataDuplication($params)
    {
        Db::getInstance()->execute(
            '
            INSERT IGNORE INTO '._DB_PREFIX_.'dfxshopbycategory_shop (id_dfxshopbycategory, id_shop)
            SELECT id_dfxshopbycategory, '.(int)$params['new_id_shop'].'
            FROM '._DB_PREFIX_.'dfxshopbycategory_shop
            WHERE id_shop = '.(int)$params['old_id_shop']
        );
        
        $this->_clearCache('*');
    }

    public function headerHTML()
    {
        if (Tools::getValue('controller') != 'AdminModules' && Tools::getValue('configure') != $this->name) {
            return;
        }

        $this->context->controller->addJqueryUI('ui.sortable');
        /* Style & js for fieldset 'Tabcategory configuration' */
        $html = '<script type="text/javascript">
            $(function() {
                var $myCategorytabs = $("#ttcategory");
                $myCategorytabs.sortable({
                    opacity: 0.6,
                    cursor: "move",
                    update: function() {
                        var order = $(this).sortable("serialize") + "&action=updateListPosition";
                        $.post("'.$this->context->shop->physical_uri.$this->context->shop->virtual_uri.'modules/'.$this->name.'/ajax_'.$this->name.'.php?secure_key='.$this->secure_key.'", order);
                        }
                    });
                $myCategorytabs.hover(function() {
                    $(this).css("cursor","move");
                    },
                    function() {
                    $(this).css("cursor","auto");
                });

            });
        </script>';

        return $html;
    }

    public function getDfxshopbycategory($active = null)
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;

        $ttcategory = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT hs.`id_dfxshopbycategory` as id_dfxshopbycategory, hss.`position`, hss.`active`, hss.`banner_image`, hss.`banner_image_w`, hss.`banner_image_h`, hss.`scrollcategories` FROM '._DB_PREFIX_.'dfxshopbycategory_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxshopbycategory hss ON (hs.id_dfxshopbycategory = hss.id_dfxshopbycategory)
            WHERE id_shop = '.(int)$id_shop.
            ($active ? ' AND hss.`active` = 1' : ' ').'
            ORDER BY hss.position ASC, hss.id_dfxshopbycategory ASC'
        );
        
        foreach ($ttcategory as &$shopcat_tab) {
            $shopcat_tab['image_url'] = $this->context->link->getMediaLink(_MODULE_DIR_.$this->name.'/views/img/'.$shopcat_tab['banner_image']);
        }
        
        return $ttcategory;
    }

    public function displayStatus($id_dfxshopbycategory, $active)
    {
        $title = ((int)$active == 0 ? $this->trans('Disabled', array(), 'Admin.Global') : $this->trans('Enabled', array(), 'Admin.Global'));
        $icon = ((int)$active == 0 ? 'icon-remove' : 'icon-check');
        $class = ((int)$active == 0 ? 'btn-danger' : 'btn-success');
        $html = '<a class="btn '.$class.'" href="'.AdminController::$currentIndex.
            '&configure='.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules').
                '&changeStatus&id_dfxshopbycategory='.(int)$id_dfxshopbycategory.'" title="'.$title.'"><i class="'.$icon.'"></i> '.$title.'</a>';

        return $html;
    }

    public function dfxshopbycategoryExists($id_dfxshopbycategory)
    {
        $req = 'SELECT hs.`id_dfxshopbycategory` as id_dfxshopbycategory
                FROM `'._DB_PREFIX_.'dfxshopbycategory_shop` hs
                WHERE hs.`id_dfxshopbycategory` = '.(int)$id_dfxshopbycategory;
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($req);

        return ($row);
    }

    
    public function renderDfxshopbycategoryForm()
    {
        $id_dfxshopbycategory = (int)Tools::getValue('id_dfxshopbycategory');
        $ttcategory = new DfxShopbycategorygroup($id_dfxshopbycategory);
        /* Image content */
        $image_url = false;
        $image_size = false;
        if ($id_dfxshopbycategory) {
            if ($ttcategory->banner_image) {
                $image_url = $this->_path . $this->banner_img_folder . $ttcategory->banner_image;
                $image_size = filesize($this->local_path . $this->banner_img_folder . $ttcategory->banner_image) / 1000;
            }
        }

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Category Block Information', array(), 'Modules.DfxShopbycategory.Admin'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id_dfxshopbycategory',
                    ),
                    array(
                        'type' => 'file',
                        'label' => $this->trans('Advertisement Image', array(), 'Modules.DfxShopbycategory.Admin'),
                        'name' => 'banner_image',
                        'desc' => $this->trans('Maximum image size: %s.', array(ini_get('upload_max_filesize')), 'Admin.Global'),
                        'display_image' => true,
                        'image' => $image_url ? '<img src="' . $image_url . '" alt="" class="img-thumbnail" style="max-width:410px;" />' : false,
                        'size' => $image_size,
                        'delete_url' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules') . '&deleteBannerImage&id_dfxshopbycategory=' . $id_dfxshopbycategory,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Width', array(), 'Modules.DfxShopbycategory.Admin'),
                        'name' => 'banner_image_w',
                        'suffix' => 'pixels',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->trans('Image width', array(), 'Modules.DfxShopbycategory.Admin'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Height', array(), 'Modules.DfxShopbycategory.Admin'),
                        'name' => 'banner_image_h',
                        'suffix' => 'pixels',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->trans('Image height', array(), 'Modules.DfxShopbycategory.Admin'),
                    ),
                    array(
                        'type' => 'select_link',
                        'label' => $this->trans('Select Category'),
                        'name' => 'scrollcategories'
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->trans('Enabled', array(), 'Admin.Global'),
                        'name' => 'active_dfxshopbycategory',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                ),
                'buttons' => array(
                    array(
                        'href' => $this->currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                        'title' => $this->trans('Back'),
                        'icon' => 'process-icon-back',
                    ),
                ),
            ),
        );
        
        if (Tools::isSubmit('id_dfxshopbycategory')) {
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_dfxshopbycategory');
            $cat_value = $ttcategory->scrollcategories;
        } else {
            if ($ttcategory->scrollcategories) {
                $cat_value = $ttcategory->scrollcategories;
            } else {
                $cat_value = 3;
            }
        }
        
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitDfxshopbycategory';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getAddFieldsValues(),
            'id_dfxshopbycategory' => $id_dfxshopbycategory,
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'cat_options' => $this->getAllCatLink(),
            'cat_value' => $cat_value
        );
         
        $helper->override_folder = '/';

    
        return $helper->generateForm(array($fields_form));
    }

    public function getAddFieldsValues()
    {
        $fields = array();
        $id_dfxshopbycategory = (int) Tools::getValue('id_dfxshopbycategory');
       
        $ttcategory = new DfxShopbycategorygroup($id_dfxshopbycategory);
       
        $fields['id_dfxshopbycategory'] = $id_dfxshopbycategory;


        $fields['active_dfxshopbycategory'] = Tools::getValue('active_dfxshopbycategory', $ttcategory->active);

    
        $languages = Language::getLanguages(false);
    
        $fields['banner_image'] = Tools::getValue('banner_image', $ttcategory->banner_image);
        $fields['banner_image_w'] = Tools::getValue('banner_image_w', $ttcategory->banner_image_w);
        $fields['banner_image_h'] = Tools::getValue('banner_image_h', $ttcategory->banner_image_h);
        $fields['scrollcategories'] = Tools::getValue('scrollcategories', $ttcategory->scrollcategories);
        return $fields;
    }
    
    public function getAllCatLink($id_lang = null, $link = false)
    {
        if (is_null($id_lang)) {
            $id_lang = (int)$this->context->language->id;
        }
        $html = '<optgroup label="'.$this->trans('Category').'">';
        $html .= $this->getCategoryOption(1, $id_lang, false, true, $link);
        $html .= '</optgroup>';
        return $html;
    }
    
    public function getCategoryOption($id_category = 1, $id_lang = false, $id_shop = false, $recursive = true, $link = false)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $id_shop = $id_shop ? (int)$id_shop : (int)Context::getContext()->shop->id;
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
        if (is_null($category->id)) {
            return;
        }
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
            $spacer = str_repeat('&nbsp;', 3 * (int)$category->level_depth);
        }
        if (!in_array($category->id, array(Configuration::get('PS_HOME_CATEGORY'), Configuration::get('PS_ROOT_CATEGORY')))) {
            if ($link) {
                $html .= '<option value="'.$this->context->link->getCategoryLink($category->id).'">'.(isset($spacer) ? $spacer : '').str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'</option>';
            } else {
                $html .= '<option value="'.(int)$category->id.'">'.str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'</option>';
            }
        } elseif ($category->id != Configuration::get('PS_ROOT_CATEGORY')) {
            $html .= '<optgroup label="'.str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'">';
        }
        if (isset($children) && count($children)) {
            foreach ($children as $child) {
                $html .= $this->getCategoryOption(
                    (int)$child['id_category'],
                    (int)$id_lang,
                    (int)$child['id_shop'],
                    $recursive,
                    $link
                );
            }
        }
        return $html;
    }
        
    public function getWarningMultishopHtml()
    {
        if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
            return '<p class="alert alert-warning">' .
            $this->trans('You cannot manage slides items from a "All Shops" or a "Group Shop" context, select directly the shop you want to edit', array(), 'Modules.DfxShopbycategory.Admin') .
            '</p>';
        } else {
            return '';
        }
    }

    public function getShopContextError($shop_contextualized_name, $mode)
    {
        if (is_array($shop_contextualized_name)) {
            $shop_contextualized_name = implode('<br/>', $shop_contextualized_name);
        }

        if ($mode == 'edit') {
            return '<p class="alert alert-danger">' .
            $this->trans('You can only edit this Category Tab from the shop(s) context: %s', array($shop_contextualized_name), 'Modules.DfxShopbycategory.Admin') .
            '</p>';
        } else {
            return '<p class="alert alert-danger">' .
            $this->trans('You cannot add Category Tab from a "All Shops" or a "Group Shop" context', array(), 'Modules.DfxShopbycategory.Admin') .
            '</p>';
        }
    }

    public function getShopAssociationError($id_dfxshopbycategory)
    {
        return '<p class="alert alert-danger">'.
                        $this->trans('Unable to get menu shop association information (id_dfxshopbycategory: %d)', array((int)$id_dfxshopbycategory), 'Modules.DfxShopbycategory.Admin') .
                '</p>';
    }


    protected function getCurrentShopInfoMsg()
    {
        $shop_info = null;

        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_SHOP) {
                $shop_info = $this->trans('The modifications will be applied to shop: %s', array($this->context->shop->name), 'Modules.DfxShopbycategory.Admin');
            } elseif (Shop::getContext() == Shop::CONTEXT_GROUP) {
                $shop_info = $this->trans('The modifications will be applied to this group: %s', array(Shop::getContextShopGroup()->name), 'Modules.DfxShopbycategory.Admin');
            } else {
                $shop_info = $this->trans('The modifications will be applied to all shops and shop groups', array(), 'Modules.DfxShopbycategory.Admin');
            }

            return '<div class="alert alert-info">'.
                        $shop_info.
                    '</div>';
        } else {
            return '';
        }
    }

    public function getSharedTabcategoryWarning()
    {
        return '<p class="alert alert-warning">'.
                    $this->trans('This slide is shared with other shops! All shops associated to this slide will apply modifications made here', array(), 'Modules.DfxShopbycategory.Admin').
                '</p>';
    }
    
    /* Front Office Features */
    public function renderList()
    {
        $dfxcategory = $this->getDfxshopbycategory();
        $id_lang = $this->context->language->id;

        foreach ($dfxcategory as $key => $ttcat) {
            $id_cat = $ttcat['scrollcategories'];
            $dfxcategory[$key]['status'] = $this->displayStatus($ttcat['id_dfxshopbycategory'], $ttcat['active']);
            $associated_shop_ids = DfxShopbycategorygroup::getAssociatedIdsShop((int)$ttcat['id_dfxshopbycategory']);
            if ($associated_shop_ids && count($associated_shop_ids) > 1) {
                $dfxcategory[$key]['is_shared'] = true;
            } else {
                $dfxcategory[$key]['is_shared'] = false;
            }
            $dfxcategory[$key]['cat_list'] = $this->getCategoryBigMenu($id_cat, (int)$id_lang);
        }
                                            
        $this->context->smarty->assign(
            array(
                'link' => $this->context->link,
                'dfxshopbycategory' => $dfxcategory,
                'banner_image_url' => $this->_path.$this->banner_img_folder,
            )
        );

        return $this->display(__FILE__, 'list.tpl');
    }

    public function renderWidget($hookName = null, array $configuration = array())
    {
	  if (!preg_match('/^displayHeader\d*$/', $hookName)) {
        if (!$this->isCached($this->templateFile, $this->getCacheId($this->name))) {
            $variables = $this->getWidgetVariables($hookName, $configuration);
            if (empty($variables)) {
                return false;
            }
            $this->smarty->assign($variables);
        }

        return $this->fetch($this->templateFile, $this->getCacheId($this->name));
	  }
    }

    public function getWidgetVariables($hookName = null, array $configuration = array())
    {
        $returnData = $this->getCatblocks();
        
        if (!empty($returnData)) {
            return $returnData;
        }
        return false;
    }
    
    public function getCatblocks()
    {
        $id_lang = (int)$this->context->language->id;
        $dfxcategory = $this->getDfxshopbycategory(true);
        foreach ($dfxcategory as $key => $ttcat) {
            $id_cat = $ttcat['scrollcategories'];
            $category = new Category($id_cat, (int)$id_lang);
            $dfxcategory[$key]['link'] = $this->context->link->getCategoryLink($id_cat, $category->link_rewrite);
            $dfxcategory[$key]['cat_list'] = $this->getCategoryBigMenu($id_cat, 1, (int)$id_lang);
        }

        return [
            'dfxshopbycategory' => $dfxcategory,
            'shopcategory_title' => Configuration::get('shopcategory_title', $this->context->language->id)
        ];
    }
    
    public function getCategoryBigMenu($id_category, $level, $id_lang = false, $id_shop = false, $recursive = true)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $category = new Category($id_category, (int)$id_lang, (int)$id_shop);

        if (is_null($category->id)) {
            return;
        }
		
        $category_link = $category->getLink();
           
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
        }
        
        if ($this->context->controller->controller_type != 'admin') {
            if ($level == 1) {
                $html .= '<a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="cattab_name">'.$category->name.'</a>';
                $html .= '<ul class="subcategory">';
            } else {
                $html .= '<li><a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="submenu">'.$category->name.'</a></li>';
            }
        } else {
            if ($level == 1) {
                $html .= '<a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="cattab_name" target="_blank">'.$category->name.'</a>';
                $html .= '<ul class="subcategory">';
            } else {
                $html .= '<li><a href="'.Tools::HtmlEntitiesUTF8($category_link).'" class="submenu" target="_blank">'.$category->name.'</a></li>';
            }
        }

        if (count($children)) {
            foreach ($children as $child) {
                $html .= $this->getCategoryBigMenu((int)$child['id_category'], 2, (int)$id_lang, (int)$child['id_shop']);
            }
        }

        if ($level == 1) {
            $html .= '</ul>';
        }
        return $html;
    }
    
    public function getCategoryByIds($id_cat, $id_lang)
    {
        $db = Db::getInstance(_PS_USE_SQL_SLAVE_);
        
        $sql_groups = '';
        if (Group::isFeatureActive()) {
            $groups = FrontController::getCurrentCustomerGroups();
            $sql_groups = 'AND cg.`id_group` '.(count($groups) ? 'IN ('.implode(',', $groups).')' : '= 1');
        }

        $results = $db->executeS('
		SELECT cp.`id_product`
		FROM `'._DB_PREFIX_.'category_product` cp
		'.(Group::isFeatureActive() ? 'INNER JOIN `'._DB_PREFIX_.'category_group` cg ON cp.`id_category` = cg.`id_category`' : '').'
		INNER JOIN `'._DB_PREFIX_.'category` c ON cp.`id_category` = c.`id_category`
		INNER JOIN `'._DB_PREFIX_.'category_lang` cl ON cl.`id_category` = c.`id_category`
		INNER JOIN `'._DB_PREFIX_.'product` p ON cp.`id_product` = p.`id_product`
		'.Shop::addSqlAssociation('product', 'p', false).'
		WHERE c.`active` = 1
		AND product_shop.`active` = 1
		AND product_shop.`active` = 1
		'.($id_cat ? 'AND c.`id_category` = '.$id_cat.'':'').'
		'.($id_lang ? 'AND cl.`id_lang` = '.$id_lang.'':'').'
		AND product_shop.`visibility` IN ("both", "search")
		AND product_shop.indexed = 1
		'.$sql_groups, true, false);
        
        return $results;
    }
}
