<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class DfxTabproductsClass extends ObjectModel
{
	public $id_tab;
    public $status;
	public $position;
	public $name;
    public $alias_name;
	public $tab_type;
	public $prod_count = 10;
	public $products;
	public $cat_id;
	public $id_shop;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'dfxtabproducts',
        'primary' => 'id_dfxtabproducts',
        'multilang' => true,
        'fields' => array(
			'status' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
            'position' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt', 'required' => true),
            // Lang fields
            
            'name' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'required' => true, 'size' => 255),
			'alias_name' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCleanHtml', 'size' => 255),
			'tab_type' => array('type' => self::TYPE_STRING),
            'prod_count' => array('type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255),
            'products' => array('type' => self::TYPE_STRING),
			'cat_id' => array('type' => self::TYPE_INT, 'validate' => 'isunsignedInt', 'required' => true),
        )
    );

    public function __construct($id_tab = null, $id_lang = null, $id_shop = null, Context $context = null)
    {
        parent::__construct($id_tab, $id_lang, $id_shop);
    }

    public function add($autodate = true, $null_values = false)
    {
        $context = Context::getContext();
        $id_shop = $context->shop->id;

        $res = parent::add($autodate, $null_values);
        $res &= Db::getInstance()->execute(
            '
			INSERT INTO `'._DB_PREFIX_.'dfxtabproducts_shop` (`id_shop`, `id_dfxtabproducts`)
			VALUES('.(int)$id_shop.', '.(int)$this->id_tab.')'
        );
        return $res;
    }

    public function delete()
    {
        $res = true;

        $res &= Db::getInstance()->execute(
            '
			DELETE FROM `'._DB_PREFIX_.'dfxtabproducts_shop`
			WHERE `id_dfxtabproducts` = '.(int)$this->id_tab 
        );

        $res &= parent::delete();
        return $res;
    }
    
    public function rePositions($id_dfxtabproducts)
    {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
			SELECT ms.`position`
			FROM '._DB_PREFIX_.'dfxtabproducts ms
			WHERE ms.`id_dfxtabproducts` = '.(int)$id_dfxtabproducts);
    }
	
	public function reOrderPositions()
	{
		$id_tab = $this->id;
		$context = Context::getContext();
		$id_shop = $context->shop->id;

		$max = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
			SELECT MAX(hss.`position`) as position
			FROM `'._DB_PREFIX_.'dfxtabproducts` hss, `'._DB_PREFIX_.'dfxtabproducts_shop` hs
			WHERE hss.`id_dfxtabproducts` = hs.`id_dfxtabproducts` AND hs.`id_shop` = '.(int)$id_shop
		);

		if ((int)$max == (int)$id_tab)
			return true;

		$rows = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
			SELECT hss.`position` as position, hss.`id_dfxtabproducts` as id_tab
			FROM `'._DB_PREFIX_.'dfxtabproducts` hss
			LEFT JOIN `'._DB_PREFIX_.'dfxtabproducts_shop` hs ON (hss.`id_dfxtabproducts` = hs.`id_dfxtabproducts`)
			WHERE hs.`id_shop` = '.(int)$id_shop.' AND hss.`position` > '.(int)$this->position
		);

		foreach ($rows as $row)
		{
			$current_tab = new DfxTabproductsClass($row['id_tab']);
			--$current_tab->position;
			$current_tab->update();
			unset($current_tab);
		}

		return true;
	}

    public static function getAssociatedIdsShop($id_dfxtabproducts)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
			SELECT hs.`id_shop`
			FROM `'._DB_PREFIX_.'dfxtabproducts_shop` hs
			WHERE hs.`id_dfxtabproducts` = '.(int)$id_dfxtabproducts
        );

        if (!is_array($result)) {
            return false;
        }

        $return = array();

        foreach ($result as $id_shop) {
            $return[] = (int)$id_shop['id_shop'];
        }

        return $return;
    }
}
