<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Adapter\NewProducts\NewProductsProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;
use PrestaShop\PrestaShop\Adapter\BestSales\BestSalesProductSearchProvider;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

include_once(_PS_MODULE_DIR_.'dfxtabproducts/classes/DfxTabproductsClass.php');

class DfxTabproducts extends Module implements WidgetInterface
{
	protected $_html = '';
    protected $link;
	public $confirmUninstall;
    private $templateFile;
    
    public function __construct()
    {
        $this->name = 'dfxtabproducts';
        $this->tab = 'front_office_features';
        $this->author = 'Digitalredfox';
        $this->version = '4.1.1';
        $this->need_instance = 0;

        $this->ps_versions_compliancy = array('min' => '1.7.2.0', 'max' => _PS_VERSION_);

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->trans('Tab Products On homepage', array(), 'Modules.DfxTabproducts.Admin');
        $this->description = $this->trans('Displays products on Tablist in Homepage.', array(), 'Modules.DfxTabproducts.Admin');
		
		$this->confirmUninstall = $this->trans('Are you sure you want to uninstall?', array(), 'Modules.DfxTabproducts.Admin');

        $this->templateFile = 'module:dfxtabproducts/views/templates/hook/dfxtabproducts.tpl';
    }

    public function install()
    {
		if (parent::install() &&
            $this->registerHook('actionProductAdd') &&
            $this->registerHook('actionProductUpdate') &&
			$this->registerHook('actionProductDelete') &&
			$this->registerHook('actionCategoryAdd') &&
			$this->registerHook('actionCategoryUpdate') &&
			$this->registerHook('actionCategoryDelete') &&
			$this->registerHook('updateOrderStatus') &&
			$this->registerHook('updateOrderStatus') &&
			$this->registerHook('displayHome') &&
			$this->registerHook('displayHeader') &&
            $this->registerHook('actionShopDataDuplication') &&
			$this->registerHook('actionObjectLanguageAddAfter')
        ) {
            /* Creates tables */
            $res = $this->createTables();

            /* Adds samples */
            if ($res) {
                $this->installSamples();
            }

            return (bool)$res;
        }

        return false;
    }
	
	protected function installSamples()
    {
        $res = 1;
        include_once dirname(__FILE__).'/sql/sample.php';
        return $res;
    }
	
    public function createTables()
    {
		 /* Tabproducts Shop */
        $res = (bool)Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxtabproducts_shop` (
                `id_dfxtabproducts` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `id_shop` int(10) unsigned NOT NULL,
                PRIMARY KEY (`id_dfxtabproducts`, `id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Tabproducts configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxtabproducts` (
              `id_dfxtabproducts` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `position` int(10) unsigned NOT NULL DEFAULT \'0\',
			  `status` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
			  `tab_type` varchar(50) NOT NULL,
			  `prod_count` varchar(255) DEFAULT \'10\',
			  `products` text DEFAULT NULL,
			  `cat_id` varchar(150) DEFAULT NULL,
              PRIMARY KEY (`id_dfxtabproducts`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');
        
        /* Tabproducts lang configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'dfxtabproducts_lang` (
              `id_dfxtabproducts` int(10) unsigned NOT NULL,
              `id_lang` int(10) unsigned NOT NULL,
              `name` varchar(255) NOT NULL,
			  `alias_name` varchar(255) NOT NULL,
              PRIMARY KEY (`id_dfxtabproducts`,`id_lang`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        return $res;
    }


	 public function uninstall()
    {
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();
            return (bool)$res;
        }

        return false;
    }
	
	 public function deleteTables()
    {
        $tabs = $this->getAllTabs();
        foreach ($tabs as $tab) {
            $to_del = new DfxTabproductsClass($tab['id_dfxtabproducts']);
            $to_del->delete();
        }

        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.'dfxtabproducts_shop`, `'._DB_PREFIX_.'dfxtabproducts`, `'._DB_PREFIX_.'dfxtabproducts_lang`;
        ');
    }
	
	public function hookDisplayHeader($params)
    {
        if ('index' === $this->context->controller->php_self) {
			$display_rtl = $this->context->language->is_rtl;
			if($display_rtl) {
			$this->context->controller->addCSS($this->_path.'views/css/front_rtl.css');
			} else {
            $this->context->controller->addCSS($this->_path.'views/css/front.css');
			}
        }
    }

    public function getContent()
    {
		$tabid = (int)Tools::getValue('tabid');
		
		if (isset($tabid) && $tabid != '') {
		  if (Tools::isSubmit('submitTab_'.$tabid)) 
		  {
			  if ($this->_postValidation($tabid)) {
				  $this->_postProcess();
			  }
			  $this->clearCache();
		  } elseif (Tools::isSubmit('id_tab') && $this->tabIdExists((int)Tools::getValue('id_tab'))) {
		    
			  $mode = 'edit';
			  $associated_shop_ids = DfxTabproductsClass::getAssociatedIdsShop((int)Tools::getValue('id_tab'));
              $context_shop_id = (int)Shop::getContextShopID();

                if ($associated_shop_ids === false) {
                    $this->_html .= $this->getShopAssociationError((int)Tools::getValue('id_tab'));
                } elseif (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL && in_array($context_shop_id, $associated_shop_ids)) {
                    if (count($associated_shop_ids) > 1) {
                        $this->_html = $this->getSharedTabWarning();
                    }
                } else {
                    $shops_name_list = array();
                    foreach ($associated_shop_ids as $shop_id) {
                        $associated_shop = new Shop((int)$shop_id);
                        $shops_name_list[] = $associated_shop->name;
                    }
                    $this->_html .= $this->getShopContextError($shops_name_list, $mode);
                }
		 }
		} elseif (Tools::isSubmit('submitAddCategory')) {
			$mode = 'add';
			if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
               $this->_postProcess();
            } else {
                $this->_html .= $this->getShopContextError(null, $mode);
            }
		} elseif (Tools::isSubmit('submitEnableSettings')) {
            $this->_postProcess();
        } else {
            $this->_html .= $this->getWarningMultishopHtml().$this->getCurrentShopInfoMsg();
        }

		return $this->_html.$this->renderNewTabForm().$this->renderList();
	}
	
	protected function _postValidation($tabid)
    {
        $errors = array();
	
		if (Tools::isSubmit('submitTab_'.$tabid)) {

            /* If edit : checks id_tab */
            if (Tools::isSubmit('id_tab')) {
                if (!Validate::isInt(Tools::getValue('id_tab')) && !$this->tabIdExists(Tools::getValue('id_tab'))) {
                    $errors[] = $this->getTranslator()->trans('Invalid tab ID', array(), 'Modules.DfxTabproducts.Admin');
                }
            }
			
			 $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
				if (Tools::strlen(Tools::getValue('aliasname_'.$tabid.'_'.$language['id_lang'])) > 255) {
                    $errors[] = $this->getTranslator()->trans('The title is too long.', array(), 'Modules.DfxTabproducts.Admin');
                }
            }
			if (Tools::strlen(Tools::getValue('prod_count_'.$tabid)) == '') {
				$errors[] = $this->getTranslator()->trans('The Product count is not set.', array(), 'Modules.DfxTabproducts.Admin');
			}
			$id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            if (Tools::strlen(Tools::getValue('aliasname_'.$tabid.'_'.$id_lang_default)) == '') {
                $errors[] = $this->getTranslator()->trans('The Tab name is not set.', array(), 'Modules.DfxTabproducts.Admin');
            }
			
		}
		
		/* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));

            return false;
        }

        /* Returns if validation is ok */

        return true;
		
	}
	
	public function _postProcess()
    {
		$languages = Language::getLanguages(false);
        $default_language = (int)Configuration::get('PS_LANG_DEFAULT');
        $id_shop = (int)Shop::getContextShopID();
		
		$errors = array();
        $shop_context = Shop::getContext();
		
		$tabid = (int)Tools::getValue('tabid');
		$sel_product = Tools::getValue('selProduct');
		
		if (Tools::isSubmit('submitTab_'.$tabid)) {
				$Tabproducts = new DfxTabproductsClass();
				if($tabid) {
				$Tabproducts = new DfxTabproductsClass($tabid);
				}
				$Tabproducts->tab_type = Tools::getValue('type_'.$tabid);
				$Tabproducts->status = (int)Tools::getValue('status_'.$tabid);
				
	            $cat_id = $Tabproducts->cat_id;
				$Tabproducts->cat_id = ($cat_id)?$cat_id:0;
				$prod_count = Tools::getValue('prod_count_'.$tabid);
				$Tabproducts->prod_count = $prod_count;

				foreach ($languages as $language) {
				$Tabproducts->alias_name[$language['id_lang']] = Tools::getValue('aliasname_'.$tabid.'_'.$language['id_lang']);
   			   }
	
				if ($Tabproducts->tab_type == 'manual') {
					$Tabproducts->products = $sel_product;
				} else {
					$Tabproducts->products = '';
				}
				
				 if (!$errors) {
                /* Adds */
					if (!$Tabproducts->update()) {
						$errors[] = $this->displayError($this->trans('The tab could not be updated.', array(), 'Modules.DfxTabproducts.Admin'));
					} else {
                        $this->_html .= $this->displayConfirmation($this->trans('Settings Updated.'));
					}
					$this->clearCache();
				}
		} 
		
		if (Tools::isSubmit('submitAddCategory')) {
            $selected_tab = Tools::getValue('rootCat');
            $val_arr = array();
            $val_arr = explode('_', $selected_tab);
			
			if($tabid) {
		     $Tabproducts = new DfxTabproductsClass($tabid);
			 if (!Validate::isLoadedObject($Tabproducts)) {
                $this->_html .= $this->displayError($this->trans('Invalid tab ID', array(), 'Modules.DfxTabproducts.Admin'));
                return false;
              }
		    } else {
                $Tabproducts = new DfxTabproductsClass();
				$Tabproducts->position = (int)$this->getNextPosition();
            }

            $Tabproducts->status = 1;
           
			$Tabproducts->tab_type = $val_arr[1];
			
			$flag_exists = 0;
            if ($val_arr[1] !== 'manual') {
                $flag_exists = $this->tabExist($val_arr[0]);
			}
            if ($flag_exists) {
                $errors[] = $val_arr[0].' '.$this->trans('tab is already added.');
            } else {
				foreach ($languages as $l) {
					if ($val_arr[1] == 'category') {
						$category = new Category($val_arr[2], (int)$l['id_lang']);
						$Tabproducts->name[$l['id_lang']] = $val_arr[0];
						$Tabproducts->alias_name[$l['id_lang']] = trim($category->name);
						$Tabproducts->cat_id = $val_arr[2] ? $val_arr[2] : 0;
					} elseif ($val_arr[1] == 'brands') {
						$manufacturer = new Manufacturer((int)$val_arr[2], (int)$l['id_lang']);
						$Tabproducts->name[$l['id_lang']] = trim($val_arr[0]);
						$Tabproducts->alias_name[$l['id_lang']] = trim($manufacturer->name);
						$Tabproducts->cat_id = $val_arr[2] ? $val_arr[2] : 0;
					} else {
						$Tabproducts->name[$l['id_lang']] = trim($val_arr[0]);
						$Tabproducts->alias_name[$l['id_lang']] = trim($val_arr[0]);
						$Tabproducts->cat_id = 0;
					}
				}
			}
           if (!$errors) {
					if (!Tools::getValue('tabid')) {
						if (!$Tabproducts->add()) {
							$errors[] = $this->displayError($this->trans('The tab could not be added.', array(), 'Modules.DfxTabproducts.Admin'));
						}
					} elseif (!$Tabproducts->update()) {
						$errors[] = $this->displayError($this->trans('The tab could not be updated.', array(), 'Modules.DfxTabproducts.Admin'));
					} else {
						$this->_html .= $this->displayConfirmation($this->trans('New section is added successfully.'));
					}
					$this->clearCache();
         	}

        }
       
         if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        }
	}

    public function renderNewTabForm()
    { 
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Category Configuration', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs',
                ),

                'description' => $this->trans('To add products to your homepage, simply add them to the corresponding product category (default: "Home").'),
                'input' => array(
                    array(
                        'type' => 'selectsection',
                        'label' => $this->trans('Add Category'),
                        'name' => 'rootCat',
                        'class' => 'fixed-width-xs',
                        'desc' => $this->trans('Add item to the list.'),
                    )
                ),
                'submit' => array(
                     'title' => $this->trans('Add Tab', array(), 'Admin.Actions'),
                     'name' => 'submitAddCategory',
                ),
            ),
        );

        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->id = (int) Tools::getValue('id_carrier');
		$this->fields_form = array();
		$helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAddCategory';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
			'cat_options' => $this->getAllCatLink(),
        );

        return $helper->generateForm(array($fields_form));
    }
	public function getAllCatLink($id_lang = null)
    {
        if (is_null($id_lang)) {
            $id_lang = (int)$this->context->language->id;
        }
        $html = '<optgroup label="'.$this->trans('Categories').'">';
        $html .= $this->getCategoryOption(1, $id_lang, false, true);
        $html .= '</optgroup>';
        $html .= '<optgroup label="' . $this->trans('Brand', [], 'Admin.Global') . '">';
        // Option to show all Manufacturers
        $manufacturers = Manufacturer::getManufacturers(false, $this->context->language->id);
        foreach ($manufacturers as $manufacturer) {
            if ($manufacturer['id_manufacturer']) {
                $html .= '<option value="'.$manufacturer['name'].'_brands_' . $manufacturer['id_manufacturer'] . '">' . $manufacturer['name'] . '</option>';
            }
        }
        $html .= '</optgroup>';
        return $html;
    }
    
    public function getCategoryOption($id_category = 1, $id_lang = false, $id_shop = false, $recursive = true)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $id_shop = $id_shop ? (int)$id_shop : (int)Context::getContext()->shop->id;
        $category = new Category((int)$id_category, (int)$id_lang, (int)$id_shop);
        if (is_null($category->id)) {
            return;
        }
        if ($recursive) {
            $children = Category::getChildren((int)$id_category, (int)$id_lang, true, (int)$id_shop);
            $spacer = str_repeat('&nbsp;', 3 * (int)$category->level_depth);
        }

        if (!in_array($category->id, array(Configuration::get('PS_HOME_CATEGORY'), Configuration::get('PS_ROOT_CATEGORY')))) {
            $html .= '<option value="'.$category->name.'_category_'.(int)$category->id.'">'.str_repeat('&nbsp;', 3 * (int)$category->level_depth).$category->name.'</option>';
        }
        
        if (isset($children) && count($children)) {
            foreach ($children as $child) {
                $html .= $this->getCategoryOption(
                    (int)$child['id_category'],
                    (int)$id_lang,
                    (int)$child['id_shop'],
                    $recursive
                );
            }
        }
        return $html;
    }

    public function renderList()
    {
        $this->context->controller->addCSS(($this->_path).'views/css/back.css', 'all');
        
        $this->context->controller->addJqueryUI('ui.sortable');

        $this->context->controller->addJS($this->_path.'views/js/back.js');
        
        $id_lang = (int)Context::getContext()->language->id;
        $id_shop = (int)Shop::getContextShopID();
        $languages = Language::getLanguages(false);
		$tabAlias = array();
		$res_prod_arr = array();
        $customPrds = array();
		
        $tabs = $this->getAllTabs();
		foreach ($tabs as $tab) {
			$Tabproducts = new DfxTabproductsClass($tab['id_tab']);
			foreach ($languages as $lang) {
            	$tabAlias[$tab['id_tab']][$lang['id_lang']] = $Tabproducts->alias_name[$lang['id_lang']];
				if ($tab['id_lang'] == $lang['id_lang']) {
					
					if ($tab['tab_type'] == "manual") {

                    $res_prod = $tab['products'];
						if ($res_prod) {
							$res_prod_arr = explode(',', $res_prod);
							$customPrds[$tab['id_tab']] = $res_prod_arr;
						}
                	}
				}
				
        	}
		}
		
        $manualPrds = array();
        $productObj = new Product();
        $res_all_prod = $productObj->getProducts($id_lang, 0, 0, 'id_product', 'ASC');
        $prodList = array();
        
		
        foreach ($res_all_prod as $prod) {
            $prodList[$prod['id_product']] = $prod['name'];
        }


        $this->context->smarty->assign(
            array(
                'tabs' => $tabs,
				'tabAlias' => $tabAlias,
                'languages' => $languages,
                'defaultFormLanguage' => $id_lang,
                'res_all_prod' => $res_all_prod,
                'manualPrds' => $manualPrds,
                'prodList' => $prodList,
                'customPrds' => $customPrds,
				'link' => $this->context->link,
                'ajaxPath' => _MODULE_DIR_.$this->name
            )
        );

        return $this->display(__FILE__, 'views/templates/admin/list.tpl');
    }
	
    public function getAllTabs($status = null)
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;
        $id_lang = $this->context->language->id;

        $tabs = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            'SELECT hs.id_dfxtabproducts as id_tab, hss.*, hssl.*
            FROM '._DB_PREFIX_.'dfxtabproducts_shop hs
            LEFT JOIN '._DB_PREFIX_.'dfxtabproducts hss ON (hs.id_dfxtabproducts = hss.id_dfxtabproducts)
            LEFT JOIN '._DB_PREFIX_.'dfxtabproducts_lang hssl ON (hss.id_dfxtabproducts = hssl.id_dfxtabproducts)
            WHERE id_shop = '.(int)$id_shop.'
            AND hssl.id_lang = '.(int)$id_lang.
            ($status ? ' AND hss.`status` = 1' : ' ').'
            ORDER BY hss.position'
        );
        return $tabs;
    }

	public function getNextPosition()
    {
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
            SELECT MAX(hss.`position`) AS `next_position`
            FROM `'._DB_PREFIX_.'dfxtabproducts` hss, `'._DB_PREFIX_.'dfxtabproducts_shop` hs
            WHERE hss.`id_dfxtabproducts` = hs.`id_dfxtabproducts` AND hs.`id_shop` = '.(int)$this->context->shop->id
        );

        return (++$row['next_position']);
    }
	
	public function tabExist($name) {
		$id_shop = (int)Shop::getContextShopID();
		$sql_sel = 'SELECT COUNT(*) FROM `'._DB_PREFIX_.'dfxtabproducts_lang` hsl
				LEFT JOIN '._DB_PREFIX_.'dfxtabproducts_shop hss ON (hsl.id_dfxtabproducts = hss.id_dfxtabproducts) 
				WHERE hss.`id_shop` = '.$id_shop.' AND hsl.`name` = "'.$name.'"';
		$flag_exists = Db::getInstance()->getValue($sql_sel);		
		return $flag_exists;
	}
	
	public function tabIdExist($id_tab) {
		$req = 'SELECT hs.`id_dfxtabproducts` as id_tab
                FROM `'._DB_PREFIX_.'dfxtabproducts_shop` hs
                WHERE hs.`id_dfxtabproducts` = '.(int)$id_tab;
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($req);

        return ($row);
	}
	
    public function getConfigFieldsValues()
    {
        $fields_value = array();

        $fields_value['rootCat'] = Tools::getValue('rootCat', Configuration::get('rootCat'));

        return $fields_value;
    }
	
	public function hookActionProductAdd($params)
    {
        $this->clearCache();
    }

    public function hookActionProductUpdate($params)
    {
        $this->clearCache();
    }

    public function hookActionProductDelete($params)
    {
        $this->clearCache();
    }

    public function hookActionCategoryAdd($params)
    {
        $this->clearCache();
    }

    public function hookActionCategoryUpdate($params)
    {
        $this->clearCache();
    }

    public function hookActionCategoryDelete($params)
    {
        $this->clearCache();
    }

    public function hookUpdateOrderStatus($params)
    {
        $this->clearCache();
    }

    public function clearCache()
    {
        parent::_clearCache($this->templateFile);
    }

    public function renderWidget($hookName = null, array $configuration = [])
    {
		if (!preg_match('/^displayHeader\d*$/', $hookName)) {
			if (!$this->isCached($this->templateFile, $this->getCacheId('Dfxtabproducts'))) {
				$variables = $this->getWidgetVariables($hookName, $configuration);
	
				if (empty($variables)) {
					return false;
				}
	
				$this->smarty->assign($variables);
			}

        return $this->fetch($this->templateFile, $this->getCacheId('Dfxtabproducts'));
		}
    }

    public function getWidgetVariables($hookName = null, array $configuration = [])
    {
        $products = $this->getTabProducts();

		if (!empty($products)) {
			return $products;
        }
        return false;
    }

    protected function getTabProducts()
    {
        $cat_id = array();
        $prod_count = array();
        $tab_type = array();
        
        $alias_name = array();
        $product_groups = array();
        $product_group_name = array();
        $allproductids = $product_list = $featuredIds = $newproductIds = $brandproductids = $bestsellersIds = $specialIds = $mproducts_ids = array();

        $lang = $this->context->language->id;


        $id_lang =(int) Context::getContext()->language->id;
        $id_shop = (int) Context::getContext()->shop->id;

           
            $resultant = $this->getAlltabs(true);
            foreach ($resultant as $results) {
                $cat_id = $results['cat_id'];
				$id_tab = $results['id_tab'];
				$prod_count = $results['prod_count'];
				$tab_type = $results['tab_type'];
				$alias_name = $results['alias_name'];
				
                if ($tab_type == 'category') {

                     $allproductids = $this->getCatproductIds($cat_id, $prod_count);

                     if (isset($allproductids) && $allproductids) {
                        $product_group_name[$id_tab] = $alias_name;
                        $product_groups[$id_tab] = $this->assignName($alias_name);
						$product_list[$id_tab] = $this->getProductByIds($this->assignIDs($allproductids));
                     }

                } elseif ($tab_type == 'brands') {
                    $brandproductids = $this->getManufactureIds($cat_id, $prod_count);
                
                    if (isset($brandproductids) && $brandproductids) {
                        $product_group_name[$id_tab] = $alias_name;
                        $product_groups[$id_tab] = $this->assignName($alias_name);
                        $product_list[$id_tab] = $this->getProductByIds($this->assignIDs($brandproductids));
                    }
                } elseif ($tab_type == 'Featured') {
                    $featuredIds = $this->getCatproductIds((int)Configuration::get('PS_HOME_CATEGORY'), $prod_count);
                    
                    if (isset($featuredIds) && $featuredIds) {
                        $product_group_name[$id_tab] = $alias_name;
                        $product_groups[$id_tab] = $this->assignName($alias_name);
                        $product_list[$id_tab] = $this->getProductByIds($this->assignIDs($featuredIds));
                    }
                } elseif ($tab_type == 'New') {
                    $newproductIds = Product::getNewProducts((int)$this->context->language->id, 0, $prod_count);
                                    
                    if (isset($newproductIds) && $newproductIds) {
                        $product_group_name[$id_tab] = $alias_name;
                        $product_groups[$id_tab] = $this->assignName($alias_name);
                        $product_list[$id_tab] = $this->getProductByIds($this->assignIDs($newproductIds));
                    }
                } elseif ($tab_type == 'Topsellers') {
                    $bestsellersIds = $this->getBestSellers($prod_count);
                        
                    if (isset($bestsellersIds) && $bestsellersIds) {
                        $product_group_name[$id_tab] = $alias_name;
                        $product_groups[$id_tab] = $this->assignName($alias_name);
                        $product_list[$id_tab] = $this->getProductByIds($this->assignIDs($bestsellersIds));
                    }
                } elseif ($tab_type == 'Specials') {
                    $specialIds = Product::getPricesDrop((int)Context::getContext()->language->id, 0, $prod_count);
                    
                    if (isset($specialIds) && $specialIds) {
                        $product_group_name[$id_tab] = $alias_name;
                        $product_groups[$id_tab] = $this->assignName($alias_name);
                        $product_list[$id_tab] = $this->getProductByIds($this->assignIDs($specialIds));
                    }
                } elseif ($tab_type == 'manual') {
					$resultProd = $results['products'];
                    if ($resultProd) {
                        $product_groups[$id_tab] = 'custom_'.str_replace(' ', '_', $id_tab);
                        $product_group_name[$id_tab] = $alias_name;
                        
                        
                        $mproducts_ids = explode(',', $resultProd);
                        
                        $sel_prod_cnt = count($mproducts_ids);
                        
                        if ($sel_prod_cnt > $prod_count) {
                            $mproducts_ids = array_slice($mproducts_ids, 0, $prod_count);
                        }
                        
                        $cat_prods = $this->getProductByIds($mproducts_ids);
						if (!empty($cat_prods)) {
                            $product_list[$id_tab] = $cat_prods;
                        }
                    }
                }
            }


       return [
                'product_groups'=> $product_groups,
                'product_list' => $product_list,
                'product_group_name'=> $product_group_name,
		];
    }

    private function assignIDs($productIds)
    {
        $assignIds = [];
        foreach ($productIds as $product) {
            if (isset($product['id_product'])) {
                $assignIds[] = $product['id_product'];
            }
        }
        return $assignIds;
    }
    private function assignName($alias)
    {
        $assignName = [];
        $string = [];
        
        $string = str_replace(' ', '-', Tools::strtolower($alias));
        $assignName = preg_replace('/[^A-Za-z0-9\-]/', '-', $string);
        /*$replaced_alias = preg_replace('/-+/', '-', $string);*/
                    
        return $assignName;
    }
        
    public function getCatproductIds($cat_id, $prod_count)
    {
        $db = Db::getInstance(_PS_USE_SQL_SLAVE_);
        
        return $db->executeS('
		SELECT cp.`id_product` as id_product
        FROM `' . _DB_PREFIX_ . 'category_product` cp
        WHERE cp.`id_category` = ' . (int) $cat_id . '
        LIMIT '.$prod_count);
    }
    
    public function getManufactureIds($brand_id, $prod_count)
    {
        $db = Db::getInstance(_PS_USE_SQL_SLAVE_);
        
        return $db->executeS('
		SELECT cp.`id_product` as id_product
        FROM `' . _DB_PREFIX_ . 'product` cp
        WHERE cp.`id_manufacturer` = ' . (int) $brand_id. '
        LIMIT '.$prod_count);
    }

   public function getBestSellers($prod_count = null)
    {
        if (Configuration::get('PS_CATALOG_MODE')) {
            return false;
        }

        $searchProvider = new BestSalesProductSearchProvider(
            $this->context->getTranslator()
        );

        $context = new ProductSearchContext($this->context);

        $query = new ProductSearchQuery();

        $query
            ->setResultsPerPage($prod_count)
            ->setPage(1)
        ;

        $query->setSortOrder(SortOrder::random());

        $result = $searchProvider->runQuery(
            $context,
            $query
        );

        $assembler = new ProductAssembler($this->context);

        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );

        $products_for_template = array();

        foreach ($result->getProducts() as $rawProduct) {
            $products_for_template[] = $presenter->present(
                $presentationSettings,
                $assembler->assembleProduct($rawProduct),
                $this->context->language
            );
        }

        return $products_for_template;
    }
    
    public function getProductByIds($productId)
    {
        if (!is_array($productId)) {
            return;
        }
        if (empty($productId)) {
            return;
        }

        $products = $this->getProductsInfoByIds($productId, $this->context->language->id);
        
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();

        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );


        if (is_array($products)) {
            foreach ($products as &$product) {
                $product = $presenter->present(
                    $presentationSettings,
                    Product::getProductProperties($this->context->language->id, $product, $this->context),
                    $this->context->language
                );
            }
            unset($product);
        }

        return $products;
    }

    public function getProductsInfoByIds($productId, $id_lang, $active = true)
    {
        $product_ids = join(',', $productId);

        $id_shop = (int) $this->context->shop->id;

        $sql = 'SELECT p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) as quantity, pl.`description`, pl.`description_short`, pl.`link_rewrite`,
					pl.`meta_description`, pl.`meta_keywords`, pl.`meta_title`, pl.`name`, pl.`available_now`, pl.`available_later`, pl.`delivery_in_stock`, pl.`delivery_out_stock`, ps.`quantity` AS sales, p.`additional_delivery_times`, image_shop.`id_image` id_image, il.`legend`, m.`name` as manufacturer_name, cl.`name` AS category_default, IFNULL(product_attribute_shop.id_product_attribute, 0) id_product_attribute,
					DATEDIFF(
						p.`date_add`,
						DATE_SUB(
							"'.date('Y-m-d').' 00:00:00",
							INTERVAL '.(Validate::isUnsignedInt(Configuration::get('PS_NB_DAYS_NEW_PRODUCT')) ? Configuration::get('PS_NB_DAYS_NEW_PRODUCT') : 20).' DAY
						)
					) > 0 AS new
				FROM  `'._DB_PREFIX_.'product` p 
				'.Shop::addSqlAssociation('product', 'p').'
				LEFT JOIN `'._DB_PREFIX_.'product_attribute_shop` product_attribute_shop
					ON (p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop='.(int)$id_shop.')
				LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (
					p.`id_product` = pl.`id_product`
					AND pl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('pl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'product_sale` ps ON (
					pl.`id_product` = ps.`id_product`
				)
				LEFT JOIN `'._DB_PREFIX_.'category_lang` cl ON (
					product_shop.`id_category_default` = cl.`id_category`
					AND cl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('cl').'
				)
				LEFT JOIN `'._DB_PREFIX_.'image_shop` image_shop
					ON (image_shop.`id_product` = p.`id_product` AND image_shop.cover=1 AND image_shop.id_shop='.(int)$id_shop.')
				LEFT JOIN `'._DB_PREFIX_.'image_lang` il ON (image_shop.`id_image` = il.`id_image` AND il.`id_lang` = '.(int)$id_lang.')
				LEFT JOIN `'._DB_PREFIX_.'manufacturer` m ON (p.`id_manufacturer`= m.`id_manufacturer`)
				'.Product::sqlStock('p', 0).'
				WHERE p.id_product IN ('.$product_ids.')'.
            ($active ? ' AND product_shop.`active` = 1 AND product_shop.`visibility` != \'none\'' : '').'
				ORDER BY FIELD(product_shop.id_product, '.$product_ids.')
				';
        if (!$result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql)) {
            return false;
        }
        foreach ($result as &$row) {
            $row['id_product_attribute'] = Product::getDefaultAttribute((int)$row['id_product']);
        }
        return Product::getProductsProperties($id_lang, $result);
    }
	
	  public function hookActionShopDataDuplication($params)
    {
        Db::getInstance()->execute('
            INSERT IGNORE INTO '._DB_PREFIX_.'dfxtabproducts_shop (id_dfxtabproducts, id_shop)
            SELECT id_dfxtabproducts, '.(int)$params['new_id_shop'].'
            FROM '._DB_PREFIX_.'dfxtabproducts_shop
            WHERE id_shop = '.(int)$params['old_id_shop']
        );
        $this->clearCache();
    }
	
	protected function getShopContextError($shop_contextualized_name, $mode)
    {
        if (is_array($shop_contextualized_name)) {
            $shop_contextualized_name = implode('<br/>', $shop_contextualized_name);
        }

        if ($mode == 'edit') {
            return '<p class="alert alert-danger">' .
            $this->trans('You can only edit this tab from the shop(s) context: %s', array($shop_contextualized_name), 'Modules.Dfxtabproducts.Admin') .
            '</p>';
        } else {
            return '<p class="alert alert-danger">' .
            $this->trans('You cannot add tabs from a "All Shops" or a "Group Shop" context', array(), 'Modules.Dfxtabproducts.Admin') .
            '</p>';
        }
    }
	
	protected function getShopAssociationError($id_tab)
    {
        return '<p class="alert alert-danger">'.
                        $this->trans('Unable to get carousal tab shop association information (id_tab: %d)', array((int)$id_tab), 'Modules.Dfxtabproducts.Admin') .
                '</p>';
    }
	
	protected function getSharedTabWarning()
    {
        return '<p class="alert alert-warning">'.
                    $this->trans('This tab is shared with other shops! All shops associated to this tab will apply modifications made here', array(), 'Modules.Dfxtabproducts.Admin').
                '</p>';
    }
	
	protected function getWarningMultishopHtml()
    {
        if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
            return '<p class="alert alert-warning">' .
            $this->getTranslator()->trans('You cannot manage tabs items from a "All Shops" or a "Group Shop" context, select directly the shop you want to edit', array(), 'Modules.Dfxtabproducts.Admin') .
            '</p>';
        } else {
            return '';
        }
    }
	
	protected function getCurrentShopInfoMsg()
    {
        $shop_info = null;

        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_SHOP) {
                $shop_info = $this->trans('The modifications will be applied to shop: %s', array($this->context->shop->name),'Modules.Dfxtabproducts.Admin');
            } else if (Shop::getContext() == Shop::CONTEXT_GROUP) {
                $shop_info = $this->trans('The modifications will be applied to this group: %s', array(Shop::getContextShopGroup()->name), 'Modules.Dfxtabproducts.Admin');
            } else {
                $shop_info = $this->trans('The modifications will be applied to all shops and shop groups', array(), 'Modules.Dfxtabproducts.Admin');
            }

            return '<div class="alert alert-info">'.
                        $shop_info.
                    '</div>';
        } else {
            return '';
        }
    }
	
	public function hookActionObjectLanguageAddAfter($params)
    {
        return $this->installSamples(array(array('id_lang' => (int)$params['object']->id)));
    }
}
