<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2021 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(dirname(__FILE__).'/WishList.php');

class DfxWishList extends Module
{
    const INSTALL_SQL_FILE = 'install.sql';
    private $html = '';
    public function __construct()
    {
        $this->name = 'dfxwishlist';
        $this->tab = 'front_office_features';
        $this->version = '1.3.2';
        $this->author = 'DigitalRedfox';
        $this->need_instance = 0;
        $this->controllers = array('mywishlist', 'view');
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->trans('Dfx Wishlist block', array(), 'Modules.DfxWishlist.Admin');
        $this->description = $this->trans('Adds a block containing the customers wishlists.', array(), 'Modules.DfxWishlist.Admin');
        $this->default_wishlist_name = $this->trans('My wishlist', array(), 'Modules.DfxWishlist.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        $this->module_key = '';

        $this->confirmUninstall = $this->trans('Warning: all the data saved in your database will be deleted.'.
            ' Are you sure you want uninstall this module?', array(), 'Modules.DfxWishlist.Admin');
        $this->html = '';
    }
    public function install($delete_params = true)
    {
        if ($delete_params) {
            if (!file_exists(dirname(__FILE__).'/'.self::INSTALL_SQL_FILE)) {
                return (false);
            } elseif (!$sql = Tools::file_get_contents(dirname(__FILE__).'/'.self::INSTALL_SQL_FILE)) {
                return (false);
            }
            $sql = str_replace(array('PREFIX_', 'ENGINE_TYPE'), array(_DB_PREFIX_, _MYSQL_ENGINE_), $sql);
            $sql = preg_split("/;\s*[\r\n]+/", $sql);
            foreach ($sql as $query) {
                if ($query) {
                    if (!Db::getInstance()->execute(trim($query))) {
                        return false;
                    }
                }
            }
        }
        if (!parent::install() ||
            !$this->registerHook('rightColumn') ||
            !$this->registerHook('productActions') ||
            !$this->registerHook('cart') ||
            !$this->registerHook('displayCustomerAccount') ||
            !$this->registerHook('displayHeader') ||
            !$this->registerHook('actionFrontControllerSetMedia') ||
            !$this->registerHook('adminCustomers') ||
            !$this->registerHook('displayTop') ||
            !$this->registerHook('displayProductwishlist') ||
            !$this->registerHook('displayWishlist')) {
            return false;
        }
        /* This hook is optional */
        $this->registerHook('displayMyAccountBlock');

        return true;
    }
    public function uninstall($delete_params = true)
    {
        if (($delete_params && !$this->deleteTables()) || !parent::uninstall()) {
            return false;
        }

        return true;
    }
    private function deleteTables()
    {
        return Db::getInstance()->execute(
            'DROP TABLE IF EXISTS
			`'._DB_PREFIX_.'dfxwishlist`,
			`'._DB_PREFIX_.'dfxwishlist_email`,
			`'._DB_PREFIX_.'dfxwishlist_product`,
			`'._DB_PREFIX_.'dfxwishlist_product_cart`'
        );
    }
    public function reset()
    {
        if (!$this->uninstall(false)) {
            return false;
        }
        if (!$this->install(false)) {
            return false;
        }

        return true;
    }
    public function getContent()
    {
        //$this->context->link->getProductLink($val['id_product']);

        if (Tools::isSubmit('viewdfxwishlist') && $id = Tools::getValue('id_product')) {
            Tools::redirect($this->context->link->getProductLink($id));
        } elseif (Tools::isSubmit('submitSettings')) {
            $activated = Tools::getValue('activated');
            if ($activated != 0 && $activated != 1) {
                $this->html .= '<div class="alert error alert-danger">'.$this->trans('Activate module : Invalid choice.', array(), 'Modules.DfxWishlist.Admin').'</div>';
            }
            $this->html .= '<div class="conf confirm alert alert-success">'.$this->trans('Settings updated', array(), 'Modules.DfxWishlist.Admin').'</div>';
        }

        $this->html .= $this->renderJS();
        $this->html .= $this->renderForm();
        if (Tools::getValue('id_customer') && Tools::getValue('id_dfxwishlist')) {
            $this->html .= $this->renderList((int)Tools::getValue('id_dfxwishlist'));
        }

        return $this->html;
    }
    public function hookDisplayProductwishlist($params)
    {
 		$cookie = $params['cookie'];

        $this->smarty->assign(array(
            'id_product' => (int)Tools::getValue('id_product'),
			'product' => $params['product']
        ));
		$id_dfxwishlist = (int)Db::getInstance()->getValue('
		SELECT w.id_dfxwishlist FROM '._DB_PREFIX_.'dfxwishlist w where w.id_customer='.(int)$cookie->id_customer.' and w.default = 1');

        if (isset($cookie->id_customer)) {
            $this->smarty->assign(array(
                'wishlists' => WishList::getByIdCustomer($cookie->id_customer),
				'id_dfxwishlist' => $id_dfxwishlist
            ));
        }

        
        return $this->fetch('module:dfxwishlist/views/templates/front/dfxwishlist_button.tpl');
    }
    public function hookDisplayTop($params)
    {
        $useSSL = ((isset($this->ssl) && $this->ssl && Configuration::get('PS_SSL_ENABLED')) || Tools::usingSecureMode()) ? true : false;
        $protocol_content = ($useSSL) ? 'https://' : 'http://';
        if ($this->context->customer->isLogged()) {
            $wishlists = WishList::getByIdCustomer($this->context->customer->id);
            if (empty($this->context->cookie->id_dfxwishlist) === true ||
                WishList::exists($this->context->cookie->id_dfxwishlist, $this->context->customer->id) === false) {
                if (!count($wishlists)) {
                    $id_dfxwishlist = false;
                } else {
                    $id_dfxwishlist = (int)$wishlists[0]['id_dfxwishlist'];
                    $this->context->cookie->id_dfxwishlist = (int)$id_dfxwishlist;
                }
            } else {
                $id_dfxwishlist = $this->context->cookie->id_dfxwishlist;
            }

            $this->smarty->assign(
                array(
                    'id_dfxwishlist' => $id_dfxwishlist,
                    'isLogged' => true,
                    'wishlist_products' => ($id_dfxwishlist == false ? false : WishList::getProductByIdCustomer(
                        $id_dfxwishlist,
                        $this->context->customer->id,
                        $this->context->language->id,
                        null,
                        true
                    )),
                    'wishlists' => $wishlists,
                    'ptoken' => Tools::getToken(false),
                )
            );
            $this->context->smarty->assign(
                array(
                'count_product' => (int)Db::getInstance()->getValue('SELECT SUM(wp.`quantity`) FROM '._DB_PREFIX_.'dfxwishlist w, '._DB_PREFIX_.'dfxwishlist_product wp where w.id_dfxwishlist = wp.id_dfxwishlist and w.id_customer='.(int)$this->context->customer->id))
            );
        } else
            $this->smarty->assign(array('wishlist_products' => false, 'wishlists' => false));
        $this->context->smarty->assign(
            array(
                'content_dir'=> $protocol_content.Tools::getHttpHost().__PS_BASE_URI__,
                'isLogged' => $this->context->customer->logged,
                'count_product' => (int)Db::getInstance()->getValue('SELECT SUM(wp.`quantity`) FROM '._DB_PREFIX_.'dfxwishlist w, '._DB_PREFIX_.'dfxwishlist_product wp where w.id_dfxwishlist = wp.id_dfxwishlist and w.id_customer='.(int)$this->context->customer->id))
        );

        return $this->fetch('module:dfxwishlist/views/templates/front/dfxwishlist_top.tpl');
    }
    public function hookDisplayHeader($params)
    {
		$id_lang = (int)Context::getContext()->language->id;
		$this->context->controller->addjqueryPlugin('fancybox');
        $this->context->controller->addJS(($this->_path).'views/js/ajax-wishlist.js');
		 $this->context->controller->addCSS($this->_path.'views/css/front.css');
        $this->smarty->assign(array('wishlist_link' => $this->context->link->getModuleLink('dfxwishlist', 'mywishlist')));
		$wishlistids = $wishids = [];
		if ($this->context->customer->isLogged()) {
		$wishproduct = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('SELECT id_product FROM '._DB_PREFIX_.'dfxwishlist w, '._DB_PREFIX_.'dfxwishlist_product wp where w.default = 1 and w.id_dfxwishlist = wp.id_dfxwishlist and w.id_customer='.$this->context->customer->id);
		
		
		
		if($wishproduct) {
			$productObj = new Product();
			$res_all_prod = $productObj->getProducts($id_lang, 0, 0, 'id_product', 'ASC');
			
			foreach($wishproduct as $wish) {
				$wishlistids[] = $wish['id_product'];
			}
			 foreach ($res_all_prod as $product) {
				if (in_array($product['id_product'], $wishlistids)) {
					$wishids[] = $product['id_product'];
				}
			 }
		
			
			}
			Media::addJsDef(
				array(
				'isLogged' =>  true,
				'wishids' => $wishids
				));
		} else {
			Media::addJsDef(
            array(
            'isLogged' =>  false,
			'wishids' => $wishids
            ));
		}
    }

    public function hookRightColumn($params)
    {
        if ($this->context->customer->isLogged()) {
            $wishlists = WishList::getByIdCustomer($this->context->customer->id);
            if (empty($this->context->cookie->id_dfxwishlist) === true ||
                WishList::exists($this->context->cookie->id_dfxwishlist, $this->context->customer->id) === false) {
                if (!count($wishlists)) {
                    $id_dfxwishlist = false;
                } else {
                    $id_dfxwishlist = (int)$wishlists[0]['id_dfxwishlist'];
                    $this->context->cookie->id_dfxwishlist = (int)$id_dfxwishlist;
                }
            } else {
                $id_dfxwishlist = $this->context->cookie->id_dfxwishlist;
            }
            $this->smarty->assign(
                array(
                    'id_dfxwishlist' => $id_dfxwishlist,
                    'isLogged' => true,
                    'wishlist_products' => ($id_dfxwishlist == false ? false : WishList::getProductByIdCustomer(
                        $id_dfxwishlist,
                        $this->context->customer->id,
                        $this->context->language->id,
                        null,
                        true
                    )),
                    'wishlists' => $wishlists,
                    'ptoken' => Tools::getToken(false)
                )
            );
        } else
            $this->smarty->assign(array('wishlist_products' => false, 'wishlists' => false));

        return $this->fetch('module:dfxwishlist/views/templates/front/dfxwishlist.tpl');
    }
    public function hookLeftColumn($params)
    {
        return $this->hookRightColumn($params);
    }
    public function hookDisplayWishlist($params)
    {
        $cookie = $params['cookie'];

        $this->smarty->assign(array(
            'id_product' => (int)Tools::getValue('id_product'),
        ));

        if (isset($cookie->id_customer)) {
            $this->smarty->assign(array(
                'wishlists' => WishList::getByIdCustomer($cookie->id_customer),
            ));
        }

        return $this->fetch('module:dfxwishlist/views/templates/front/dfxwishlist-extra.tpl');
    }
    public function hookDisplayCustomerAccount($params)
    {
        return $this->fetch('module:dfxwishlist/views/templates/front/my-account.tpl');
    }
    public function hookDisplayMyAccountBlock($params)
    {
        return $this->hookCustomerAccount($params);
    }
    private function _displayProducts($id_dfxwishlist)
    {
        include_once(dirname(__FILE__).'/WishList.php');

        $wishlist = new WishList($id_dfxwishlist);
        $products = WishList::getProductByIdCustomer($id_dfxwishlist, $wishlist->id_customer, $this->context->language->id);
        $nb_products = count($products);
        for ($i = 0; $i < $nb_products; ++$i) {
            $obj = new Product((int)$products[$i]['id_product'], false, $this->context->language->id);
            if (!Validate::isLoadedObject($obj)) {
                continue;
            } else {
                $images = $obj->getImages($this->context->language->id);
                foreach ($images as $image) {
                    if ($image['cover']) {
                        $products[$i]['cover'] = $obj->id.'-'.$image['id_image'];
                        break;
                    }
                }
                if (!isset($products[$i]['cover'])) {
                    $products[$i]['cover'] = $this->context->language->iso_code.'-default';
                }
            }
        }
        $this->html .= '
		<table class="table">
			<thead>
				<tr>
					<th class="first_item" style="width:600px;">'.$this->trans('Product', array(), 'Modules.DfxWishlist.Admin').'</th>
					<th class="item" style="text-align:center;width:150px;">'.$this->trans('Quantity', array(), 'Modules.DfxWishlist.Admin').'</th>
					<th class="item" style="text-align:center;width:150px;">'.$this->trans('Priority', array(), 'Modules.DfxWishlist.Admin').'</th>
				</tr>
			</thead>
			<tbody>';
        $priority = array($this->trans('High', array(), 'Modules.DfxWishlist.Admin'), $this->trans('Medium', array(), 'Modules.DfxWishlist.Admin'), $this->trans('Low', array(), 'Modules.DfxWishlist.Admin'));
        foreach ($products as $product) {
            $tmp = $this->context->link->getImageLink(
                $product['link_rewrite'],
                $product['cover'],
                ImageType::getFormatedName('small')
            );
            $this->html .= '
				<tr>
					<td class="first_item">
						<img src="'.$tmp.'" alt="'.htmlentities($product['name'], ENT_COMPAT, 'UTF-8').'" style="float:left;" />
						'.$product['name'];
            if (isset($product['attributes_small'])) {
                $this->html .= '<br /><i>'.htmlentities($product['attributes_small'], ENT_COMPAT, 'UTF-8').'</i>';
            }
            $this->html .= '
					</td>
					<td class="item" style="text-align:center;">'.(int)$product['quantity'].'</td>
					<td class="item" style="text-align:center;">'.$priority[(int)$product['priority'] % 3].'</td>
				</tr>';
        }
        $this->html .= '</tbody></table>';
    }
    public function hookAdminCustomers($params)
    {
        $customer = new Customer((int)$params['id_customer']);
        if (!Validate::isLoadedObject($customer)) {
            die(Tools::displayError());
        }
        $this->html = '<h2>'.$this->trans('Wishlists', array(), 'Modules.DfxWishlist.Admin').'</h2>';

        $wishlists = WishList::getByIdCustomer((int)$customer->id);
        if (!count($wishlists)) {
            $this->html .= $customer->lastname.' '.$customer->firstname.' '.$this->trans('No wishlist.', array(), 'Modules.DfxWishlist.Admin');
        } else {
            $this->html .= '<form action="'.Tools::safeOutput($_SERVER['REQUEST_URI']).'" method="post" id="listing">';

            $id_dfxwishlist = (int)Tools::getValue('id_dfxwishlist');
            if (!$id_dfxwishlist) {
                $id_dfxwishlist = $wishlists[0]['id_dfxwishlist'];
            }

            $this->html .= '<span>'.$this->trans('Wishlist', array(), 'Modules.DfxWishlist.Admin').': </span> <select name="id_dfxwishlist" onchange="$(\'#listing\').submit();">';

            if (is_array($wishlists)) {
                foreach ($wishlists as $wishlist) {
                    $this->html .= '<option value="'.(int)$wishlist['id_dfxwishlist'].'"';
                    if ($wishlist['id_dfxwishlist'] == $id_dfxwishlist) {
                        $this->html .= ' selected="selected"';
                        //$counter = $wishlist['counter'];
                    }
                    $this->html .= '>'.htmlentities($wishlist['name'], ENT_COMPAT, 'UTF-8').'</option>';
                }
            }
            $this->html .= '</select>';

            $this->_displayProducts((int)$id_dfxwishlist);

            $this->html .= '</form><br />';

            return $this->html;
        }
    }
    /*
    * Display Error from controler
    */
    public function errorLogged()
    {
        return $this->trans('You must be logged in to manage your wishlists.', array(), 'Modules.DfxWishlist.Admin');
    }
    public function renderJS()
    {
        return "<script>
			$(document).ready(function () { $('#id_customer, #id_dfxwishlist').change( function () "."{ $('#module_form').submit();}); });
		</script>";
    }
    public function renderForm()
    {
        $customers = array();
        foreach (WishList::getCustomers() as $c) {
            $customers[$c['id_customer']]['id_customer'] = $c['id_customer'];
            $customers[$c['id_customer']]['name'] = $c['firstname'].' '.$c['lastname'];
        }

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Listing', array(), 'Modules.DfxWishlist.Admin'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'select',
                        'label' => $this->trans('Customers :', array(), 'Modules.DfxWishlist.Admin'),
                        'name' => 'id_customer',
                        'options' => array(
                            'default' => array('value' => 0, 'label' => $this->trans('Choose customer', array(), 'Modules.DfxWishlist.Admin')),
                            'query' => $customers,
                            'id' => 'id_customer',
                            'name' => 'name'
                        ),
                    )
                ),
            ),
        );

        if ($id_customer = Tools::getValue('id_customer')) {
            $wishlists = WishList::getByIdCustomer($id_customer);
            $fields_form['form']['input'][] = array(
                'type' => 'select',
                'label' => $this->trans('Wishlist :', array(), 'Modules.DfxWishlist.Admin'),
                'name' => 'id_dfxwishlist',
                'options' => array(
                    'default' => array('value' => 0, 'label' => $this->trans('Choose wishlist', array(), 'Modules.DfxWishlist.Admin')),
                    'query' => $wishlists,
                    'id' => 'id_dfxwishlist',
                    'name' => 'name'
                ),
            );
        }

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $tmp = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
        $helper->allow_employee_form_lang = $tmp ? $tmp : 0;
        
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name
            .'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }
    public function getConfigFieldsValues()
    {
        return array(
            'id_customer' => Tools::getValue('id_customer'),
            'id_dfxwishlist' => Tools::getValue('id_dfxwishlist'),
        );
    }
    public function renderList($id_dfxwishlist)
    {
        $wishlist = new WishList($id_dfxwishlist);
        $products = WishList::getProductByIdCustomer($id_dfxwishlist, $wishlist->id_customer, $this->context->language->id);

        foreach ($products as $key => $val) {
            $image = Image::getCover($val['id_product']);
            $products[$key]['image'] = $this->context->link->getImageLink($val['link_rewrite'], $image['id_image'], ImageType::getFormatedName('small'));
        }

        $fields_list = array(
            'image' => array(
                'title' => $this->trans('Image', array(), 'Modules.DfxWishlist.Admin'),
                'type' => 'image',
            ),
            'name' => array(
                'title' => $this->trans('Product', array(), 'Modules.DfxWishlist.Admin'),
                'type' => 'text',
            ),
            'attributes_small' => array(
                'title' => $this->trans('Combination', array(), 'Modules.DfxWishlist.Admin'),
                'type' => 'text',
            ),
            'quantity' => array(
                'title' => $this->trans('Quantity', array(), 'Modules.DfxWishlist.Admin'),
                'type' => 'text',
            ),
            'priority' => array(
                'title' => $this->trans('Priority', array(), 'Modules.DfxWishlist.Admin'),
                'type' => 'priority',
                'values' => array($this->trans('High', array(), 'Modules.DfxWishlist.Admin'), $this->trans('Medium', array(), 'Modules.DfxWishlist.Admin'), $this->trans('Low', array(), 'Modules.DfxWishlist.Admin')),
            ),
        );

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = true;
        $helper->no_link = true;
        $helper->actions = array('view');
        $helper->show_toolbar = false;
        $helper->module = $this;
        $helper->identifier = 'id_product';
        $helper->title = $this->trans('Product list', array(), 'Modules.DfxWishlist.Admin');
        $helper->table = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->tpl_vars = array('priority' => array($this->trans('High', array(), 'Modules.DfxWishlist.Admin'), $this->trans('Medium', array(), 'Modules.DfxWishlist.Admin'), $this->trans('Low', array(), 'Modules.DfxWishlist.Admin')));

        return $helper->generateList($products, $fields_list);
    }
}
