/* global $, prestashop */

/**
 * This module exposes an extension point in the form of the `showModal` function.
 *
 * If you want to override the way the modal window is displayed, simply define:
 *
 * prestashop.blockcart = prestashop.blockcart || {};
 * prestashop.blockcart.showModal = function myOwnShowModal (modalHTML) {
 *   // your own code
 *   // please not that it is your responsibility to handle closing the modal too
 * };
 *
 * Attention: your "override" JS needs to be included **before** this file.
 * The safest way to do so is to place your "override" inside the theme's main JS file.
 *
 */

$(document).ready(function () {
  prestashop.blockcart = prestashop.blockcart || {};

 prestashop.on(
    'updateCart',
    function (event) {
      var refreshURL = $('.blockcart').data('refresh-url');
      var requestData = {};
      if (event && event.reason && typeof event.resp !== 'undefined' && !event.resp.hasError) {
        requestData = {
          id_customization: event.reason.idCustomization,
          id_product_attribute: event.reason.idProductAttribute,
          id_product: event.reason.idProduct,
          action: event.reason.linkAction
        };
      }
      if (event && event.resp && event.resp.hasError) {
        prestashop.emit('showErrorNextToAddtoCartButton', { errorMessage: event.resp.errors.join('<br/>')});
      }

      $.post(refreshURL, requestData).then(function (resp) {
		$('.blockcart .shopping_cart').replaceWith($(resp.preview).find('.blockcart .shopping_cart'));
        $('#blockcart-sideview .cart-wrap').replaceWith($(resp.preview).find('#blockcart-sideview .cart-wrap'));
		$('.blockcart').removeClass('inactive').addClass('active');
		$('.add-to-cart').removeClass('disabled');
      
        if ($('.cart_block_ajax').length) {
			 if(resp.modal) {
              $('html').addClass('dfx-cart-open');
              $('.cart-overlayer').fadeIn(500);
          } 
	   }
        var items = $('.add-to-cart');
		$('body').on('click', '.remove-from-cart', function(event){
			var theID  = $(this).data("id-product");
			items.filter(function() {
				return $(this).data('id-product') === theID;
			}).addClass('no-added disabled').removeClass('is-added');
           event.preventDefault();
		});

      }).fail(function (resp) {
        prestashop.emit('handleError', { eventType: 'updateShoppingCart', resp: resp });
      });
    }
  );


});
